﻿using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationDDCTNS;
using Ato.EN.IntegrationServices.CodeGenerationNRFI;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using Ato.EN.IntegrationServices.CodeGenerationRNTLPRPTY;
using Ato.EN.IntegrationServices.CodeGenerationRS;
using Ato.EN.IntegrationServices.CodeGenerationWRE;

namespace Ato.EN.IntegrationServices.CodeGenerationIITR
{
    public partial class IITR2019CrossformValidator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="IITR2019CrossformValidator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public IITR2019CrossformValidator(int maxParameterNameLength = 80, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }

        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Get string value between [first] a and [last] b.
        /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        /// <summary>
        /// Get string value after [first] a.
        /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

        /// <summary>
        /// Get string value after [last] a.
        /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
                return 0;
            else
                return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return ((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }
        //The logic in After function expects "---" for day and "--" for month. 
        //Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
            return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }


        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length < 13 || abn.Length < 11)
                {
                    response = false;
                }
                else
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 2), out numeric))
                        response = false;
                    else if (Regex.IsMatch(usi, @"^[a-zA-Z]{3}\d{4}[a-zA-Z]{2}"))
                        response = false;
                    else
                        response = true;
                }
            }
            return response;
        }


        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
            return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
            decimal numberValue;
            decimal.TryParse(value, out numberValue);
            return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':
                        options = options | RegexOptions.IgnoreCase;
                        break;
                    case 'm':
                        options = options | RegexOptions.Multiline;
                        break;
                    case 's':
                        options = options | RegexOptions.Singleline;
                        break;
                    case 'n':
                        options = options | RegexOptions.ExplicitCapture;
                        break;
                    case 'x':
                        options = options | RegexOptions.IgnorePatternWhitespace;
                        break;
                }
            }

            return options;
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public IITR2019Crossform ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string, ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(IITR2019Crossform report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            bool assertion;

            this.ConsumedReport = report;

            #region VR.ATO.IITR.310090

            /*  VR.ATO.IITR.310090
            Where the short form Income Tax Return Type is being used, no attached schedules are allowed, and no entries for Employment termination payment or Superannuation lump sum payment details are permitted.
    
            Legacy Rule Format:
            IF [iitr13] <> NULL AND (COUNT(SCHEDULE = "RS") > 0 OR COUNT(SCHEDULE = "CGTS") = 1 OR COUNT(SCHEDULE = "NRFI") = 1 OR COUNT(SCHEDULE = "PSS") = 1 OR COUNT(SCHEDULE = "DDCTNS") = 1 OR COUNT(SCHEDULE = "WRE") = 1 OR COUNT(SCHEDULE = "RNTLPRPTY") = 1 OR COUNT(RP.{ETPSeqNum}) > 0 OR COUNT(RP.{SLSPSeqNum}) > 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:
    
            IITR:RP:^IITR13 = tns:IncomeTax.ReturnType.Code
            */
            assertion = (report.iitrParentDocument.IITR13 != null && (Count(report.rsChildDocuments) > 0 || Count(report.cgtsChildDocuments) == 1 || Count(report.nrfiChildDocuments) == 1 || Count(report.pssChildDocuments) == 1 || Count(report.ddctnsChildDocuments) == 1 || Count(report.wreChildDocuments) == 1 || Count(report.rntlprptyChildDocuments) == 1 || Count(report.iitrParentDocument.RPETPSeqNumContextCollection) > 0 || Count(report.iitrParentDocument.RPSLSPSeqNumContextCollection) > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310090",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attached schedules or ETP/SLSP details are invalid",
                    LongDescription = @"When the 'Income Tax Return Type' is provided, implying that the return is to be treated as a short Individual return, no attached schedules are allowed, and no entries for Employment termination payment or Superannuation lump sum payment details are permitted",
                    Location = "/xbrli:xbrl/tns:IncomeTax.ReturnType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310090" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR13", Value = report.iitrParentDocument.IITR13 });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310090

            #region VR.ATO.IITR.730309

            /* VR.ATO.IITR.730309
            Rental property schedules (RS) or a Multi-Property Rental schedule (RNTLPRPTY) may be provided but not both

            Legacy Rule Format:
            IF (COUNT(SCHEDULE = "RS") > 0) AND (COUNT(SCHEDULE = "RNTLPRPTY") = 1)
               RETURN VALIDATION MESSAGE
            ENDIF

            */
            assertion = Count(report.rsChildDocuments) > 0 && Count(report.rntlprptyChildDocuments) == 1;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.730309",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental property schedules (RS) or a Multi-Property Rental schedule (RNTLPRPTY) may be provided but not both",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730309" } },
                };

                response.Add(processMessage);
            }
            #endregion

            // ####################################################################################################
            //                                         CGTS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.310066

            /* VR.ATO.IITR.310066
            When net capital gain is in excess of $10,000 a Capital Gains Tax schedule must be attached.

            Legacy Rule Format:
            IF ([iitr311] > 10000) AND (COUNT(SCHEDULE = "CGTS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR311 = tns:Capital.Gains.Net.Amount
            */
            assertion = (report.iitrParentDocument.IITR311.GetValueOrDefault() > 10000 && (report.cgtsChildDocuments == null || Count(report.cgtsChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310066",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule required",
                    LongDescription = @"When Net capital gain is in excess of $10,000, a Capital Gains Tax schedule must be attached",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.Net.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR311", Value = report.iitrParentDocument.IITR311.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310066

            #region VR.ATO.IITR.310067

            /* VR.ATO.IITR.310067
            When current year capital gains is in excess of $10,000 a Capital Gains Tax schedule must be attached.

            Legacy Rule Format:
            IF ([iitr312] > 10000) AND (COUNT(SCHEDULE = "CGTS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:
            IITR:RP:^IITR312 = tns:Capital.Gains.Total.Amount
            */
            assertion = (report.iitrParentDocument.IITR312.GetValueOrDefault() > 10000 && (report.cgtsChildDocuments == null || Count(report.cgtsChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310067",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule required",
                    LongDescription = @"When Total current year capital gains is in excess of $10,000, a Capital Gains Tax schedule must be attached",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310067" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR312", Value = report.iitrParentDocument.IITR312.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310067

            #region VR.ATO.IITR.310069

            /* VR.ATO.IITR.310069
            The number of Capital Gains Tax schedules must not exceed 1.

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "CGTS") > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.cgtsChildDocuments != null && Count(report.cgtsChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Capital Gains Tax schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310069" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310069

            #region CGTS Child Present

            if (report.cgtsChildDocuments != null && Count(report.cgtsChildDocuments) == 1)
            {
                CGTS2018 cgtsChildDocument = report.cgtsChildDocuments[0];

                #region VR.ATO.GEN.402009

                /* VR.ATO.GEN.402009
                Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.402009",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                        LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.402009

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedExists == true && cgtsChildDocument.RPAppliedStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0Exists == true && cgtsChildDocument.RPAppliedY0StartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0Id + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0MinusExists == true && cgtsChildDocument.RPAppliedY0MinusStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0MinusId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedCompanySharesExists == true && cgtsChildDocument.RPListedCompanySharesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedCompanySharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherSharesExists == true && cgtsChildDocument.RPOtherSharesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherSharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedUnitTrustsExists == true && cgtsChildDocument.RPListedUnitTrustsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedUnitTrustsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherUnitsExists == true && cgtsChildDocument.RPOtherUnitsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherUnitsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAustralianRealPropertyExists == true && cgtsChildDocument.RPAustralianRealPropertyStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAustralianRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOverseasRealPropertyExists == true && cgtsChildDocument.RPOverseasRealPropertyStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOverseasRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCapitalGainsTrustDistributionsExists == true && cgtsChildDocument.RPCapitalGainsTrustDistributionsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCapitalGainsTrustDistributionsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCollectablesExists == true && cgtsChildDocument.RPCollectablesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCollectablesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherCapitalAssetsExists == true && cgtsChildDocument.RPOtherCapitalAssetsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherCapitalAssetsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPTransfersToOtherClassesExists == true && cgtsChildDocument.RPTransfersToOtherClassesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPTransfersToOtherClassesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedExists == true && cgtsChildDocument.RPAppliedEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0Exists == true && cgtsChildDocument.RPAppliedY0EndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0Id + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0MinusExists == true && cgtsChildDocument.RPAppliedY0MinusEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0MinusId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedCompanySharesExists == true && cgtsChildDocument.RPListedCompanySharesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedCompanySharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherSharesExists == true && cgtsChildDocument.RPOtherSharesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherSharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedUnitTrustsExists == true && cgtsChildDocument.RPListedUnitTrustsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedUnitTrustsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherUnitsExists == true && cgtsChildDocument.RPOtherUnitsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherUnitsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAustralianRealPropertyExists == true && cgtsChildDocument.RPAustralianRealPropertyEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAustralianRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOverseasRealPropertyExists == true && cgtsChildDocument.RPOverseasRealPropertyEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOverseasRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCapitalGainsTrustDistributionsExists == true && cgtsChildDocument.RPCapitalGainsTrustDistributionsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCapitalGainsTrustDistributionsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCollectablesExists == true && cgtsChildDocument.RPCollectablesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCollectablesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherCapitalAssetsExists == true && cgtsChildDocument.RPOtherCapitalAssetsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherCapitalAssetsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCollectablesUnappliedInstantInstantDate != null && cgtsChildDocument.RPCollectablesUnappliedInstantInstantDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCollectablesUnappliedInstantId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCGTOtherUnappliedInstantInstantDate != null && cgtsChildDocument.RPCGTOtherUnappliedInstantInstantDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCGTOtherUnappliedInstantId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPTransfersToOtherClassesExists == true && cgtsChildDocument.RPTransfersToOtherClassesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPTransfersToOtherClassesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438022

                /* VR.ATO.GEN.438022
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS127 = tns:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("A - Small business active asset reduction") : false) && cgtsChildDocument.CGTS127 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438022",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business active asset reduction amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS127", Value = cgtsChildDocument.CGTS127.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "A - Small business active asset reduction" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438022

                #region VR.ATO.GEN.438023

                /* VR.ATO.GEN.438023
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "B - Small business retirement exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS128] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS128 = tns:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("B - Small business retirement exemption") : false) && cgtsChildDocument.CGTS128 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438023",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business retirement exemption amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438023" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS128", Value = cgtsChildDocument.CGTS128.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "B - Small business retirement exemption" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438023

                #region VR.ATO.GEN.438024

                /* VR.ATO.GEN.438024
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS129 = tns:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("C - Small business roll-over") : false) && cgtsChildDocument.CGTS129 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438024",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business roll-over amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS129", Value = cgtsChildDocument.CGTS129.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "C - Small business roll-over" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438024

                #region VR.ATO.GEN.438025

                /* VR.ATO.GEN.438025
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS138 = tns:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("D - Small business 15 year exemption") : false) && cgtsChildDocument.CGTS138 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438025",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business 15 year exemption amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS138", Value = cgtsChildDocument.CGTS138.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "D - Small business 15 year exemption" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438025

                #region VR.ATO.GEN.438026

                /* VR.ATO.GEN.438026
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "E - Foreign resident CGT exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS140] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS140 = tns:Capital.Gains.ForeignResidentExemption.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("E - Foreign resident CGT exemption") : false) && cgtsChildDocument.CGTS140 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438026",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438026" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS140", Value = cgtsChildDocument.CGTS140.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "E - Foreign resident CGT exemption" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438026

                #region VR.ATO.GEN.438027

                /* VR.ATO.GEN.438027
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS45  = tns:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("F - Scrip for scrip roll-over") : false) && cgtsChildDocument.CGTS45 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438027",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS45", Value = cgtsChildDocument.CGTS45.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "F - Scrip for scrip roll-over" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438027

                #region VR.ATO.CGTS.402165

                /* VR.ATO.CGTS.402165
                Total losses transferred in applied is only valid for a company

                Legacy Rule Format:
                IF (PARENT RETURN <> "CTR") AND ([CGTS122] <> NULL)  
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.Applied:^CGTS122 = tns:Capital.Losses.TransferredTotal.Amount
                */
                assertion = (cgtsChildDocument.CGTS122 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402165",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Total losses transferred in applied is only valid for a company",
                        Location = "/xbrli:xbrl/tns:Capital.Losses.TransferredTotal.Amount[@contextRef='" + cgtsChildDocument.RPAppliedId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402165" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS122", Value = cgtsChildDocument.CGTS122.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402165

                #region VR.ATO.CGTS.402226

                /* VR.ATO.CGTS.402226
                If the Amendment indicator on a parent return is set to TRUE, then the Income year earnout right created and Amount to be amended cannot be supplied.

                Legacy Rule Format:
                IF (PARENT RETURN:RP:pyin.xx.xx:Report.Amendment.Indicator = TRUE) AND ([CGTS150] <> NULL AND [CGTS151] <> NULL)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.{EarnoutArrangementGroupNumber}:^CGTS150 = tns:Report.TargetFinancial.Year
                CGTS:RP.{EarnoutArrangementGroupNumber}:^CGTS151 = tns:Miscellaneous.RequestedAmendment.Amount
                IITR:RP:^IITR16                                  = tns:Report.Amendment.Indicator
                */
                assertion = (report.iitrParentDocument.IITR16 == true && cgtsChildDocument.CGTS150 != null && cgtsChildDocument.CGTS151 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402226",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Income year earnout right created and Amount to be amended cannot be supplied when Parent return Amendment indicator is present.",
                        Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year[@contextRef='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402226" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS150", Value = cgtsChildDocument.CGTS150.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS151", Value = cgtsChildDocument.CGTS151.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR16", Value = report.iitrParentDocument.IITR16.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402226

                #region VR.ATO.CGTS.402230

                /* VR.ATO.CGTS.402230
                A multi-class treatment can only be reported with a Trust Tax Return for Attribution Managed Investments (TRTAMI) when a multi-class election has been made

                Legacy Rule Format:
                IF PARENT RETURN  <> "TRTAMI" AND ([CGTS153] <> NULL OR [CGTS154] <> NULL OR [CGTS155] <> NULL)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS153                         = tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description
                CGTS:RP.TransfersToOtherClasses:^CGTS154 = tns:Capital.Gains.Total.Amount
                CGTS:RP.TransfersToOtherClasses:^CGTS155 = tns:Capital.Losses.Total.Amount
                */
                assertion = (cgtsChildDocument.CGTS153 != null || cgtsChildDocument.CGTS154 != null || cgtsChildDocument.CGTS155 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402230",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Multi-class treatment fields incorrectly supplied",
                        Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description[@contextRef='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402230" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS153", Value = cgtsChildDocument.CGTS153 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS154", Value = cgtsChildDocument.CGTS154.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS155", Value = cgtsChildDocument.CGTS155.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402230

                #region VR.ATO.CGTS.402240

                /* VR.ATO.CGTS.402240
                The year entered at Income year earnout right created cannot be the current year.

                Legacy Rule Format:
                IF ([CGTS150] <> NULL) AND ([CGTS150] = (PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.{EarnoutArrangementGroupNumber}:^CGTS150 = tns:Report.TargetFinancial.Year
                IITR:RP:^IITR10                                  = tns:Report.TargetFinancial.Year
                */
                assertion = (cgtsChildDocument.CGTS150 != null && cgtsChildDocument.CGTS150 == report.iitrParentDocument.IITR10);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402240",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Income year earnout right created incorrect",
                        LongDescription = @"The year entered at Income year earnout right created cannot be the current year.",
                        Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year[@contextRef='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402240" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS150", Value = cgtsChildDocument.CGTS150.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR10", Value = report.iitrParentDocument.IITR10.ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402240

                #region VR.ATO.IITR.310063

                /* VR.ATO.IITR.310063
                For an attached Capital Gains Tax schedule, if Net capital gain is greater than zero, there cannot be an amount at Other net capital losses carried forward to later income years.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([cgts125] > 0) AND ([cgts131] > 0)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.CGTOther.Unapplied(Instant):^CGTS125 = tns:Capital.Losses.CarriedForward.Net.Amount
                CGTS:RP:^CGTS131                             = tns:Income.CapitalGainsNet.Amount
                */
                assertion = (cgtsChildDocument.CGTS125.GetValueOrDefault() > 0 && cgtsChildDocument.CGTS131.GetValueOrDefault() > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310063",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other net capital losses carried forward to later income years on CGTS schedule is invalid",
                        LongDescription = @"For an attached Capital Gains Tax (CGTS) schedule, when a 'Net capital gain' amount above zero is provided, 'Other net capital losses carried forward to later income years' must be zero or not provided",
                        Location = "/xbrli:xbrl/xbrli:context",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310063" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS125", Value = cgtsChildDocument.CGTS125.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS131", Value = cgtsChildDocument.CGTS131.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310063

                #region VR.ATO.IITR.310068

                /* VR.ATO.IITR.310068
                The total current year capital gains on the Capital Gains Tax schedule must be equal to the total current year capital gains on the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([iitr312] <> [cgts118])
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS118 = tns:Capital.Gains.Total.Amount
                IITR:RP:^IITR312 = tns:Capital.Gains.Total.Amount
                */
                assertion = (report.iitrParentDocument.IITR312.GetValueOrDefault() != cgtsChildDocument.CGTS118.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310068",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Total current year capital gains amount invalid",
                        LongDescription = @"The Total current year capital gains on the Capital Gains Tax schedule must be equal to the Total current year capital gains on the Individual return",
                        Location = "/xbrli:xbrl/tns:Capital.Gains.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310068" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS118", Value = cgtsChildDocument.CGTS118.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR312", Value = report.iitrParentDocument.IITR312.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310068

                #region VR.ATO.IITR.310070

                /* VR.ATO.IITR.310070
                The sum of unapplied net capital losses carried forward on the Capital Gains Tax schedule must equal net capital losses carried forward on the main income tax return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([iitr313] <> CGTS:RP.Collectables.Unapplied:rvctc3.xx.xx:Capital.Losses.CarriedForward.Net.Amount + CGTS:RP.CGTOther.Unapplied:rvctc3.xx.xx:Capital.Losses.CarriedForward.Net.Amount)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.Collectables.Unapplied:^CGTS124 = tns:Capital.Losses.CarriedForward.Net.Amount
                CGTS:RP.CGTOther.Unapplied:^CGTS125     = tns:Capital.Losses.CarriedForward.Net.Amount
                IITR:RP(Instant):^IITR313               = tns:Capital.Losses.CarriedForward.Net.Amount
                */
                assertion = (report.iitrParentDocument.IITR313.GetValueOrDefault() != (cgtsChildDocument.CGTS124.GetValueOrDefault() + cgtsChildDocument.CGTS125.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310070",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Net capital losses carried forward incorrect",
                        LongDescription = @"The sum of unapplied net capital losses carried forward on the Capital Gains Tax schedule must equal Net capital losses carried forward to later income years on the main income tax return",
                        Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount[@contextRef='" + report.iitrParentDocument.RPInstantId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310070" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS124", Value = cgtsChildDocument.CGTS124.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS125", Value = cgtsChildDocument.CGTS125.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR313", Value = report.iitrParentDocument.IITR313.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310070

                #region VR.ATO.IITR.310071

                /* VR.ATO.IITR.310071
                The net capital gain amount on the main return form must equal the net capital gain amount on the Capital Gains Tax schedule.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([iitr311] <> CGTS:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS131 = tns:Income.CapitalGainsNet.Amount
                IITR:RP:^IITR311 = tns:Capital.Gains.Net.Amount
                */
                assertion = (report.iitrParentDocument.IITR311.GetValueOrDefault() != cgtsChildDocument.CGTS131.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310071",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Net capital gain amount in return incorrect",
                        LongDescription = @"The Net capital gain amount on the main return form must equal the Net capital gain amount on the Capital Gains Tax schedule",
                        Location = "/xbrli:xbrl/tns:Capital.Gains.Net.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310071" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS131", Value = cgtsChildDocument.CGTS131.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR311", Value = report.iitrParentDocument.IITR311.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310071

                #region VR.ATO.IITR.310085

                /* VR.ATO.IITR.310085
                The Total CGT discount applied is greater than 50% of the Total current year capital gains less Total capital losses applied.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([CGTS126] > ((([CGTS118] - [CGTS123]) *0.5) +1))
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS118         = tns:Capital.Gains.Total.Amount
                CGTS:RP.Applied:^CGTS123 = tns:Capital.Losses.Total.Amount
                CGTS:RP.Applied:^CGTS126 = tns:TaxConcession.CapitalGains.DiscountTotal.Amount
                */
                assertion = (cgtsChildDocument.CGTS126.GetValueOrDefault() > (((cgtsChildDocument.CGTS118.GetValueOrDefault() - cgtsChildDocument.CGTS123.GetValueOrDefault()) * (decimal)0.5) + 1));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310085",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Total CGT discount applied amount incorrect",
                        LongDescription = @"The Total CGT discount applied amount must not exceed 50% of the combined amount of Total current year capital gains, less Total capital losses applied",
                        Location = "/xbrli:xbrl/xbrli:context",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310085" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS118", Value = cgtsChildDocument.CGTS118.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS123", Value = cgtsChildDocument.CGTS123.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS126", Value = cgtsChildDocument.CGTS126.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310085

                #region Repeating RPEarnoutArrangementGroupNumberContext

                if (cgtsChildDocument.RPEarnoutArrangementGroupNumberContextCollection != null)
                {
                    foreach (CGTS2018.RPEarnoutArrangementGroupNumberContext cgtsEarnoutArrangementGroupNumberContext in cgtsChildDocument.RPEarnoutArrangementGroupNumberContextCollection)
                    {
                        #region VR.ATO.GEN.438000

                        /* VR.ATO.GEN.438000
                        Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                        Legacy Rule Format:
                        IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                            RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (cgtsEarnoutArrangementGroupNumberContext.StartDate != report.iitrParentDocument.RPStartDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.GEN.438000",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"The context period start date is incorrect.",
                                LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsEarnoutArrangementGroupNumberContext.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.GEN.438000

                        #region VR.ATO.GEN.438001

                        /* VR.ATO.GEN.438001
                        Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                        Legacy Rule Format:
                        IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                            RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (cgtsEarnoutArrangementGroupNumberContext.EndDate != report.iitrParentDocument.RPEndDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.GEN.438001",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"The context period end date is incorrect.",
                                LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsEarnoutArrangementGroupNumberContext.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.GEN.438001
                    }
                }

                #endregion // Repeating RPEarnoutArrangementGroupNumberContext
            }

            #endregion // CGTS Child Present

            // ####################################################################################################
            //                                        DDCTNS CROSSFORM RULES
            // ####################################################################################################

            // Count the number of DDCTNS schedules in the report: should be 0 or 1
            int ddctnsScheduleCount = Count(report.ddctnsChildDocuments);

            #region Sum and Value calculation amounts

            decimal sumDDCTNS138 = 0;
            decimal SumDDCTNS138_D = 0;
            decimal SumDDCTNS138_U = 0;
            decimal sumDDCTNS407 = 0;
            decimal valDDCTNS417 = 0;
            decimal valDDCTNS310 = 0;
            decimal valDDCTNS311 = 0;

            if (ddctnsScheduleCount == 1)
            {
                // Calculate the sum of the super contribution deduction amounts
                sumDDCTNS407 = Count(report.ddctnsChildDocuments[0].RPSCSeqNumContextCollection) > 0 ? Math.Truncate(report.ddctnsChildDocuments[0].RPSCSeqNumContextCollection.Sum(f => f.DDCTNS407.GetValueOrDefault())) : 0;

                // Calculate the sum of the other deductions amounts
                sumDDCTNS138 = Count(report.ddctnsChildDocuments[0].RPOthDedSeqNumContextCollection) > 0 ? Math.Truncate(report.ddctnsChildDocuments[0].RPOthDedSeqNumContextCollection.Sum(f => f.DDCTNS138.GetValueOrDefault())) : 0;

                // Determine the value of DDCTNS417 (0 if it doesn't exist)
                valDDCTNS417 = Math.Truncate(report.ddctnsChildDocuments[0].DDCTNS417.GetValueOrDefault());

                // Determine the value of DDCTNS310 (0 if it doesn't exist)
                valDDCTNS310 = Math.Truncate(report.ddctnsChildDocuments[0].DDCTNS310.GetValueOrDefault());

                // Determine the value of DDCTNS311 (0 if it doesn't exist)
                valDDCTNS311 = Math.Truncate(report.ddctnsChildDocuments[0].DDCTNS311.GetValueOrDefault());

                // Sum all of the values of DDCTNS138 where DDCTNS136 are equal to "D - Deductions relating to financial investments"
                SumDDCTNS138_D = Count(report.ddctnsChildDocuments[0].RPOthDedSeqNumContextCollection) > 0 ? Math.Truncate(report.ddctnsChildDocuments[0].RPOthDedSeqNumContextCollection.Sum(context => context.DDCTNS136 == "D - Deductions relating to financial investments" ? context.DDCTNS138.GetValueOrDefault() : 0)) : 0;

                // Sum all of the values of DDCTNS138 where DDCTNS136 are equal to "U - Debt deductions incurred in deriving foreign rental income"
                SumDDCTNS138_U = Count(report.ddctnsChildDocuments[0].RPOthDedSeqNumContextCollection) > 0 ? Math.Truncate(report.ddctnsChildDocuments[0].RPOthDedSeqNumContextCollection.Sum(context => context.DDCTNS136 == "U - Debt deductions incurred in deriving foreign rental income" ? context.DDCTNS138.GetValueOrDefault() : 0)) : 0;
            }

            #endregion Sum and Value calculation amounts

            #region VR.ATO.IITR.300093

            /*  VR.ATO.IITR.300093
            If the total of financial investment income minus financial investment deductions is less than zero then the difference must be inserted at Net financial investment loss. If the difference exceeds zero then zero must be inserted at Net financial investment loss.
    
            Legacy Rule Format:
            IF ([iitr13] = NULL)
            AND (((([iitr118] + [iitr119] + [iitr120] + [iitr249] + [iitr253] + [iitr257] + [iitr282] + [iitr291] + [iitr295] + [iitr325] + [iitr328]+ [iitr341] + SUM([iitr345]) + SUM([iitr351])) < (ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns310], 0), 0) + [iitr147] + [iitr261] + [iitr262] + [iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM(ConditionalValue([ddctns136] = "D - Deductions relating to financial investments", [ddctns138], 0)), 0), 0)))
                  AND (([iitr204] = NULL)
                     OR ([iitr204] <> ((ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns310], 0), 0) + [iitr147] + [iitr261] + [iitr262] +[iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM(ConditionalValue([ddctns136] = "D - Deductions relating to financial investments", [ddctns138], 0)), 0), 0)) - ([iitr118] + [iitr119] + [iitr120] + [iitr249] + [iitr253] + [iitr257] + [iitr282] + [iitr291] + [iitr295] + [iitr325] + [iitr328]+ [iitr341] + SUM([iitr345]) + SUM([iitr351]))))))
               OR ((([iitr118] + [iitr119] + [iitr120] + [iitr249]  + [iitr253] + [iitr257] + [iitr282] + [iitr291] + [iitr295] + [iitr325] + [iitr328]+ [iitr341] + SUM([iitr345]) + SUM([iitr351])) >= (ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns310], 0), 0) + [iitr147] + [iitr261] + [iitr262] + [iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM(ConditionalValue([ddctns136] = "D - Deductions relating to financial investments", [ddctns138], 0)), 0), 0)))
                   AND ([iitr204] = NULL OR [iitr204] <> 0)))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            */

            // Total financial investment income
            decimal cal300093_1 = report.iitrParentDocument.IITR118.GetValueOrDefault() + report.iitrParentDocument.IITR119.GetValueOrDefault() + report.iitrParentDocument.IITR120.GetValueOrDefault() + report.iitrParentDocument.IITR249.GetValueOrDefault() + report.iitrParentDocument.IITR253.GetValueOrDefault() + report.iitrParentDocument.IITR257.GetValueOrDefault() + report.iitrParentDocument.IITR282.GetValueOrDefault() + report.iitrParentDocument.IITR291.GetValueOrDefault() + report.iitrParentDocument.IITR295.GetValueOrDefault() + report.iitrParentDocument.IITR325.GetValueOrDefault() + report.iitrParentDocument.IITR328.GetValueOrDefault() + report.iitrParentDocument.IITR341.GetValueOrDefault() + (report.iitrParentDocument.RPIncSeqNumContextCollection == null ? 0 : report.iitrParentDocument.RPIncSeqNumContextCollection.Sum(f => f.IITR345.GetValueOrDefault())) + (report.iitrParentDocument.RPIncSeqNumContextCollection == null ? 0 : report.iitrParentDocument.RPIncSeqNumContextCollection.Sum(f => f.IITR351.GetValueOrDefault()));

            // Total financial investment deductions
            decimal cal300093_2 = valDDCTNS310 + report.iitrParentDocument.IITR147.GetValueOrDefault() + report.iitrParentDocument.IITR261.GetValueOrDefault() + report.iitrParentDocument.IITR262.GetValueOrDefault() + report.iitrParentDocument.IITR368.GetValueOrDefault() + SumDDCTNS138_D;

            assertion = (report.iitrParentDocument.IITR13 == null
                && ((cal300093_1 < cal300093_2
                    && (report.iitrParentDocument.IITR204 == null || report.iitrParentDocument.IITR204.GetValueOrDefault() != cal300093_2 - cal300093_1))
                || (cal300093_1 >= cal300093_2
                    && (report.iitrParentDocument.IITR204 == null || report.iitrParentDocument.IITR204 != 0))));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300093",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net Financial Investment Loss is incorrect",
                    LongDescription = @"If the total of financial investment income minus financial investment deductions is less than zero then the difference must be inserted at Net financial investment loss. If the difference exceeds zero then zero must be inserted at Net financial investment loss.",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.FinancialInvestmentNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300093" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR13", Value = report.iitrParentDocument.IITR13 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR118", Value = report.iitrParentDocument.IITR118.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR119", Value = report.iitrParentDocument.IITR119.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR120", Value = report.iitrParentDocument.IITR120.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR249", Value = report.iitrParentDocument.IITR249.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR253", Value = report.iitrParentDocument.IITR253.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR257", Value = report.iitrParentDocument.IITR257.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR282", Value = report.iitrParentDocument.IITR282.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR291", Value = report.iitrParentDocument.IITR291.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR295", Value = report.iitrParentDocument.IITR295.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR325", Value = report.iitrParentDocument.IITR325.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR328", Value = report.iitrParentDocument.IITR328.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR341", Value = report.iitrParentDocument.IITR341.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR345", Value = "IITR345" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR351", Value = "IITR351" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS310", Value = valDDCTNS310.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR147", Value = report.iitrParentDocument.IITR147.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR261", Value = report.iitrParentDocument.IITR261.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR262", Value = report.iitrParentDocument.IITR262.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR368", Value = report.iitrParentDocument.IITR368.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300093

            #region VR.ATO.IITR.300094

            /*  VR.ATO.IITR.300094
            If the total of Net rental property income less Net rental property deductions is less than zero then the difference must be inserted at Net Rental property loss. If the difference exceeds zero then zero must be inserted at Net Rental property loss.
    
            Legacy Rule Format:
            IF ((([iitr250] + [iitr283] + [iitr292] + [iitr296] + [iitr324] + ([iitr334] - ([iitr335] + [iitr336] + [iitr337])) + SUM([iitr346])) < (ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns311], 0), 0) + [iitr263] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM(ConditionalValue([ddctns136] = "U - Debt deductions incurred in deriving foreign rental income", [ddctns138], 0)), 0), 0)))
               AND ((([iitr205] = NULL)
                  OR ([iitr205] <> ((ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns311], 0), 0) + [iitr263] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM(ConditionalValue([ddctns136] = "U - Debt deductions incurred in deriving foreign rental income", [ddctns138], 0)), 0), 0)) - ([iitr250] + [iitr283] + [iitr292] + [iitr296] + [iitr324] + ([iitr334] - ([iitr335] + [iitr336] + [iitr337])) + SUM([iitr346])))))))
            OR ((([iitr250] + [iitr283] + [iitr292] + [iitr296] + [iitr324] + ([iitr334] - ([iitr335] + [iitr336] + [iitr337])) + SUM([iitr346])) >= (ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns311], 0), 0) + [iitr263] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM(ConditionalValue([ddctns136] = "U - Debt deductions incurred in deriving foreign rental income", [ddctns138], 0)), 0), 0))) 
               AND ([iitr205] <> 0))
                    RETURN VALIDATION MESSAGE
            ENDIF */

            // Total net rental property income
            decimal cal300094_1 = report.iitrParentDocument.IITR250.GetValueOrDefault() + report.iitrParentDocument.IITR283.GetValueOrDefault() + report.iitrParentDocument.IITR292.GetValueOrDefault() + report.iitrParentDocument.IITR296.GetValueOrDefault() + report.iitrParentDocument.IITR324.GetValueOrDefault() + (report.iitrParentDocument.IITR334.GetValueOrDefault() - (report.iitrParentDocument.IITR335.GetValueOrDefault() + report.iitrParentDocument.IITR336.GetValueOrDefault() + report.iitrParentDocument.IITR337.GetValueOrDefault())) + (report.iitrParentDocument.RPIncSeqNumContextCollection == null ? 0 : report.iitrParentDocument.RPIncSeqNumContextCollection.Sum(f => f.IITR346.GetValueOrDefault()));

            // Total net rental property deductions
            decimal cal300094_2 = valDDCTNS311 + report.iitrParentDocument.IITR263.GetValueOrDefault() + SumDDCTNS138_U;

            assertion = ((cal300094_1 < cal300094_2
                && (report.iitrParentDocument.IITR205 == null
                || report.iitrParentDocument.IITR205.GetValueOrDefault() != cal300094_2 - cal300094_1))
                || (cal300094_1 >= cal300094_2
                && report.iitrParentDocument.IITR205.GetValueOrDefault() != 0));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300094",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net Rental Property Loss is incorrect",
                    LongDescription = @"If the total of Net rental property income less Net rental property deductions is less than zero then the difference must be inserted at Net Rental property loss. If the difference exceeds zero then zero must be inserted at Net Rental property loss.",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.RentalPropertyNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300094" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR250", Value = report.iitrParentDocument.IITR250.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR283", Value = report.iitrParentDocument.IITR283.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR292", Value = report.iitrParentDocument.IITR292.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR296", Value = report.iitrParentDocument.IITR296.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR324", Value = report.iitrParentDocument.IITR324.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR336", Value = report.iitrParentDocument.IITR336.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR346", Value = "IITR346" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR143", Value = valDDCTNS311.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR263", Value = report.iitrParentDocument.IITR263.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR373", Value = "IITR373" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300094

            #region VR.ATO.IITR.730105

            /*  VR.ATO.IITR.730105
            The number of Deductions schedules must not exceed 1

            Legacy Rule Format:
            IF COUNT(SCHEDULE="DDCTNS") > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:
            N/A

            */
            assertion = ddctnsScheduleCount > 1;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.730105",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Deductions schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730105" } },
                };

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.730105

            #region VR.ATO.IITR.300013

            /*  VR.ATO.IITR.300013
            Where no income and deduction fields are provided, check if this is a nil return.

            Legacy Rule Format:
            IF (ALL OCCURRENCES OF([iitr62]) = NULL) AND ([iitr65] = NULL) AND (ALL OCCURRENCES OF([iitr68]) = NULL) AND (ALL OCCURRENCES OF([iitr71]) = NULL) AND ([iitr86] = NULL) AND ([iitr89] = NULL) AND ([iitr92] = NULL) AND ([iitr93] = NULL) AND ([iitr505] = NULL) AND ([iitr94] = NULL) AND ([iitr95] = NULL) AND ([iitr115] = NULL) AND ([iitr118] = NULL) AND ([iitr119] = NULL) AND ([iitr120] = NULL) AND ([iitr134] = NULL) AND ([iitr136] = NULL) AND ([iitr137] = NULL) AND ([iitr139] = NULL) AND ([iitr141] = NULL) AND ([iitr148] = NULL) AND ([iitr502] = NULL) AND ([iitr503] = NULL) AND ([iitr504] = NULL) AND ([iitr154] = NULL) AND ([iitr156] = NULL) AND ([iitr242] = NULL) AND ([iitr243] = NULL) AND ([iitr245] = NULL) AND ([iitr246] = NULL) AND ([iitr252] = NULL) AND ([iitr255] = NULL) AND ([iitr259] = NULL) AND ([iitr260] = NULL) AND ([iitr265] = NULL) AND ([iitr281] = NULL) AND ([iitr285] = NULL) AND ([iitr306] = NULL) AND ([iitr311] = NULL) AND ([iitr316] = NULL) AND ([iitr318] = NULL) AND ([iitr323] = NULL) AND ([iitr327] = NULL) AND ([iitr334] = NULL) AND ([iitr335] = NULL) AND ([iitr337] = NULL) AND ([iitr339] = NULL) AND (ALL OCCURRENCES OF([iitr348]) = NULL) AND (ALL OCCURRENCES OF([iitr561]) = NULL) AND ([iitr650] = NULL) AND (ALL OCCURRENCES OF([iitr655]) = NULL) AND ([iitr358] = NULL) AND COUNT(SCHEDULE = "DDCTNS") = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            */
            assertion = ((report.iitrParentDocument.RPSaWSeqNumContextCollection != null ? report.iitrParentDocument.RPSaWSeqNumContextCollection.Exists(context => context.IITR62 != null) == false : true) && report.iitrParentDocument.IITR65 == null && (report.iitrParentDocument.RPEmpLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPEmpLSPSeqNumContextCollection.Exists(context => context.IITR68 != null) == false && report.iitrParentDocument.RPEmpLSPSeqNumContextCollection.Exists(context => context.IITR71 != null) == false : true) && report.iitrParentDocument.IITR86 == null && report.iitrParentDocument.IITR89 == null && report.iitrParentDocument.IITR92 == null && report.iitrParentDocument.IITR93 == null && report.iitrParentDocument.IITR505 == null && report.iitrParentDocument.IITR94 == null && report.iitrParentDocument.IITR95 == null && report.iitrParentDocument.IITR115 == null && report.iitrParentDocument.IITR118 == null && report.iitrParentDocument.IITR119 == null && report.iitrParentDocument.IITR120 == null && report.iitrParentDocument.IITR134 == null && report.iitrParentDocument.IITR136 == null && report.iitrParentDocument.IITR137 == null && report.iitrParentDocument.IITR139 == null && report.iitrParentDocument.IITR141 == null && report.iitrParentDocument.IITR148 == null && report.iitrParentDocument.IITR502 == null && report.iitrParentDocument.IITR503 == null && report.iitrParentDocument.IITR504 == null && report.iitrParentDocument.IITR154 == null && report.iitrParentDocument.IITR156 == null && report.iitrParentDocument.IITR242 == null && report.iitrParentDocument.IITR243 == null && report.iitrParentDocument.IITR245 == null && report.iitrParentDocument.IITR246 == null && report.iitrParentDocument.IITR252 == null && report.iitrParentDocument.IITR255 == null && report.iitrParentDocument.IITR259 == null && report.iitrParentDocument.IITR260 == null && report.iitrParentDocument.IITR265 == null && report.iitrParentDocument.IITR281 == null && report.iitrParentDocument.IITR285 == null && report.iitrParentDocument.IITR306 == null && report.iitrParentDocument.IITR311 == null && report.iitrParentDocument.IITR316 == null && report.iitrParentDocument.IITR318 == null && report.iitrParentDocument.IITR323 == null && report.iitrParentDocument.IITR327 == null && report.iitrParentDocument.IITR334 == null && report.iitrParentDocument.IITR335 == null && report.iitrParentDocument.IITR337 == null && report.iitrParentDocument.IITR339 == null && (report.iitrParentDocument.RPIncSeqNumContextCollection != null ? report.iitrParentDocument.RPIncSeqNumContextCollection.Exists(context => context.IITR348 != null) == false && report.iitrParentDocument.RPIncSeqNumContextCollection.Exists(context => context.IITR561 != null) == false && report.iitrParentDocument.RPIncSeqNumContextCollection.Exists(context => context.IITR655 != null) == false : true) && report.iitrParentDocument.IITR650 == null && report.iitrParentDocument.IITR358 == null && Count(report.ddctnsChildDocuments) == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300013",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"No income or deduction amounts present. Is this a nil return?",
                    Location = "/xbrli:xbrl/tns:Income.SalaryOrWages.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR68", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR71", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR86", Value = report.iitrParentDocument.IITR86.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR89", Value = report.iitrParentDocument.IITR89.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR92", Value = report.iitrParentDocument.IITR92.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR93", Value = report.iitrParentDocument.IITR93.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR505", Value = report.iitrParentDocument.IITR505.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR94", Value = report.iitrParentDocument.IITR94.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR95", Value = report.iitrParentDocument.IITR95.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR115", Value = report.iitrParentDocument.IITR115.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR118", Value = report.iitrParentDocument.IITR118.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR119", Value = report.iitrParentDocument.IITR119.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR120", Value = report.iitrParentDocument.IITR120.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR134", Value = report.iitrParentDocument.IITR134.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR136", Value = report.iitrParentDocument.IITR136.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR137", Value = report.iitrParentDocument.IITR137.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR141", Value = report.iitrParentDocument.IITR141.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR148", Value = report.iitrParentDocument.IITR148.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR502", Value = report.iitrParentDocument.IITR502.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR503", Value = report.iitrParentDocument.IITR503.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR504", Value = report.iitrParentDocument.IITR504.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR154", Value = report.iitrParentDocument.IITR154.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR156", Value = report.iitrParentDocument.IITR156.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR242", Value = report.iitrParentDocument.IITR242.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR243", Value = report.iitrParentDocument.IITR243.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR245", Value = report.iitrParentDocument.IITR245.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR246", Value = report.iitrParentDocument.IITR246.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR252", Value = report.iitrParentDocument.IITR252.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR255", Value = report.iitrParentDocument.IITR255.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR259", Value = report.iitrParentDocument.IITR259.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR260", Value = report.iitrParentDocument.IITR260.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR265", Value = report.iitrParentDocument.IITR265.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR281", Value = report.iitrParentDocument.IITR281.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR285", Value = report.iitrParentDocument.IITR285.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR306", Value = report.iitrParentDocument.IITR306.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR311", Value = report.iitrParentDocument.IITR311.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR316", Value = report.iitrParentDocument.IITR316.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR318", Value = report.iitrParentDocument.IITR318.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR323", Value = report.iitrParentDocument.IITR323.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR327", Value = report.iitrParentDocument.IITR327.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR339", Value = report.iitrParentDocument.IITR339.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR348", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR561", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR650", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR655", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR358", Value = report.iitrParentDocument.IITR358.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300013

            #region VR.ATO.IITR.300085

            /*  VR.ATO.IITR.300085
            The taxpayer has passed the ESS income test. However, the total assessable discount amount declared can not be reduced by more than the upfront discount amount declared at Discount from taxed upfront schemes - eligible for reduction.

            Legacy Rule Format:
            IF (ConditionalValue(([iitr157] - [iitr650]  - [iitr127] + [iitr123] + [iitr124] + [iitr125] + [iitr126] ) > 0, ([iitr157] - [iitr650]  - [iitr127] + [iitr123] + [iitr124] + [iitr125] + [iitr126]), 0) + [iitr201] + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) <= 180000 AND [iitr123] <= 1000 AND ([iitr124] + [iitr125] + [iitr126]) > [iitr127]
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ConditionalValue((^IITR157 - ^IITR650  - ^IITR127 + ^IITR123 + ^IITR124 + ^IITR125 + ^IITR126) > 0, (^IITR157 - ^IITR650  - ^IITR127 + ^IITR123 + ^IITR124 + ^IITR125 + ^IITR126), 0) + ^IITR201 + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ConditionalValue(CountDocument('DDCTNS') = 1, RoundDown(Sum(^DDCTNS407), 0), 0)) <= 180000 AND ^IITR123 <= 1000 AND (^IITR124 + ^IITR125 + ^IITR126) > ^IITR127

            Data Elements:

            ^IITR123 = RP:Income.EmployeeShareScheme.TaxUpfrontReducibleDiscount.Amount

            ^IITR157 = RP:Income.Taxable.Amount

            ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

            ^IITR127 = RP:Income.EmployeeShareScheme.DiscountAssessable.Amount

            ^IITR124 = RP:Income.EmployeeShareScheme.TaxUpfrontNonReducibleDiscount.Amount

            ^IITR125 = RP:Income.EmployeeShareScheme.DeferralDiscount.Amount

            ^IITR126 = RP:Income.EmployeeShareScheme.AcquiredBeforeJuly2009Discount.Amount

            ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

            ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

            ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

            ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

            ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
            */

            assertion = (ConditionalValue(report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() - report.iitrParentDocument.IITR127.GetValueOrDefault() + report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault() > 0, report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() - report.iitrParentDocument.IITR127.GetValueOrDefault() + report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR201.GetValueOrDefault() + ConditionalValue(report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + sumDDCTNS407 <= 180000 && report.iitrParentDocument.IITR123.GetValueOrDefault() <= 1000 && report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault() > report.iitrParentDocument.IITR127.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300085",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"'Discount from taxed upfront schemes - eligible for reduction' amount is incorrect",
                    LongDescription = @"The taxpayer has passed the ESS income test. However, the total assessable discount amount declared can not be reduced by more than the upfront discount amount declared at Discount from taxed upfront schemes - eligible for reduction.",
                    Location = "/xbrli:xbrl/tns:Income.EmployeeShareScheme.TaxUpfrontReducibleDiscount.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300085" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR127", Value = report.iitrParentDocument.IITR127.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR123", Value = report.iitrParentDocument.IITR123.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR124", Value = report.iitrParentDocument.IITR124.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR125", Value = report.iitrParentDocument.IITR125.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR126", Value = report.iitrParentDocument.IITR126.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300085

            #region VR.ATO.IITR.300086

            /*  VR.ATO.IITR.300086
            Your income test amount exceeds $180,000. Consequently you are not entitled to a reduction on the upfront discount amount. The Total assessable discount amount declared must equal the sum of Discount from taxed upfront schemes - eligible for reduction, Discount from taxed upfront schemes - not eligible for reduction, Discount from deferral schemes and Discount on ESS Interests acquired pre 1 July 2009 and 'cessation time' occurred during financial year.

            Legacy Rule Format:
            IF (ConditionalValue(([iitr157] - [iitr650] - [iitr127] + [iitr123] + [iitr124] + [iitr125] + [iitr126]) > 0, ([iitr157] - [iitr650] - [iitr127] + [iitr123] + [iitr124] + [iitr125] + [iitr126]),0) + [iitr201] + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) > 180000 AND (([iitr123] + [iitr124] + [iitr125] + [iitr126]) > [iitr127])
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ConditionalValue((^IITR157 - ^IITR650 - ^IITR127 + ^IITR123 + ^IITR124 + ^IITR125 + ^IITR126) > 0, (^IITR157 - ^IITR650 - ^IITR127 + ^IITR123 + ^IITR124 + ^IITR125 + ^IITR126),0) + ^IITR201 + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ConditionalValue(CountDocument('DDCTNS') = 1, RoundDown(Sum(^DDCTNS407), 0), 0)) > 180000 AND ((^IITR123 + ^IITR124 + ^IITR125 + ^IITR126) > ^IITR127)

            Data Elements:

            ^IITR124 = RP:Income.EmployeeShareScheme.TaxUpfrontNonReducibleDiscount.Amount

            ^IITR157 = RP:Income.Taxable.Amount

            ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

            ^IITR127 = RP:Income.EmployeeShareScheme.DiscountAssessable.Amount

            ^IITR123 = RP:Income.EmployeeShareScheme.TaxUpfrontReducibleDiscount.Amount

            ^IITR125 = RP:Income.EmployeeShareScheme.DeferralDiscount.Amount

            ^IITR126 = RP:Income.EmployeeShareScheme.AcquiredBeforeJuly2009Discount.Amount

            ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

            ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

            ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

            ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

            ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
            */

            assertion = (ConditionalValue(report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() - report.iitrParentDocument.IITR127.GetValueOrDefault() + report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault() > 0, report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() - report.iitrParentDocument.IITR127.GetValueOrDefault() + report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR201.GetValueOrDefault() + ConditionalValue(report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + sumDDCTNS407 > 180000 && report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault() > report.iitrParentDocument.IITR127.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reduction on the upfront discount amount has been disallowed",
                    LongDescription = @"Your income test amount exceeds $180,000. Consequently you are not entitled to a reduction on the upfront discount amount. The Total assessable discount amount declared must equal the sum of Discount from taxed upfront schemes - eligible for reduction, Discount from taxed upfront schemes - not eligible for reduction, Discount from deferral schemes and Discount on ESS Interests acquired pre 1 July 2009 and 'cessation time' occurred during financial year.",
                    Location = "/xbrli:xbrl/tns:Income.EmployeeShareScheme.TaxUpfrontNonReducibleDiscount.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300086" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR127", Value = report.iitrParentDocument.IITR127.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR123", Value = report.iitrParentDocument.IITR123.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR124", Value = report.iitrParentDocument.IITR124.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR125", Value = report.iitrParentDocument.IITR125.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR126", Value = report.iitrParentDocument.IITR126.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300086

            #region VR.ATO.IITR.300084

            /*  VR.ATO.IITR.300084
            The taxpayer has passed the ESS income test. However, the total assessable discount amount cannot be reduced by more than $1,000.

            Legacy Rule Format:
            IF [iitr123] <> NULL AND (ConditionalValue(([iitr157] - [iitr650] - [iitr127] + [iitr123] + [iitr124] + [iitr125] + [iitr126]) > 0, ([iitr157] - [iitr650] - [iitr127] + [iitr123] + [iitr124] + [iitr125] + [iitr126]), 0) + [iitr201] + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) <= 180000 AND ([iitr123] + [iitr124] + [iitr125] + [iitr126] - 1000) > [iitr127]
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IITR123 <> NULL AND (ConditionalValue((^IITR157 - ^IITR650 - ^IITR127 + ^IITR123 + ^IITR124 + ^IITR125 + ^IITR126) > 0, (^IITR157 - ^IITR650 - ^IITR127 + ^IITR123 + ^IITR124 + ^IITR125 + ^IITR126), 0) + ^IITR201 + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ConditionalValue(CountDocument('DDCTNS') = 1, RoundDown(Sum(^DDCTNS407), 0), 0)) <= 180000 AND (^IITR123 + ^IITR124 + ^IITR125 + ^IITR126 - 1000) > ^IITR127

            Data Elements:

            ^IITR127 = RP:Income.EmployeeShareScheme.DiscountAssessable.Amount

            ^IITR123 = RP:Income.EmployeeShareScheme.TaxUpfrontReducibleDiscount.Amount

            ^IITR157 = RP:Income.Taxable.Amount

            ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

            ^IITR124 = RP:Income.EmployeeShareScheme.TaxUpfrontNonReducibleDiscount.Amount

            ^IITR125 = RP:Income.EmployeeShareScheme.DeferralDiscount.Amount

            ^IITR126 = RP:Income.EmployeeShareScheme.AcquiredBeforeJuly2009Discount.Amount

            ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

            ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

            ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

            ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

            ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
            */
            assertion = (report.iitrParentDocument.IITR123 != null && ConditionalValue(report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() - report.iitrParentDocument.IITR127.GetValueOrDefault() + report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault() > 0, report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() - report.iitrParentDocument.IITR127.GetValueOrDefault() + report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR201.GetValueOrDefault() + ConditionalValue(report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + sumDDCTNS407 <= 180000 && report.iitrParentDocument.IITR123.GetValueOrDefault() + report.iitrParentDocument.IITR124.GetValueOrDefault() + report.iitrParentDocument.IITR125.GetValueOrDefault() + report.iitrParentDocument.IITR126.GetValueOrDefault() - 1000 > report.iitrParentDocument.IITR127.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300084",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total assessable discount amount is incorrect",
                    LongDescription = @"The taxpayer has passed the ESS income test. However, the total assessable discount amount cannot be reduced by more than $1,000.",
                    Location = "/xbrli:xbrl/tns:Income.EmployeeShareScheme.DiscountAssessable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300084" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR123", Value = report.iitrParentDocument.IITR123.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR127", Value = report.iitrParentDocument.IITR127.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR124", Value = report.iitrParentDocument.IITR124.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR125", Value = report.iitrParentDocument.IITR125.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR126", Value = report.iitrParentDocument.IITR126.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300084

            #region VR.ATO.IITR.600024

            /*  VR.ATO.IITR.600024
            Where Deduction claimed (Personal superannuation contributions) and Gifts or donations have created or added to a loss (i.e. after calculating total income less all deductions) before deducting total production losses (PP and non-PP) being claimed for this income year, either partial or full adjustment is required to Deduction claimed and Gifts or donations.  To determine the amount of allowable Deduction claimed and Gifts or donations, subtract all deductions except Deduction claimed, Gifts or donations, and total production losses (PP and non-PP) from Individual total income or loss amount and:  a)  if this result is zero or negative then set Deduction claimed and Gifts or donations amounts to zero; or b) if this result is positive and greater than Deduction claimed and Gifts or donations then allow both amounts in full; or c)  if this result is positive and less than or equal to Personal contributions and gifts then only that amount that reduces Taxable income or loss to zero is allowed (i.e. disallow remainder of Deduction claimed and Gifts or donations that would create a loss).

            Legacy Rule Format:
            IF (([iitr157] + [iitr154] + [iitr156]) < 0) AND ((ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr148]) > 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (([iitr157] + [iitr154] + [iitr156]) < 0) AND ((ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr148]) > 0)

            Data Elements:

            ^IITR148 = RP:IncomeTax.Deduction.GiftDonation.Amount

            ^IITR157 = RP:Income.Taxable.Amount

            ^IITR154 = RP:IncomeTax.LossesClaimedPrimaryProduction.Amount

            ^IITR156 = RP:IncomeTax.LossesClaimedNonPrimaryProduction.Amount

            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount (Previously known as IITR362)
            */

            assertion = (report.iitrParentDocument.IITR157.GetValueOrDefault() + report.iitrParentDocument.IITR154.GetValueOrDefault() + report.iitrParentDocument.IITR156.GetValueOrDefault() < 0 && sumDDCTNS407 + report.iitrParentDocument.IITR148.GetValueOrDefault() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.601024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Deductions claimed amount combined with Gifts or donations amount is invalid",
                    LongDescription = @"When the 'Taxable income or loss' amount, excluding the 'Primary production losses claimed this income year' and 'Non-primary production losses claimed this income year' amounts, results in a loss, the sum of amounts 'Superannuation deduction claimed' and 'Gifts or donations' must not be a positive amount. To determine the amount of allowable 'Superannuation deduction claimed' and 'Gifts or donations', subtract all deduction item amounts from 'Individual total income or loss amount', except 'Superannuation deduction claimed', 'Gifts or donations', 'Primary production losses claimed this income year', and 'Non-primary production losses claimed this income year' and: a) if this result is zero or negative then set both 'Superannuation deduction claimed' and 'Gifts or donations' amounts to zero; or b) if this result is positive and higher than the sum of 'Deduction claimed' and 'Gifts or donations', then allow both amounts in full; or c) if this result is positive and lower than, or the same as, the sum of 'Superannuation deduction claimed' and 'Gifts or donations', then only the amount that reduces 'Taxable income or loss' to zero is allowed (i.e. disallow remainder of 'Superannuation deduction claimed' and 'Gifts or donations' that would create a loss).",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.GiftDonation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600024" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR154", Value = report.iitrParentDocument.IITR154.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR156", Value = report.iitrParentDocument.IITR156.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR148", Value = report.iitrParentDocument.IITR148.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.600024

            #region VR.ATO.IITR.600014

            /*  VR.ATO.IITR.600014
            The total of all the deductions fields does not equal the amount at the Total deduction field.

            Legacy Rule Format:
            IF (([iitr150] <> NULL) AND ([iitr150] <> ([iitr134] + [iitr136] + [iitr137] + [iitr139] + [iitr141] + [iitr145] + [iitr146] + [iitr147] + [iitr148] + [iitr502] + [iitr503] + [iitr504] + [iitr358] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr366] + [iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns417], 0), 0) + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns138]), 0), 0))))
            OR (([iitr150] = NULL) AND (([iitr134] + [iitr136] + [iitr137] + [iitr139] + [iitr141] + [iitr145] + [iitr146] + [iitr147] + [iitr148] + [iitr502] + [iitr503] + [iitr504] + [iitr358] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr366] + [iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns417], 0), 0) + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns138]), 0), 0)) <> 0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            IF (([iitr150] <> NULL) AND ([iitr150] <> ([iitr134] + [iitr136] + [iitr137] + [iitr139] + [iitr141] + [iitr145] + [iitr146] + [iitr147] + [iitr148] + [iitr502] + [iitr503] + [iitr504] + [iitr358] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr366] + [iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns417], 0), 0) + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns138]), 0), 0))))
            OR (([iitr150] = NULL) AND (([iitr134] + [iitr136] + [iitr137] + [iitr139] + [iitr141] + [iitr145] + [iitr146] + [iitr147] + [iitr148] + [iitr502] + [iitr503] + [iitr504] + [iitr358] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1,RoundDown(SUM([ddctns407]), 0), 0) + [iitr366] + [iitr368] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown([ddctns417], 0), 0) + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns138]), 0), 0)) <> 0))

            Data Elements:

            ^IITR150 = RP:Expense.DeductionsTotal.Amount

            ^IITR134 = RP:IncomeTax.Deduction.Car.Amount

            ^IITR136 = RP:IncomeTax.Deduction.Travel.Amount

            ^IITR137 = RP:IncomeTax.Deduction.Clothing.Amount

            ^IITR139 = RP:IncomeTax.Deduction.EducationNet.Amount

            ^IITR141 = RP:IncomeTax.Deduction.WorkRelatedOther.Amount

            ^IITR145 = RP:IncomeTax.Deduction.LowValuePool.Amount

            ^IITR146 = RP:IncomeTax.Deduction.Interest.Amount

            ^IITR147 = RP:IncomeTax.Deduction.Dividend.Amount

            ^IITR148 = RP:IncomeTax.Deduction.GiftDonation.Amount

            ^IITR502 = RP:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount

            ^IITR503 = RP:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount

            ^IITR504 = RP:IncomeTax.Deduction.TaxAffairManagementOther.Amount

            ^IITR358 = RP:IncomeTax.Deduction.ForeignPensionAnnuityIncomeUndeductedPurchasePrice.Amount

            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount  (Formerly known as IITR362)

            ^IITR366 = RP:IncomeTax.Deduction.ProjectPool.Amount

            ^IITR368 = RP:Expense.ForestryManagedInvestmentSchemeDeduction.Amount

            ^IITR370 = RP:IncomeTax.Deduction.Election.Amount

            ^DDCTNS138 = RP.{OTHSeqNum}:Expense.DeductibleOther.Amount  (Formerly known as IITR375)
            */

            // Calculate the sum of all deductions
            decimal cal600014_1 = report.iitrParentDocument.IITR134.GetValueOrDefault() + report.iitrParentDocument.IITR136.GetValueOrDefault() + report.iitrParentDocument.IITR137.GetValueOrDefault() + report.iitrParentDocument.IITR139.GetValueOrDefault() + report.iitrParentDocument.IITR141.GetValueOrDefault() + report.iitrParentDocument.IITR145.GetValueOrDefault() + report.iitrParentDocument.IITR146.GetValueOrDefault() + report.iitrParentDocument.IITR147.GetValueOrDefault() + report.iitrParentDocument.IITR148.GetValueOrDefault() + report.iitrParentDocument.IITR502.GetValueOrDefault() + report.iitrParentDocument.IITR503.GetValueOrDefault() + report.iitrParentDocument.IITR504.GetValueOrDefault() + report.iitrParentDocument.IITR358.GetValueOrDefault() + sumDDCTNS407 + report.iitrParentDocument.IITR366.GetValueOrDefault() + report.iitrParentDocument.IITR368.GetValueOrDefault() + valDDCTNS417 + sumDDCTNS138;

            assertion = ((report.iitrParentDocument.IITR150 != null && report.iitrParentDocument.IITR150.GetValueOrDefault() != cal600014_1)
                || (report.iitrParentDocument.IITR150 == null && cal600014_1 != 0));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.602014",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Individual total deductions amount is incorrect",
                    LongDescription = @"The 'Individual total deductions amount' must equal the sum of all deductions claimed, which includes 'Work related car expenses', 'Work related travel expenses', 'Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses', 'Work related self-education expenses', 'Other work related expenses', 'Low value pool deduction', 'Interest deductions', 'Dividend deductions', 'Gifts or donations', 'Cost of managing tax affairs - Interest charged by the ATO', 'Cost of managing tax affairs - Litigation costs', 'Cost of managing tax affairs - Other expenses', 'Deductible amount of UPP of foreign pension or annuity', 'Superannuation deduction claimed', 'Individual - Deduction for project pool', 'Forestry managed investment scheme deduction', 'Election expenses amount' and 'Other deductions amount'",
                    Location = "/xbrli:xbrl/tns:Expense.DeductionsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600014" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR150", Value = report.iitrParentDocument.IITR150.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR134", Value = report.iitrParentDocument.IITR134.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR136", Value = report.iitrParentDocument.IITR136.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR137", Value = report.iitrParentDocument.IITR137.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR141", Value = report.iitrParentDocument.IITR141.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR145", Value = report.iitrParentDocument.IITR145.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR146", Value = report.iitrParentDocument.IITR146.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR147", Value = report.iitrParentDocument.IITR147.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR148", Value = report.iitrParentDocument.IITR148.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR502", Value = report.iitrParentDocument.IITR502.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR503", Value = report.iitrParentDocument.IITR503.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR504", Value = report.iitrParentDocument.IITR504.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR358", Value = report.iitrParentDocument.IITR358.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR366", Value = report.iitrParentDocument.IITR366.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR368", Value = report.iitrParentDocument.IITR368.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS417", Value = valDDCTNS417.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS138)", Value = sumDDCTNS138.ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.600014

            #region VR.ATO.IITR.200034

            /*  VR.ATO.IITR.200034
            Spouse's or de facto spouse's family name is present and the taxpayer had a spouse for the full year and had no Medicare levy surcharge. Combined Total reportable fringe benefits amounts exceed $3,772 and taxable income or loss plus total reportable fringe benefits amounts plus net amount on which family trust has been paid plus exempt foreign employment income exceeds $22,398 and number of days not liable for MLS, spouse income details also required.

            Legacy Rule Format:
            IF ([iitr213] <> NULLORBLANK OR [iitr220] = TRUE) AND [iitr176] = FALSE AND ([iitr157] + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue(ANY OCCURRENCE OF([iitr102]) <> TRUE AND [iitr29] <> NULL AND ([iitr29] > ConvertToDate(30,06,([iitr10] - 55)) OR [iitr29] <= ConvertToDate(01,07,([iitr10] - 61))), SUM([iitr99]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) > 22398 AND ([iitr177] = NULL OR ([iitr225] = NULL AND [iitr226] = NULL AND [iitr227] = NULL AND [iitr492] = NULL AND [iitr493] = NULL AND [iitr231] = NULL AND [iitr234] = NULL AND [iitr236] = NULL))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ([iitr213] <> NULLORBLANK OR [iitr220] = TRUE) AND [iitr176] = FALSE AND ([iitr157] + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue(ANY OCCURRENCE OF([iitr102]) <> TRUE AND [iitr29] <> NULL AND ([iitr29] > ConvertToDate(30,06,([iitr10] - 55)) OR [iitr29] <= ConvertToDate(01,07,([iitr10] - 61))), SUM([iitr99]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) > 22398 AND ([iitr177] = NULL OR ([iitr225] = NULL AND [iitr226] = NULL AND [iitr227] = NULL AND [iitr492] = NULL AND [iitr493] = NULL AND [iitr231] = NULL AND [iitr234] = NULL AND [iitr236] = NULL))

            Data Elements:

            RP:^IITR177 = tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count

            RP.SPOUSE:^IITR213 = tns:PersonNameDetails.FamilyName.Text

            RP:^IITR220 = tns:PersonDemographicDetails.SpouseFullYear.Indicator

            RP:^IITR176 = tns:IncomeTax.MedicareLevy.PrivateHospitalCoverWholePeriod.Indicator

            RP:^IITR157 = tns:Income.Taxable.Amount

            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount

            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount

            RP:^IITR330 = tns:InternationalDealings.ExemptForeignEmploymentIncome.Amount

            RP:^IITR399 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount

            RP.{SLSPSeqNum}:^IITR102 = tns:Income.Superannuation.DeathBenefitPaymentNonDependant.Indicator

            RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date

            RP:^IITR10 = tns:Report.TargetFinancial.Year

            RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount

            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount

            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount

            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount

            RP.{SCSeqNum}:^DDCTNS407 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount  (Previously known as IITR362)

            RP.SPOUSE:^IITR225 = tns:Income.Taxable.Amount

            RP.SPOUSE:^IITR226 = tns:IncomeTax.TrustIncomeNotIncludedInTaxableIncome.Amount

            RP.SPOUSE:^IITR227 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount

            RP.SPOUSE.Eligible:^IITR492 = tns:Income.FringeBenefitsReportable.Amount

            RP.SPOUSE.Ineligible:^IITR493 = tns:Income.FringeBenefitsReportable.Amount

            RP.SPOUSE:^IITR231 = tns:SuperannuationContribution.EmployerReportable.Amount

            RP.SPOUSE:^IITR234 = tns:Tax.Losses.InvestmentNet.Amount

            RP.SPOUSE:^IITR236 = tns:SuperannuationPayment.BenefitLumpSumZeroTaxRate.Amount
            */
            assertion = ((string.IsNullOrWhiteSpace(report.iitrParentDocument.IITR213) != true || report.iitrParentDocument.IITR220 == true) && report.iitrParentDocument.IITR176 == false && (report.iitrParentDocument.IITR157.GetValueOrDefault() + ConditionalValue(report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault(), 0) + ConditionalValue(report.iitrParentDocument.IITR157.GetValueOrDefault() > 0, report.iitrParentDocument.IITR330.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR399.GetValueOrDefault() - ConditionalValue((report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.Any(IITR102Repeat => IITR102Repeat.IITR102 != true) : false) && report.iitrParentDocument.IITR29 != null && (report.iitrParentDocument.IITR29.GetValueOrDefault() > ConvertToDate(30, 6, report.iitrParentDocument.IITR10.GetValueOrDefault() - 55) || report.iitrParentDocument.IITR29.GetValueOrDefault() <= ConvertToDate(1, 7, report.iitrParentDocument.IITR10.GetValueOrDefault() - 61)), (report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.Sum(f => f.IITR99.GetValueOrDefault()) : 0), 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407) > 22398 && (report.iitrParentDocument.IITR177 == null || (report.iitrParentDocument.IITR225 == null && report.iitrParentDocument.IITR226 == null && report.iitrParentDocument.IITR227 == null && report.iitrParentDocument.IITR492 == null && report.iitrParentDocument.IITR493 == null && report.iitrParentDocument.IITR231 == null && report.iitrParentDocument.IITR234 == null && report.iitrParentDocument.IITR236 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.202034",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of days not liable for surcharge or spouse income details is required",
                    LongDescription = @"Spouse's or de facto spouse's family name is present and the taxpayer had a spouse for the full year and had no Medicare levy surcharge. Combined Total reportable fringe benefits amounts exceed $3,772 and taxable income or loss plus total reportable fringe benefits amounts plus net amount on which family trust has been paid plus exempt foreign employment income exceeds $22,398 and number of days not liable for MLS, spouse income details also required.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.200034" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR213", Value = report.iitrParentDocument.IITR213 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR220", Value = GetValueOrEmpty(report.iitrParentDocument.IITR220) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR176", Value = GetValueOrEmpty(report.iitrParentDocument.IITR176) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR330", Value = report.iitrParentDocument.IITR330.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR399", Value = report.iitrParentDocument.IITR399.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR102", Value = (report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.Any(IITR102Repeat => IITR102Repeat.IITR102 != true).ToString() : "false") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR29", Value = GetValueOrEmpty(report.iitrParentDocument.IITR29) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR10", Value = report.iitrParentDocument.IITR10.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR99", Value = (report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.Sum(f => f.IITR99.GetValueOrDefault()).ToString() : "0") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "SUM(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR177", Value = report.iitrParentDocument.IITR177.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR225", Value = report.iitrParentDocument.IITR225.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR226", Value = report.iitrParentDocument.IITR226.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR227", Value = report.iitrParentDocument.IITR227.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR492", Value = report.iitrParentDocument.IITR492.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR493", Value = report.iitrParentDocument.IITR493.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR231", Value = report.iitrParentDocument.IITR231.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR234", Value = report.iitrParentDocument.IITR234.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR236", Value = report.iitrParentDocument.IITR236.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.200034

            #region VR.ATO.IITR.000013

            /*  VR.ATO.IITR.000013
            Where the number of days not liable for medicare levy surcharge is less than the whole reporting period, and either income test is triggered; and the reportable employer superannuation contributions is greater than 8% of all income from salary and wages, check reportable employer superannuation contributions as MLS liability may be incurred.

            Legacy Rule Format:
            WHERE IN CONTEXT(RP.{SLSPSeqNum})
            IF ([iitr177] = NULL OR [iitr177] < (RP:period.endDate - RP:period.startDate + 1)) AND ((([iitr213] <> NULLORBLANK OR [iitr171] <> NULL) AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue((([iitr29] > ConvertToDate(30,6, [iitr10] - 55)) OR ([iitr29] <= ConvertToDate(1,7, [iitr10] - 61))), SUM(ConditionalValue([iitr102] <> TRUE, [iitr99], 0)), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr225] + ConditionalValue(([iitr492] + [iitr493]) > 3772, ([iitr492] + [iitr493]), 0) + [iitr227] + [iitr234] + [iitr231] - [iitr236]) > (180000 + ConditionalValue([iitr171] > 1, ((1500 * [iitr171]) - 1), 0)))) OR (([iitr213] = NULLORBLANK) AND ([iitr171] = NULL) AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue((([iitr29] > ConvertToDate(30,6, [iitr10] - 55)) OR ([iitr29] <= ConvertToDate(1,7, [iitr10] - 61))), SUM(ConditionalValue([iitr102] <> TRUE, [iitr99], 0)), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) > 90000))) AND ([iitr201] > (SUM([iitr62]) * 0.08))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ([iitr177] = NULL OR [iitr177] < (RP:period.endDate - RP:period.startDate + 1)) AND ((([iitr213] <> NULLORBLANK OR [iitr171] <> NULL) AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue((([iitr29] > ConvertToDate(30,6, [iitr10] - 55)) OR ([iitr29] <= ConvertToDate(1,7, [iitr10] - 61))), SUM(ConditionalValue([iitr102] <> TRUE, [iitr99], 0)), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr225] + ConditionalValue(([iitr492] + [iitr493]) > 3772, ([iitr492] + [iitr493]), 0) + [iitr227] + [iitr234] + [iitr231] - [iitr236]) > (180000 + ConditionalValue([iitr171] > 1, ((1500 * [iitr171]) - 1), 0)))) OR (([iitr213] = NULLORBLANK) AND ([iitr171] = NULL) AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue((([iitr29] > ConvertToDate(30,6, [iitr10] - 55)) OR ([iitr29] <= ConvertToDate(1,7, [iitr10] - 61))), SUM(ConditionalValue([iitr102] <> TRUE, [iitr99], 0)), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0)) > 90000))) AND ([iitr201] > (SUM([iitr62]) * 0.08))

            Data Elements:

            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount

            RP:^EndDate = RP

            RP:^StartDate1 = RP

            RP:^IITR177 = tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count

            RP.SPOUSE:^IITR213 = tns:PersonNameDetails.FamilyName.Text

            RP:^IITR171 = tns:IncomeTax.MedicareLevy.DependentChildrenAndStudents.Count

            RP:^IITR157 = tns:Income.Taxable.Amount

            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount

            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount

            RP:^IITR330 = tns:InternationalDealings.ExemptForeignEmploymentIncome.Amount

            RP:^IITR399 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount

            RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date

            RP:^IITR10 = tns:Report.TargetFinancial.Year

            RP.{SLSPSeqNum}:^IITR102 = tns:Income.Superannuation.DeathBenefitPaymentNonDependant.Indicator

            RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount

            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount

            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount

            RP.{SCSeqNum}:^DDCTNS407 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount   (previously known as IITR362)

            RP.SPOUSE:^IITR225 = tns:Income.Taxable.Amount

            RP.SPOUSE.Eligible:^IITR492 = tns:Income.FringeBenefitsReportable.Amount

            RP.SPOUSE.Ineligible:^IITR493 = tns:Income.FringeBenefitsReportable.Amount

            RP.SPOUSE:^IITR227 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount

            RP.SPOUSE:^IITR234 = tns:Tax.Losses.InvestmentNet.Amount

            RP.SPOUSE:^IITR231 = tns:SuperannuationContribution.EmployerReportable.Amount

            RP.SPOUSE:^IITR236 = tns:SuperannuationPayment.BenefitLumpSumZeroTaxRate.Amount

            RP.{SaWSeqNum}:^IITR62 = tns:Income.SalaryOrWages.Amount
            */

            assertion = ((report.iitrParentDocument.IITR177 == null || Convert.ToDouble(report.iitrParentDocument.IITR177.GetValueOrDefault()) < (report.iitrParentDocument.RPEndDate - report.iitrParentDocument.RPStartDate).GetValueOrDefault().TotalDays + 1) && (((!string.IsNullOrWhiteSpace(report.iitrParentDocument.IITR213) || report.iitrParentDocument.IITR171 != null) && ((report.iitrParentDocument.IITR157.GetValueOrDefault() > 0 ? report.iitrParentDocument.IITR157.GetValueOrDefault() : 0) + ((report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault()) > 3772 ? (report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault()) : 0) + (report.iitrParentDocument.IITR157.GetValueOrDefault() > 0 ? report.iitrParentDocument.IITR330.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR399.GetValueOrDefault() - (report.iitrParentDocument.IITR29 > ConvertToDate(30, 6, Convert.ToInt32(report.iitrParentDocument.IITR10.GetValueOrDefault()) - 55) || report.iitrParentDocument.IITR29 <= ConvertToDate(1, 7, Convert.ToInt32(report.iitrParentDocument.IITR10.GetValueOrDefault()) - 61) ? (report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.FindAll(f => f.IITR102 != true).Sum(f => f.IITR99.GetValueOrDefault()) : 0) : 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 + report.iitrParentDocument.IITR225.GetValueOrDefault() + (report.iitrParentDocument.IITR492.GetValueOrDefault() + report.iitrParentDocument.IITR493.GetValueOrDefault() > 3772 ? report.iitrParentDocument.IITR492.GetValueOrDefault() + report.iitrParentDocument.IITR493.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR227.GetValueOrDefault() + report.iitrParentDocument.IITR234.GetValueOrDefault() + report.iitrParentDocument.IITR231.GetValueOrDefault() - report.iitrParentDocument.IITR236.GetValueOrDefault()) > (180000 + (report.iitrParentDocument.IITR171.GetValueOrDefault() > 1 ? 1500 * report.iitrParentDocument.IITR171.GetValueOrDefault() - 1 : 0))) || (string.IsNullOrWhiteSpace(report.iitrParentDocument.IITR213) && report.iitrParentDocument.IITR171 == null && ((report.iitrParentDocument.IITR157.GetValueOrDefault() > 0 ? report.iitrParentDocument.IITR157.GetValueOrDefault() : 0) + (report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772 ? report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() : 0) + (report.iitrParentDocument.IITR157.GetValueOrDefault() > 0 ? report.iitrParentDocument.IITR330.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR399.GetValueOrDefault() - (report.iitrParentDocument.IITR29 > ConvertToDate(30, 6, Convert.ToInt32(report.iitrParentDocument.IITR10) - 55) || report.iitrParentDocument.IITR29 <= ConvertToDate(1, 7, Convert.ToInt32(report.iitrParentDocument.IITR10) - 61) ? (report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.FindAll(f => f.IITR102 != true).Sum(f => f.IITR99.GetValueOrDefault()) : 0) : 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407) > 90000)) && (report.iitrParentDocument.RPSaWSeqNumContextCollection != null ? Convert.ToDouble(report.iitrParentDocument.IITR201.GetValueOrDefault()) > Convert.ToDouble(report.iitrParentDocument.RPSaWSeqNumContextCollection.Sum(f => f.IITR62.GetValueOrDefault())) * 0.08 : false));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.000013",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check reportable employer superannuation contributions as MLS liability may be incurred",
                    LongDescription = @"Check that compulsory super contributions have not been included in reportable employer superannuation contributions at the income test section. Some reportable employer superannuation contributions are incorrectly reported on payment summaries, these amounts are generally salary sacrifice super contributions only. Incorrect reporting may cause a MLS liability.",
                    Location = "/xbrli:xbrl/tns:SuperannuationContribution.EmployerReportable.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR177", Value = report.iitrParentDocument.IITR177.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR213", Value = report.iitrParentDocument.IITR213 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR171", Value = report.iitrParentDocument.IITR171.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR330", Value = report.iitrParentDocument.IITR330.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR399", Value = report.iitrParentDocument.IITR399.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR29", Value = GetValueOrEmpty(report.iitrParentDocument.IITR29) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR10", Value = report.iitrParentDocument.IITR10.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR99", Value = (report.iitrParentDocument.RPSLSPSeqNumContextCollection != null ? report.iitrParentDocument.RPSLSPSeqNumContextCollection.FindAll(f => f.IITR102 != true).Sum(f => f.IITR99).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "SUM(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR225", Value = report.iitrParentDocument.IITR225.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR492", Value = report.iitrParentDocument.IITR492.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR493", Value = report.iitrParentDocument.IITR493.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR227", Value = report.iitrParentDocument.IITR227.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR234", Value = report.iitrParentDocument.IITR234.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR231", Value = report.iitrParentDocument.IITR231.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR236", Value = report.iitrParentDocument.IITR236.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = (report.iitrParentDocument.RPSaWSeqNumContextCollection != null ? report.iitrParentDocument.RPSaWSeqNumContextCollection.Sum(f => f.IITR62).ToString() : "null") });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.000013

            #region VR.ATO.IITR.200015

            /*  VR.ATO.IITR.200015
            Where the 'Seniors and pensioners - Tax offset code' is being claimed, and the sum of income fields is greater than $83,580 where there is a spouse, or $50,119 where there is no spouse; and the Reportable employer superannuation contributions is greater than 8% of all income from salary and wages, check Reportable employer superannuation contributions as SPO entitlement may be impacted.

            Legacy Rule Format:
            IF [iitr163] <> NULLORBLANK AND (([iitr213] <> NULLORBLANK AND (ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr204] + [iitr205] + [iitr225] + [iitr230] + [iitr226] + ConditionalValue(([iitr492] + [iitr493]) > 3772, (([iitr492] * 0.530) + [iitr493]), 0) + [iitr231] + [iitr234]) > 83580) OR ([iitr213] = NULLORBLANK AND (ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr204] + [iitr205]) > 50119)) AND [iitr201] > (SUM([iitr62]) * 0.08)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            [iitr163] <> NULLORBLANK AND (([iitr213] <> NULLORBLANK AND (ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr204] + [iitr205] + [iitr225] + [iitr230] + [iitr226] + ConditionalValue(([iitr492] + [iitr493]) > 3772, (([iitr492] * 0.530) + [iitr493]), 0) + [iitr231] + [iitr234]) > 83580) OR ([iitr213] = NULLORBLANK AND (ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr204] + [iitr205]) > 50119)) AND [iitr201] > (SUM([iitr62]) * 0.08)

            Data Elements:

            ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

            ^IITR163 = RP:TaxOffsetClaim.SeniorAndPensioner.Code

            ^IITR213 = RP.SPOUSE:PersonNameDetails.FamilyName.Text

            ^IITR157 = RP:Income.Taxable.Amount

            ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

            ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount    (previously IITR362)

            ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

            ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

            ^IITR225 = RP.SPOUSE:Income.Taxable.Amount

            ^IITR230 = RP.SPOUSE:Pension.SpouseExempt.Amount

            ^IITR226 = RP.SPOUSE:IncomeTax.TrustIncomeNotIncludedInTaxableIncome.Amount

            ^IITR492 = RP.SPOUSE.Eligible:Income.FringeBenefitsReportable.Amount

            ^IITR493 = RP.SPOUSE.Ineligible:Income.FringeBenefitsReportable.Amount

            ^IITR231 = RP.SPOUSE:SuperannuationContribution.EmployerReportable.Amount

            ^IITR234 = RP.SPOUSE:Tax.Losses.InvestmentNet.Amount

            ^IITR62 = RP.{SaWSeqNum}:Income.SalaryOrWages.Amount
            */
            assertion = (string.IsNullOrWhiteSpace(report.iitrParentDocument.IITR163) != true && (string.IsNullOrWhiteSpace(report.iitrParentDocument.IITR213) != true && ConditionalValue(report.iitrParentDocument.IITR157.GetValueOrDefault() > 0, report.iitrParentDocument.IITR157.GetValueOrDefault(), 0) + ConditionalValue(report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR490.GetValueOrDefault() * 0.530M + report.iitrParentDocument.IITR491.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR225.GetValueOrDefault() + report.iitrParentDocument.IITR230.GetValueOrDefault() + report.iitrParentDocument.IITR226.GetValueOrDefault() + ConditionalValue(report.iitrParentDocument.IITR492.GetValueOrDefault() + report.iitrParentDocument.IITR493.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR492.GetValueOrDefault() * 0.530M + report.iitrParentDocument.IITR493.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR231.GetValueOrDefault() + report.iitrParentDocument.IITR234.GetValueOrDefault() > 83580 || string.IsNullOrWhiteSpace(report.iitrParentDocument.IITR213) == true && ConditionalValue(report.iitrParentDocument.IITR157.GetValueOrDefault() > 0, report.iitrParentDocument.IITR157.GetValueOrDefault(), 0) + ConditionalValue(report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772, report.iitrParentDocument.IITR490.GetValueOrDefault() * 0.530M + report.iitrParentDocument.IITR491.GetValueOrDefault(), 0) + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() > 50119) && report.iitrParentDocument.IITR201.GetValueOrDefault() > (report.iitrParentDocument.RPSaWSeqNumContextCollection == null ? 0 : report.iitrParentDocument.RPSaWSeqNumContextCollection.Sum(f => f.IITR62.GetValueOrDefault())) * 0.08M);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.200015",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check reportable employer superannuation contributions as SPO entitlement may be impacted",
                    LongDescription = @"Check that compulsory super contributions have not been included in reportable employer superannuation contributions at the income test section. Some reportable employer superannuation contributions are incorrectly reported on payment summaries, these amounts are generally salary sacrifice super contributions only.  Incorrect reporting will result in reduced Senior and pensioner offset entitlement.",
                    Location = "/xbrli:xbrl/tns:SuperannuationContribution.EmployerReportable.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.200015" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR163", Value = report.iitrParentDocument.IITR163 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR213", Value = report.iitrParentDocument.IITR213 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR225", Value = report.iitrParentDocument.IITR225.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR230", Value = report.iitrParentDocument.IITR230.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR226", Value = report.iitrParentDocument.IITR226.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR492", Value = report.iitrParentDocument.IITR492.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR493", Value = report.iitrParentDocument.IITR493.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR231", Value = report.iitrParentDocument.IITR231.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR234", Value = report.iitrParentDocument.IITR234.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = "IITR62" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.200015

            #region VR.ATO.IITR.100043

            /*  VR.ATO.IITR.100043
            Adjusted taxable income must equal the sum of (when amount above $0): Taxable income or loss income (excluding any assessable First home super saver released amount), Reportable employer superannuation contributions, Tax-free government pensions, Target foreign income, Net financial investment loss, Net rental property loss, and all Deduction claimed amounts, plus Total reportable fringe benefits amount from employers not exempt under section 57A and 53.0% of Total reportable fringe benefits amount from employers exempt under section 57A (when combined Total reportable fringe benefits amount above $3,772), less Child support you paid. If you are completing a tax return for a deceased person, this value must be divided by the number of days the taxpayer was alive during the financial year and multiplied by the total number of days in the financial year.

            Legacy Rule Format:
            IF (([iitr28] = NULL OR [iitr28] > RP:period.endDate) AND (([iitr208] <> NULL AND [iitr208] <> ((ConditionalValue(([iitr157] - [iitr650]) > 0, ([iitr157] - [iitr650]), 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) +/- 1)) OR ([iitr208] = NULL AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) +/- 1) <> 0))) OR (([iitr28] > (RP:period.startDate - 1) AND [iitr28] < (RP:period.endDate + 1)) AND (([iitr208] <> NULL AND [iitr208] <> (((ConditionalValue(([iitr157] - [iitr650]) > 0, ([iitr157] - [iitr650]), 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) * (RP:period.endDate - RP:period.startDate + 1) / ([iitr28] - RP:period.startDate + 1)) +/- 1)) OR ([iitr208] = NULL AND ((((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) * (RP:period.endDate - RP:period.startDate + 1) / ([iitr28] - RP:period.startDate + 1)) +/- 1) <> 0)))) OR ([iitr28] < RP:period.startDate AND [iitr208] <> 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            IF (([iitr28] = NULL OR [iitr28] > RP:period.endDate) AND (([iitr208] <> NULL AND [iitr208] <> ((ConditionalValue(([iitr157] - [iitr650]) > 0, ([iitr157] - [iitr650]), 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) +/- 1)) OR ([iitr208] = NULL AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) +/- 1) <> 0))) OR (([iitr28] > (RP:period.startDate - 1) AND [iitr28] < (RP:period.endDate + 1)) AND (([iitr208] <> NULL AND [iitr208] <> (((ConditionalValue(([iitr157] - [iitr650]) > 0, ([iitr157] - [iitr650]), 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) * (RP:period.endDate - RP:period.startDate + 1) / ([iitr28] - RP:period.startDate + 1)) +/- 1)) OR ([iitr208] = NULL AND ((((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - [iitr206]) * (RP:period.endDate - RP:period.startDate + 1) / ([iitr28] - RP:period.startDate + 1)) +/- 1) <> 0)))) OR ([iitr28] < RP:period.startDate AND [iitr208] <> 0)
            Data Elements:

            RP:^IITR208 = tns:Income.TaxableAdjusted.Amount

            RP:^EndDate = RP

            RP:^StartDate1 = RP

            RP:^StartDate2 = RP

            RP:^StartDate3 = RP

            RP:^StartDate4 = RP

            RP:^StartDate5 = RP

            RP:^StartDate6 = RP

            RP:^IITR28 = tns:PersonDemographicDetails.Death.Date

            RP:^IITR157 = tns:Income.Taxable.Amount

            RP:^IITR650 = tns:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount

            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount

            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount

            RP:^IITR202 = tns:Pension.TaxFree.Amount

            RP:^IITR203 = tns:InternationalDealings.ForeignIncomeTarget.Amount

            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount

            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount

            RP.{SCSeqNum}:^DDCTNS407 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount    (previously known as IITR362 )

            RP:^IITR206 = tns:ChildSupport.Payment.Amount
            */
            double calc1_100043 = (report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault() > 0 ? Convert.ToDouble(report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) : 0) + (report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772 ? Math.Floor((Convert.ToDouble(report.iitrParentDocument.IITR490.GetValueOrDefault()) * 0.53) + Convert.ToDouble(report.iitrParentDocument.IITR491.GetValueOrDefault())) : 0) + Convert.ToDouble(report.iitrParentDocument.IITR201.GetValueOrDefault() + report.iitrParentDocument.IITR202.GetValueOrDefault() + report.iitrParentDocument.IITR203.GetValueOrDefault() + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + sumDDCTNS407 - report.iitrParentDocument.IITR206.GetValueOrDefault());
            double calc2_100043 = (report.iitrParentDocument.IITR28 != null ? calc1_100043 * (((report.iitrParentDocument.RPEndDate - report.iitrParentDocument.RPStartDate).Value.TotalDays + 1) / ((Convert.ToDateTime(report.iitrParentDocument.IITR28) - report.iitrParentDocument.RPStartDate).Value.TotalDays + 1)) : 0);

            assertion = (((report.iitrParentDocument.IITR28 == null || Convert.ToDateTime(report.iitrParentDocument.IITR28) > report.iitrParentDocument.RPEndDate) && (((report.iitrParentDocument.IITR208 != null && Convert.ToDouble(report.iitrParentDocument.IITR208) < (calc1_100043 - 1) || Convert.ToDouble(report.iitrParentDocument.IITR208) > (calc1_100043 + 1))) || (report.iitrParentDocument.IITR208 == null && (0 < (calc1_100043 - 1) || 0 > (calc1_100043 + 1))))) || (report.iitrParentDocument.IITR28 != null ? ((Convert.ToDateTime(report.iitrParentDocument.IITR28) > report.iitrParentDocument.RPStartDate.Value.AddDays(-1) && Convert.ToDateTime(report.iitrParentDocument.IITR28) < report.iitrParentDocument.RPEndDate.Value.AddDays(1)) && ((report.iitrParentDocument.IITR208 != null && (Convert.ToDouble(report.iitrParentDocument.IITR208.GetValueOrDefault()) < (calc2_100043 - 1) || Convert.ToDouble(report.iitrParentDocument.IITR208.GetValueOrDefault()) > (calc2_100043 + 1))) || report.iitrParentDocument.IITR208 == null && (0 < (calc2_100043 - 1) || 0 > (calc2_100043 + 1)))) : false) || ((report.iitrParentDocument.IITR28 != null ? report.iitrParentDocument.IITR28 < report.iitrParentDocument.RPStartDate : false) && (report.iitrParentDocument.IITR208 != null ? report.iitrParentDocument.IITR208 : 0) != 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.024015",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Adjusted taxable income is incorrect",
                    LongDescription = @"Adjusted taxable income must equal the sum of (when amount above $0): Taxable income or loss income (excluding any assessable First home super saver released amount), Reportable employer superannuation contributions, Tax-free government pensions, Target foreign income, Net financial investment loss, Net rental property loss, and all Deduction claimed amounts, plus Total reportable fringe benefits amount from employers not exempt under section 57A and 53.0% of Total reportable fringe benefits amount from employers exempt under section 57A (when combined Total reportable fringe benefits amount above $3,772), less Child support you paid. If you are completing a tax return for a deceased person, this value must be divided by the number of days the taxpayer was alive during the financial year and multiplied by the total number of days in the financial year.",
                    Location = "/xbrli:xbrl/tns:Income.TaxableAdjusted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100043" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR28", Value = GetValueOrEmpty(report.iitrParentDocument.IITR28) });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR202", Value = report.iitrParentDocument.IITR202.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR203", Value = report.iitrParentDocument.IITR203.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR206", Value = report.iitrParentDocument.IITR206.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR208", Value = report.iitrParentDocument.IITR208.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "StartDate", Value = GetValueOrEmpty(report.iitrParentDocument.RPStartDate) });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "EndDate", Value = GetValueOrEmpty(report.iitrParentDocument.RPEndDate) });

                response.Add(processMessage);
            }

            #endregion

            #region Repeating RPLOSSSeqNumContext
            // RPLOSSSeqNumContext
            // if (report.RPLOSSSeqNumContextCollection != null)
            if (report.iitrParentDocument.RPLOSSSeqNumContextCollection != null)
            {
                int itemIndexContext2 = 0;
                foreach (IITR2019.RPLOSSSeqNumContext rplossSeqNumContext in report.iitrParentDocument.RPLOSSSeqNumContextCollection)
                {
                    itemIndexContext2++;

                    #region VR.ATO.IITR.100031

                    /*  VR.ATO.IITR.100031
                    For a resident taxpayer, where the sum of taxable income or loss and the net loss amounts that exceeds or equal to zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl

                    Legacy Rule Format:
                    IF [iitr19] = TRUE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","5","8"), [iitr487], 0))) >= 0 AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","5","8"), [iitr487], 0)) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    [iitr19] = TRUE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","5","8"), [iitr487], 0))) >= 0 AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","5","8"), [iitr487], 0)) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000

                    Data Elements:

                    ^IITR487 = RP.{LOSSSeqNum}:Tax.Losses.Net.Amount

                    ^IITR19 = RP:Residency.TaxPurposesPersonStatus.Indicator

                    ^IITR482 = RP.{LOSSSeqNum}:IncomeTax.LossesType.Code

                    ^IITR157 = RP:Income.Taxable.Amount

                    ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

                    ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

                    ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

                    ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

                    ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

                    ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

                    ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount    (previously was IITR362)

                    ^IITR153 = RP:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount

                    ^IITR154 = RP:IncomeTax.LossesClaimedPrimaryProduction.Amount

                    ^IITR155 = RP:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount

                    ^IITR156 = RP:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
                    */

                    List<IITR2019.RPLOSSSeqNumContext> lossContextsIITR482_100031 = (report.iitrParentDocument.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|5|8)$")));

                    assertion = (report.iitrParentDocument.IITR19 == true && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && ((report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) + (lossContextsIITR482_100031.Count > 0 ? lossContextsIITR482_100031.Sum(f => f.IITR487.GetValueOrDefault()) : 0)) >= 0 && ((report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) + (lossContextsIITR482_100031.Count > 0 ? lossContextsIITR482_100031.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + (report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772 ? report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 - (report.iitrParentDocument.IITR153.GetValueOrDefault() - report.iitrParentDocument.IITR154.GetValueOrDefault()) - (report.iitrParentDocument.IITR155.GetValueOrDefault() - report.iitrParentDocument.IITR156.GetValueOrDefault())) >= 250000);

                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.100031",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"The income requirement for non-commercial losses has not been met",
                            LongDescription = @"For a resident taxpayer, where the sum of taxable income or loss and the net loss amounts that exceeds or equal to zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl",
                            Location = "/xbrli:xbrl/tns:Tax.Losses.Net.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100031" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR19", Value = GetValueOrEmpty(report.iitrParentDocument.IITR19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR482", Value = rplossSeqNumContext.IITR482 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR487", Value = rplossSeqNumContext.IITR487.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR153", Value = report.iitrParentDocument.IITR153.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR154", Value = report.iitrParentDocument.IITR154.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR155", Value = report.iitrParentDocument.IITR155.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR156", Value = report.iitrParentDocument.IITR156.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.100031

                    #region VR.ATO.IITR.100032

                    /*  VR.ATO.IITR.100032
                    For a resident taxpayer where the sum of taxable income or loss and the net loss amounts is less than zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl

                    Legacy Rule Format:
                    IF [iitr19] = TRUE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0))) < 0 AND (ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    [iitr19] = TRUE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0))) < 0 AND (ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000

                    Data Elements:

                    ^IITR487 = RP.{LOSSSeqNum}:Tax.Losses.Net.Amount

                    ^IITR19 = RP:Residency.TaxPurposesPersonStatus.Indicator

                    ^IITR482 = RP.{LOSSSeqNum}:IncomeTax.LossesType.Code

                    ^IITR157 = RP:Income.Taxable.Amount
                    
                    ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

                    ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

                    ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

                    ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

                    ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

                    ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

                    ^sumDDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount    (was previously IITR362)

                    ^IITR153 = RP:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount

                    ^IITR154 = RP:IncomeTax.LossesClaimedPrimaryProduction.Amount

                    ^IITR155 = RP:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount

                    ^IITR156 = RP:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
                    */

                    List<IITR2019.RPLOSSSeqNumContext> lossContextsIITR482_100032 = (report.iitrParentDocument.RPLOSSSeqNumContextCollection != null ? report.iitrParentDocument.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|8)$")) : new List<IITR2019.RPLOSSSeqNumContext>());

                    assertion = (report.iitrParentDocument.IITR19 == true && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && ((report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) + (lossContextsIITR482_100032.Count > 0 ? lossContextsIITR482_100032.Sum(f => f.IITR487.GetValueOrDefault()) : 0)) < 0 && ((report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772 ? report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 - (report.iitrParentDocument.IITR153.GetValueOrDefault() - report.iitrParentDocument.IITR154.GetValueOrDefault()) - (report.iitrParentDocument.IITR155.GetValueOrDefault() - report.iitrParentDocument.IITR156.GetValueOrDefault())) >= 250000);

                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.100032",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"The income requirement for non-commercial losses has not been met",
                            LongDescription = @"For a resident taxpayer where the sum of taxable income or loss and the net loss amounts is less than zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl",
                            Location = "/xbrli:xbrl/tns:Tax.Losses.Net.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100032" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR19", Value = GetValueOrEmpty(report.iitrParentDocument.IITR19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR482", Value = rplossSeqNumContext.IITR482 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR487", Value = rplossSeqNumContext.IITR487.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR153", Value = report.iitrParentDocument.IITR153.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR154", Value = report.iitrParentDocument.IITR154.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR155", Value = report.iitrParentDocument.IITR155.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR156", Value = report.iitrParentDocument.IITR156.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.100032

                    #region VR.ATO.IITR.100033

                    /*  VR.ATO.IITR.100033
                    For a non-resident taxpayer where the sum of taxable income or loss and the net loss amounts exceeds or equal to zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl

                    Legacy Rule Format:
                    IF [iitr19] = FALSE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0)) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328]) >= 0 AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0)) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328] - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    [iitr19] = FALSE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0)) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328]) >= 0 AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0)) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328] - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000

                    Data Elements:

                    ^IITR487 = RP.{LOSSSeqNum}:Tax.Losses.Net.Amount

                    ^IITR19 = RP:Residency.TaxPurposesPersonStatus.Indicator

                    ^IITR482 = RP.{LOSSSeqNum}:IncomeTax.LossesType.Code

                    ^IITR157 = RP:Income.Taxable.Amount
                    
                    ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

                    ^IITR146 = RP:IncomeTax.Deduction.Interest.Amount

                    ^IITR147 = RP:IncomeTax.Deduction.Dividend.Amount

                    ^IITR115 = RP:Income.Interest.Gross.Amount

                    ^IITR118 = RP:Income.DividendsUnfranked.Amount

                    ^IITR119 = RP:Income.DividendsFranked.Amount

                    ^IITR120 = RP:IncomeTax.FrankingCredits.Amount

                    ^IITR328 = RP:IncomeTax.FrankingCreditsNewZealand.Amount

                    ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

                    ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

                    ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

                    ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

                    ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

                    ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount    (previously IITR362)

                    ^IITR153 = RP:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount

                    ^IITR154 = RP:IncomeTax.LossesClaimedPrimaryProduction.Amount

                    ^IITR155 = RP:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount

                    ^IITR156 = RP:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
                    */
                    //assertion = VRATOIITR100033(report, rplossSeqNumContext);

                    List<IITR2019.RPLOSSSeqNumContext> lossContextsIITR482_100033 = (report.iitrParentDocument.RPLOSSSeqNumContextCollection != null ? report.iitrParentDocument.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|8)$")) : new List<IITR2019.RPLOSSSeqNumContext>());

                    assertion = (report.iitrParentDocument.IITR19 == false && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && ((report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) + (lossContextsIITR482_100033.Count > 0 ? lossContextsIITR482_100033.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + report.iitrParentDocument.IITR146.GetValueOrDefault() + report.iitrParentDocument.IITR147.GetValueOrDefault() - report.iitrParentDocument.IITR115.GetValueOrDefault() - report.iitrParentDocument.IITR118.GetValueOrDefault() - report.iitrParentDocument.IITR119.GetValueOrDefault() - report.iitrParentDocument.IITR120.GetValueOrDefault() - report.iitrParentDocument.IITR328.GetValueOrDefault()) >= 0 && ((report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) + (lossContextsIITR482_100033.Count > 0 ? lossContextsIITR482_100033.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + (report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772 ? report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 + report.iitrParentDocument.IITR146.GetValueOrDefault() + report.iitrParentDocument.IITR147.GetValueOrDefault() - report.iitrParentDocument.IITR115.GetValueOrDefault() - report.iitrParentDocument.IITR118.GetValueOrDefault() - report.iitrParentDocument.IITR119.GetValueOrDefault() - report.iitrParentDocument.IITR120.GetValueOrDefault() - report.iitrParentDocument.IITR328.GetValueOrDefault() - (report.iitrParentDocument.IITR153.GetValueOrDefault() - report.iitrParentDocument.IITR154.GetValueOrDefault()) - (report.iitrParentDocument.IITR155.GetValueOrDefault() - report.iitrParentDocument.IITR156.GetValueOrDefault())) >= 250000);

                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.100033",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"The income requirement for non-commercial losses has not been met",
                            LongDescription = @"For a non-resident taxpayer where the sum of taxable income or loss and the net loss amounts exceeds or equal to zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl",
                            Location = "/xbrli:xbrl/tns:Tax.Losses.Net.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100033" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR19", Value = GetValueOrEmpty(report.iitrParentDocument.IITR19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR482", Value = rplossSeqNumContext.IITR482 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR487", Value = rplossSeqNumContext.IITR487.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR146", Value = report.iitrParentDocument.IITR146.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR147", Value = report.iitrParentDocument.IITR147.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR115", Value = report.iitrParentDocument.IITR115.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR118", Value = report.iitrParentDocument.IITR118.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR119", Value = report.iitrParentDocument.IITR119.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR120", Value = report.iitrParentDocument.IITR120.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR328", Value = report.iitrParentDocument.IITR328.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR153", Value = report.iitrParentDocument.IITR153.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR154", Value = report.iitrParentDocument.IITR154.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR155", Value = report.iitrParentDocument.IITR155.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR156", Value = report.iitrParentDocument.IITR156.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.100033

                    #region VR.ATO.IITR.100034

                    /*  VR.ATO.IITR.100034
                    For a non-resident taxpayer where the sum of taxable income or loss and the net loss amounts is less than zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl

                    Legacy Rule Format:
                    IF [iitr19] = FALSE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0)) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328]) < 0 AND (ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    [iitr19] = FALSE AND ANY OCCURRENCE OF([iitr482]) = SET("1","2","3","4") AND (([iitr157] - [iitr650]) + SUM(ConditionalValue([iitr482] <> SET("0","8"), [iitr487], 0)) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328]) < 0 AND (ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + ConditionalValue(COUNT(SCHEDULE = "DDCTNS") = 1, RoundDown(SUM([ddctns407]), 0), 0) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000

                    Data Elements:

                    ^IITR487 = RP.{LOSSSeqNum}:Tax.Losses.Net.Amount

                    ^IITR19 = RP:Residency.TaxPurposesPersonStatus.Indicator

                    ^IITR482 = RP.{LOSSSeqNum}:IncomeTax.LossesType.Code

                    ^IITR157 = RP:Income.Taxable.Amount
                    
                    ^IITR650 = RP:IncomeTax.AssessableFirstHomeSuperSaverReleased.Amount

                    ^IITR146 = RP:IncomeTax.Deduction.Interest.Amount

                    ^IITR147 = RP:IncomeTax.Deduction.Dividend.Amount

                    ^IITR115 = RP:Income.Interest.Gross.Amount

                    ^IITR118 = RP:Income.DividendsUnfranked.Amount

                    ^IITR119 = RP:Income.DividendsFranked.Amount

                    ^IITR120 = RP:IncomeTax.FrankingCredits.Amount

                    ^IITR328 = RP:IncomeTax.FrankingCreditsNewZealand.Amount

                    ^IITR490 = RP.Eligible:Income.FringeBenefitsReportable.Amount

                    ^IITR491 = RP.Ineligible:Income.FringeBenefitsReportable.Amount

                    ^IITR204 = RP:Tax.Losses.FinancialInvestmentNet.Amount

                    ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount

                    ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount

                    ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount    (previously IITR362)

                    ^IITR153 = RP:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount

                    ^IITR154 = RP:IncomeTax.LossesClaimedPrimaryProduction.Amount

                    ^IITR155 = RP:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount

                    ^IITR156 = RP:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
                    */
                    //assertion = VRATOIITR100034(report, rplossSeqNumContext);

                    List<IITR2019.RPLOSSSeqNumContext> lossContextsIITR482_100034 = (report.iitrParentDocument.RPLOSSSeqNumContextCollection != null ? report.iitrParentDocument.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|8)$")) : new List<IITR2019.RPLOSSSeqNumContext>());

                    assertion = (report.iitrParentDocument.IITR19 == false && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && ((report.iitrParentDocument.IITR157.GetValueOrDefault() - report.iitrParentDocument.IITR650.GetValueOrDefault()) + (lossContextsIITR482_100034.Count > 0 ? lossContextsIITR482_100034.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + report.iitrParentDocument.IITR146.GetValueOrDefault() + report.iitrParentDocument.IITR147.GetValueOrDefault() - report.iitrParentDocument.IITR115.GetValueOrDefault() - report.iitrParentDocument.IITR118.GetValueOrDefault() - report.iitrParentDocument.IITR119.GetValueOrDefault() - report.iitrParentDocument.IITR120.GetValueOrDefault() - report.iitrParentDocument.IITR328.GetValueOrDefault()) < 0 && ((report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() > 3772 ? report.iitrParentDocument.IITR490.GetValueOrDefault() + report.iitrParentDocument.IITR491.GetValueOrDefault() : 0) + report.iitrParentDocument.IITR204.GetValueOrDefault() + report.iitrParentDocument.IITR205.GetValueOrDefault() + report.iitrParentDocument.IITR201.GetValueOrDefault() + sumDDCTNS407 - (report.iitrParentDocument.IITR153.GetValueOrDefault() - report.iitrParentDocument.IITR154.GetValueOrDefault()) - (report.iitrParentDocument.IITR155.GetValueOrDefault() - report.iitrParentDocument.IITR156.GetValueOrDefault())) >= 250000);

                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.100034",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"The income requirement for non-commercial losses has not been met",
                            LongDescription = @"For a non-resident taxpayer where the sum of taxable income or loss and the net loss amounts is less than zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl",
                            Location = "/xbrli:xbrl/tns:Tax.Losses.Net.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100034" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR19", Value = GetValueOrEmpty(report.iitrParentDocument.IITR19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR482", Value = rplossSeqNumContext.IITR482 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR157", Value = report.iitrParentDocument.IITR157.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR650", Value = report.iitrParentDocument.IITR650.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR487", Value = rplossSeqNumContext.IITR487.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR146", Value = report.iitrParentDocument.IITR146.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR147", Value = report.iitrParentDocument.IITR147.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR115", Value = report.iitrParentDocument.IITR115.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR118", Value = report.iitrParentDocument.IITR118.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR119", Value = report.iitrParentDocument.IITR119.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR120", Value = report.iitrParentDocument.IITR120.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR328", Value = report.iitrParentDocument.IITR328.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR490", Value = report.iitrParentDocument.IITR490.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR491", Value = report.iitrParentDocument.IITR491.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR204", Value = report.iitrParentDocument.IITR204.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR205", Value = report.iitrParentDocument.IITR205.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR201", Value = report.iitrParentDocument.IITR201.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR153", Value = report.iitrParentDocument.IITR153.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR154", Value = report.iitrParentDocument.IITR154.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR155", Value = report.iitrParentDocument.IITR155.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR156", Value = report.iitrParentDocument.IITR156.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.100034
                }
            }

            #region VR.ATO.IITR.730107

            /* VR.ATO.IITR.730107
            When any total deduction amounts above zero are present, the details must be provided in the Deductions schedule

            Legacy Rule Format:
            IF ([iitr134] > 0 OR [iitr136] > 0 OR [iitr137] > 0 OR [iitr139] > 0 OR [iitr141] > 0 OR [iitr145] > 0 OR [iitr146] > 0 OR [iitr147] > 0 OR [iitr148] > 0 OR [iitr502] > 0 OR [iitr503] > 0 OR [iitr504] > 0 OR [iitr366] > 0 OR [iitr368] > 0) AND COUNT(SCHEDULE = "DDCTNS") = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            */

            assertion = (report.iitrParentDocument.IITR134.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR136.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR137.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR139.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR141.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR145.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR146.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR147.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR148.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR502.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR503.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR504.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR366.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR368.GetValueOrDefault() > 0) && ddctnsScheduleCount == 0;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.730107",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Deductions schedule must be provided",
                    LongDescription = @"When any total deduction amounts above zero are present, the details must be provided in the Deductions schedule",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730107" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.730107

            #endregion // Foreach loop

            #region DDCTNS Child Present
            DDCTNS2019 ddctnsChildDocument = null;

            if (ddctnsScheduleCount == 1)
            {
                ddctnsChildDocument = report.ddctnsChildDocuments[0];

                #region VR.ATO.IITR.600031

                /* VR.ATO.IITR.600031
                The work related self-education expenses action code is present and the amount of work related self-education expenses is missing.

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [ddctns200] <> NULL AND [iitr139] = NULL
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR139 =     tns:IncomeTax.Deduction.EducationNet.Amount
                DDCTNS:RP:^DDCTNS200 = tns:IncomeTax.Deduction.EducationCircumstances.Code
                */
                assertion = (ddctnsChildDocument.DDCTNS200 != null && report.iitrParentDocument.IITR139 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.601031",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related self-education expenses amount must be provided",
                        LongDescription = @"When the Self education reason is present on the attached Deductions schedule, the Work related self-education expenses amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationNet.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600031" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS200", Value = ddctnsChildDocument.DDCTNS200 });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600031

                #region VR.ATO.IITR.700304
                /* VR.ATO.IITR.700304
                When the taxpayer is over the age of 76 on the 1st June for the Year of return, Personal Superannuation Contributions must not be provided on the Deductions schedule

                Legacy Rule Format
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND ([iitr29] <> NULL AND [iitr29] < ConvertToDate(1, 6, [iitr10] - 76)) AND RoundDown(SUM([ddctns407]), 0) > 0
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR10 =                   tns:Report.TargetFinancial.Year
                IITR:RP:^IITR29 =                   tns:PersonDemographicDetails.Birth.Date
                DDCTNS:RP.{SCSeqNum}
                DDCTNS:RP.{SCSeqNum}:^DDCTNS407 =   tns:IncomeTax.Deduction.SuperannuationContribution.Amount
                */
                assertion = report.iitrParentDocument.IITR29.HasValue && report.iitrParentDocument.IITR10.HasValue
                            && report.iitrParentDocument.IITR29.Value < new DateTime(report.iitrParentDocument.IITR10.Value - 76, 6, 1)
                            && Count(ddctnsChildDocument.RPSCSeqNumContextCollection) > 0 && sumDDCTNS407 > 0;
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.700304",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Personal Superannuation Contributions must not be provided on the Deductions schedule",
                        LongDescription = @"When the taxpayer is over the age of 76 on the 1st June for the Year of return, an amount above zero must not be provided on the Deductions schedule",
                        Location = "/xbrli:xbrl/tns:PersonDemographicDetails.Birth.Date[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.700304" } }
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter()
                    { Name = "IITR10", Value = GetValueOrEmpty(report.iitrParentDocument.IITR10) });

                    processMessage.Parameters.Add(new ProcessMessageParameter()
                    { Name = "IITR29", Value = GetValueOrEmpty(report.iitrParentDocument.IITR29) });

                    processMessage.Parameters.Add(new ProcessMessageParameter()
                    { Name = "Sum(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                    response.Add(processMessage);
                }
                #endregion // VR.ATO.IITR.700304

                #region VR.ATO.IITR.730000
                /* VR.ATO.IITR.730000
                When work related car expense details are present in the attached Deductions schedule, the Work related car expenses amount must be provided

                Legacy Rule Format:
                IF [iitr134] = NULL AND COUNT(SCHEDULE = ""DDCTNS"") = 1 AND COUNT(DDCTNS:RP.{CarSeqNum}) > 0
                    RETURN VALIDATION MESSAGE
                ENDIF


                Data Elements:

                IITR:RP:^IITR134      = tns:IncomeTax.Deduction.Car.Amount
                DDCTNS:RP.{CarSeqNum}
                */
                assertion = ((Count(ddctnsChildDocument.RPCarSeqNumContextCollection) > 0) && report.iitrParentDocument.IITR134 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related car expenses amount must be provided",
                        LongDescription = @"When work related car expense details are present in the attached Deductions schedule, the Work related car expenses amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Car.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730000" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR134", Value = "null" });

                    response.Add(processMessage);
                }
                #endregion

                #region VR.ATO.IITR.730001
                /* VR.ATO.IITR.730001
                The Work related car expenses must equal the sum of all car expense amounts claimed in the attached Deductions schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = ""DDCTNS"") = 1) AND ([iitr134] <> NULL AND [iitr134] <> RoundDown(((SUM(([ddctns105] + [ddctns106]) * ([ddctns104] / 100))) + (SUM([ddctns103]) * 0.68)), 0))
                    RETURN VALIDATION MESSAGE
                ENDIF
                
                Data Elements:

                IITR:RP:^IITR134      = tns:IncomeTax.Deduction.Car.Amount
                DDCTNS:RP.{CarSeqNum}
                */
                decimal vr730001_LogbookSum = 0;
                decimal vr730001_CentsPerKmSum = 0;
                decimal vr730001_TotalSum = 0;
                if (ddctnsChildDocument.RPCarSeqNumContextCollection != null)
                {
                    foreach (DDCTNS2019.RPCarSeqNumContext context in ddctnsChildDocument.RPCarSeqNumContextCollection)
                    {
                        vr730001_LogbookSum += ((context.DDCTNS105.GetValueOrDefault() + context.DDCTNS106.GetValueOrDefault()) * (context.DDCTNS104.GetValueOrDefault() / 100));
                        vr730001_CentsPerKmSum += (context.DDCTNS103.GetValueOrDefault() * 0.68M);
                    }
                    vr730001_TotalSum = Math.Truncate(vr730001_LogbookSum + vr730001_CentsPerKmSum);
                }
                assertion = (report.iitrParentDocument.IITR134 != null && report.iitrParentDocument.IITR134.GetValueOrDefault() != vr730001_TotalSum);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related car expenses amount is incorrect",
                        LongDescription = @"The Work related car expenses must equal the sum of all car expense amounts claimed in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Car.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730001" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR134", Value = report.iitrParentDocument.IITR134.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Car Logbook Total", Value = vr730001_LogbookSum.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Car Cents Per Km Total", Value = vr730001_CentsPerKmSum.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Car Deductions Total", Value = vr730001_TotalSum.ToString() });

                    response.Add(processMessage);
                }
                #endregion

                #region VR.ATO.IITR.730002

                /* VR.ATO.IITR.730002
                When Dividend deductions expense details are present on the attached Deductions schedule, the Dividend deductions amount must be provided

                Legacy Rule Format:
                IF [iitr147] = NULL AND COUNT(SCHEDULE = ""DDCTNS"") = 1 AND COUNT(DDCTNS:RP.{DivSeqNum}) > 0
                    RETURN VALIDATION MESSAGE
                ENDIF


                Data Elements:

                IITR:RP:^IITR147 = tns:IncomeTax.Deduction.Dividend.Amount
                */
                assertion = (report.iitrParentDocument.IITR147 == null && Count(ddctnsChildDocument.RPDivDedSeqNumContextCollection) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730002",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Dividend deductions amount must be provided",
                        LongDescription = @"When Dividend deductions expense details are present on the attached Deductions schedule, the Dividend deductions amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Dividend.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730002" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR147", Value = "null" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730002

                #region VR.ATO.IITR.730003

                /* VR.ATO.IITR.730003
                The Dividend deductions amount must equal the sum of all Dividend deduction amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = ""DDCTNS"") = 1 AND [iitr147] <> NULL AND [iitr147] <> RoundDown(SUM([ddctns132]), 0)
                    RETURN VALIDATION MESSAGE
                ENDIF


                Data Elements:

                IITR:RP:^IITR147 =                    tns:IncomeTax.Deduction.Dividend.Amount
                DDCTNS:RP.{DivDedSeqNum}:^DDCTNS132 = tns:IncomeTax.Deduction.Dividend.Amount
                */
                assertion = (report.iitrParentDocument.IITR147 != null && (Count(ddctnsChildDocument.RPDivDedSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPDivDedSeqNumContextCollection.Sum(context => context.DDCTNS132.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR147.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730003",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Dividend deductions amount is incorrect",
                        LongDescription = @"The Dividend deductions amount must equal the sum of all Dividend deduction amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Dividend.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730003" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR147", Value = report.iitrParentDocument.IITR147.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS132", Value = (Count(ddctnsChildDocument.RPDivDedSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPDivDedSeqNumContextCollection.Sum(context => context.DDCTNS132.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730003

                #region VR.ATO.IITR.730100

                /* VR.ATO.IITR.730100
                When project pool details are present in the attached Deductions schedule, the 'Individual - Deduction for project pool' amount must be provided

                Legacy Rule Format:
                IF [iitr366] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND COUNT(DDCTNS:RP.{ProjPoolSeqNum}) > 0
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR366 =     tns:IncomeTax.Deduction.ProjectPool.Amount
                */
                assertion = (report.iitrParentDocument.IITR366 == null && (ddctnsChildDocument.RPProjPoolSeqNumContextCollection != null && ddctnsChildDocument.RPProjPoolSeqNumContextCollection.Count() > 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730100",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Individual - Deduction for project pool amount must be provided",
                        LongDescription = @"When project pool details are present in the attached Deductions schedule, the 'Individual - Deduction for project pool' amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.ProjectPool.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730100" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR366", Value = report.iitrParentDocument.IITR366.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730100

                #region VR.ATO.IITR.730101

                /* VR.ATO.IITR.730101
                The 'Individual - Deduction for project pool' amount must equal the sum of all Project pool amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND ([iitr366] <> NULL AND [iitr366] <> RoundDown(SUM([ddctns212]), 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR366 =     tns:IncomeTax.Deduction.ProjectPool.Amount
                */

                // Calculate the sum of the granular project pool amounts (if a project pool context exists, add each occurance of DDCTNS212. If not, sum equals 0)
                decimal sumDDCTNS212 = Count(ddctnsChildDocument.RPProjPoolSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPProjPoolSeqNumContextCollection.Sum(context => context.DDCTNS212.GetValueOrDefault())) : 0;

                assertion = (report.iitrParentDocument.IITR366 != null && sumDDCTNS212 != report.iitrParentDocument.IITR366.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730101",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Individual - Deduction for project pool amount is incorrect",
                        LongDescription = @"The 'Individual - Deduction for project pool' amount must equal the sum of all Project pool amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.ProjectPool.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730101" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR366", Value = report.iitrParentDocument.IITR366.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730101

                #region VR.ATO.IITR.730102

                /* VR.ATO.IITR.730102
                The TFN provided for the Deductions schedule must be the same as the TFN of the parent form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND (DDCTNS:RP:entity.identifier.TFN <> RP:entity.identifier.TFN)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (ddctnsChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730102",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax file number invalid",
                        LongDescription = @"The TFN provided for the Deductions schedule must be the same as the TFN of the parent form",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + ddctnsChildDocument.RPId + "']/xbrli:entity/xbrli:identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730102" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730102

                #region VR.ATO.IITR.730103

                /* VR.ATO.IITR.730103
                Invalid context. Period start date for the Deductions schedule must be the same as the Period start date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND (DDCTNS:RP:period.startDate <> RP:period.startDate)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (ddctnsChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730103",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period start date for the Deductions schedule is invalid",
                        LongDescription = @"Invalid context. Period start date for the Deductions schedule must be the same as the Period start date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + ddctnsChildDocument.RPId + "']/xbrli:period/xbrli:startDate",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730103" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730103

                #region VR.ATO.IITR.730104

                /* VR.ATO.IITR.730104
                Invalid context. Period end date for the Deductions schedule must be the same as the Period end date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND (DDCTNS:RP:period.endDate <> RP:period.endDate)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (ddctnsChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730104",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period end date for the Deductions schedule is invalid",
                        LongDescription = @"Invalid context. Period end date for the Deductions schedule must be the same as the Period end date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + ddctnsChildDocument.RPId + "']/xbrli:period/xbrli:endDate",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730104" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730104

                #region VR.ATO.IITR.730106

                /* VR.ATO.IITR.730106
                The Work related self education expenses must not exceed the sum of all self education amounts and self education car expenses claimed in the attached Deductions schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND ([iitr139] <> NULL AND [iitr139] > RoundDown((SUM([ddctns204]) + (SUM(([ddctns124] + [ddctns125]) * ([ddctns123] / 100))) + (SUM([ddctns122]) * 0.68)), 0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                
                Data Elements:

                IITR:RP:^IITR1349     = tns:IncomeTax.Deduction.EducationNet.Amount
                DDCTNS:RP.{EduSeqNum}
                */
                decimal vr730106_LogbookSum = 0;
                decimal vr730106_CentsPerKmSum = 0;
                decimal vr730106_SelfEduAmountSum = 0;
                decimal vr730106_TotalSum = 0;
                if (ddctnsChildDocument.RPEduSeqNumContextCollection != null)
                {
                    foreach (DDCTNS2019.RPEduSeqNumContext context in ddctnsChildDocument.RPEduSeqNumContextCollection)
                    {
                        vr730106_SelfEduAmountSum += (context.DDCTNS204.GetValueOrDefault());
                        vr730106_LogbookSum += ((context.DDCTNS124.GetValueOrDefault() + context.DDCTNS125.GetValueOrDefault()) * (context.DDCTNS123.GetValueOrDefault() / 100));
                        vr730106_CentsPerKmSum += (context.DDCTNS122.GetValueOrDefault() * 0.68M);
                    }
                    vr730106_TotalSum = Math.Truncate(vr730106_LogbookSum + vr730106_CentsPerKmSum + vr730106_SelfEduAmountSum);
                }
                assertion = (report.iitrParentDocument.IITR139.HasValue && report.iitrParentDocument.IITR139.Value > vr730106_TotalSum);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730106",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related self-education expenses amount is incorrect",
                        LongDescription = @"The Work related self education expenses must not exceed the sum of all self education amounts and self education car expenses claimed in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationNet.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730106" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Self Education Car Logbook Total", Value = vr730106_LogbookSum.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Self Education Car Cents Per Km Total", Value = vr730106_CentsPerKmSum.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Self Education Expense Total", Value = vr730106_SelfEduAmountSum.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Self Education Deductions Total", Value = vr730106_TotalSum.ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730106

                #region VR.ATO.IITR.730108

                /*  VR.ATO.IITR.730108
                The sum of Superannuation deduction claimed amounts is the same as the amount provided for Reportable employer superannuation contributions. A deduction for contributions paid by the employer from before-tax income (including the compulsory super guarantee and salary sacrifice amounts) is not claimable, and the return may be adjusted. Check and correct if necessary.

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr201] > 0 AND (ANY OCCURRENCE OF(RoundDown([ddctns407], 0)) = [iitr201] OR [iitr201] = RoundDown(SUM([ddctns407]), 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Technical Business Rule Format:
                COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr201] > 0 AND (ANY OCCURRENCE OF(RoundDown([ddctns407], 0)) = [iitr201] OR [iitr201] = RoundDown(SUM([ddctns407]), 0))

                Data Elements:

                ^DDCTNS407 = rvctc2.02.14:IncomeTax.Deduction.SuperannuationContribution.Amount

                ^IITR201 = RP:SuperannuationContribution.EmployerReportable.Amount
                */
                assertion = report.iitrParentDocument.IITR201.GetValueOrDefault() > 0 && ((report.ddctnsChildDocuments[0].RPSCSeqNumContextCollection != null ? report.ddctnsChildDocuments[0].RPSCSeqNumContextCollection.Any(f => Math.Truncate(f.DDCTNS407.GetValueOrDefault()) == report.iitrParentDocument.IITR201.GetValueOrDefault()) : false) || report.iitrParentDocument.IITR201.GetValueOrDefault() == sumDDCTNS407);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730108",
                        Severity = ProcessMessageSeverity.Warning,
                        Description = @"Check personal superannuation deductions claimed",
                        LongDescription = @"Either one or the sum of Superannuation deduction claimed amounts is the same as the amount provided for Reportable employer superannuation contributions. A deduction for contributions paid by the employer from before-tax income (including the compulsory super guarantee and salary sacrifice amounts) is not claimable, and the return may be adjusted. Check and correct if necessary.",
                        Location = "/xbrli:xbrl/tns:SuperannuationContribution.EmployerReportable.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730108" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR201", Value = GetValueOrEmpty(report.iitrParentDocument.IITR201) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(DDCTNS407)", Value = sumDDCTNS407.ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730108

                #region VR.ATO.IITR.730200

                /* VR.ATO.IITR.730200
                The Cost of managing tax affairs - Interest charged by the ATO amount must equal the sum of all Interest charged by the ATO amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr502] <> NULL AND RoundDown(SUM(ConditionalValue([ddctns304] = "Interest charged by the ATO", [ddctns306], 0)), 0) <> [iitr502]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR502 =                        tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS306 = tns:IncomeTax.Deduction.TaxAffairManagement.Amount
                */
                assertion = (report.iitrParentDocument.IITR502 != null && (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Where(context => context.DDCTNS304.Equals("Interest charged by the ATO")).Sum(context => context.DDCTNS306.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR502.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730200",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Cost of managing tax affairs - Interest charged by the ATO amount is incorrect",
                        LongDescription = @"The Cost of managing tax affairs - Interest charged by the ATO amount must equal the sum of all Interest charged by the ATO amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730200" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR502", Value = report.iitrParentDocument.IITR502.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS304", Value = "Interest charged by the ATO" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS306", Value = (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Where(context => context.DDCTNS304.Equals("Interest charged by the ATO")).Sum(context => context.DDCTNS306.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730200

                #region VR.ATO.IITR.730201

                /* VR.ATO.IITR.730201
                The Cost of managing tax affairs - Litigation costs amount must equal the sum of all Litigation costs amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr503] <> NULL AND RoundDown(SUM(ConditionalValue([ddctns304] = "Litigation costs", [ddctns306], 0)), 0) <> [iitr503]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR503 =                        tns:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS306 = tns:IncomeTax.Deduction.TaxAffairManagement.Amount
                */
                assertion = (report.iitrParentDocument.IITR503 != null && (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Where(context => context.DDCTNS304.Equals("Litigation costs")).Sum(context => context.DDCTNS306.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR503.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730201",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Cost of managing tax affairs - Litigation costs amount is incorrect",
                        LongDescription = @"The Cost of managing tax affairs - Litigation costs amount must equal the sum of all Litigation costs amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730201" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR503", Value = report.iitrParentDocument.IITR503.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS304", Value = "Litigation costs" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS306", Value = (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Where(context => context.DDCTNS304.Equals("Litigation costs")).Sum(context => context.DDCTNS306.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730201

                #region VR.ATO.IITR.730202

                /* VR.ATO.IITR.730202
                The Cost of managing tax affairs - Other expenses incurred in managing your tax affairs amount must equal the sum of all Other expenses incurred in managing your tax affairs in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr504] <> NULL AND RoundDown(SUM(ConditionalValue([ddctns304] = "Other expenses incurred in managing your tax affairs", [ddctns306], 0)), 0) <> [iitr504]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR504 =                        tns:IncomeTax.Deduction.TaxAffairManagementOther.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS306 = tns:IncomeTax.Deduction.TaxAffairManagement.Amount
                */
                assertion = (report.iitrParentDocument.IITR504 != null && (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Where(context => context.DDCTNS304.Equals("Other expenses incurred in managing your tax affairs")).Sum(context => context.DDCTNS306.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR504.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730202",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Cost of managing tax affairs - Other expenses amount is incorrect",
                        LongDescription = @"The Cost of managing tax affairs - Other expenses incurred in managing your tax affairs amount must equal the sum of all Other expenses incurred in managing your tax affairs in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730202" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR504", Value = report.iitrParentDocument.IITR504.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS304", Value = "Litigation costs" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS306", Value = (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Where(context => context.DDCTNS304.Equals("Other expenses incurred in managing your tax affairs")).Sum(context => context.DDCTNS306.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730202

                #region VR.ATO.IITR.730203

                /* VR.ATO.IITR.730203
                When Cost of managing tax affairs - Interest charged by the ATO details are present in the attached Deductions schedule, the Cost of managing tax affairs - Interest charged by the ATO amount must be provided

                Legacy Rule Format:
                IF [iitr502] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND ANY OCCURRENCE OF([ddctns304] = "Interest charged by the ATO")
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR502 =                        tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                */
                assertion = (report.iitrParentDocument.IITR502 == null && (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Any(context => context.DDCTNS304.Equals("Interest charged by the ATO")) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730203",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Cost of managing tax affairs - Interest charged by the ATO amount must be provided",
                        LongDescription = @"When Cost of managing tax affairs - Interest charged by the ATO details are present in the attached Deductions schedule, the Cost of managing tax affairs - Interest charged by the ATO amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730203" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR502", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS304", Value = "Interest charged by the ATO" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730203

                #region VR.ATO.IITR.730204

                /* VR.ATO.IITR.730204
                When Cost of managing tax affairs - Litigation costs details are present in the attached Deductions schedule, the Cost of managing tax affairs - Litigation costs amount must be provided

                Legacy Rule Format:
                IF [iitr503] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND ANY OCCURRENCE OF([ddctns304] = "Litigation costs")
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR503 =                        tns:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                */
                assertion = (report.iitrParentDocument.IITR503 == null && (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Any(context => context.DDCTNS304.Equals("Litigation costs")) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730204",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Cost of managing tax affairs - Litigation costs amount must be provided",
                        LongDescription = @"When Cost of managing tax affairs - Litigation costs details are present in the attached Deductions schedule, the Cost of managing tax affairs - Litigation costs amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730204" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR503", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS304", Value = "Litigation costs" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730204

                #region VR.ATO.IITR.730205

                /* VR.ATO.IITR.730205
                When Cost of managing tax affairs - Other expenses details are present in the attached Deductions schedule, the Cost of managing tax affairs - Other expenses amount must be provided

                Legacy Rule Format:
                IF [iitr504] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND ANY OCCURRENCE OF([ddctns304] = "Other expenses incurred in managing your tax affairs")
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR504 =                        tns:IncomeTax.Deduction.TaxAffairManagementOther.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                */
                assertion = (report.iitrParentDocument.IITR504 == null && (Count(ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPMangTaxAffSeqNumContextCollection.Any(context => context.DDCTNS304.Equals("Other expenses incurred in managing your tax affairs")) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730205",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Cost of managing tax affairs - Other expenses amount must be provided",
                        LongDescription = @"When Cost of managing tax affairs - Other expenses details are present in the attached Deductions schedule, the Cost of managing tax affairs - Other expenses amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730205" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR504", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS304", Value = "Other expenses incurred in managing your tax affairs" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730205

                #region VR.ATO.IITR.730206

                /* VR.ATO.IITR.730206
                Forestry managed investment scheme deduction amount must equal the sum of all Forestry managed investment amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr368] <> NULL AND [iitr368] <> RoundDown(SUM([ddctns309]), 0)
                     RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR368 =                        tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS309 = tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
                */
                assertion = (report.iitrParentDocument.IITR368 != null && (Count(ddctnsChildDocument.RPForestManInvSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPForestManInvSeqNumContextCollection.Sum(context => context.DDCTNS309.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR368.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730206",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Forestry managed investment scheme deduction amount is incorrect",
                        LongDescription = @"Forestry managed investment scheme deduction amount must equal the sum of all Forestry managed investment amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730206" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR368", Value = report.iitrParentDocument.IITR368.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS309", Value = (Count(ddctnsChildDocument.RPForestManInvSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPForestManInvSeqNumContextCollection.Sum(context => context.DDCTNS309.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730206

                #region VR.ATO.IITR.730207

                /* VR.ATO.IITR.730207
                When Forestry managed investment scheme deduction details are present in the attached Deductions schedule, the Forestry managed investment scheme deduction amount must be provided

                Legacy Rule Format:
                IF [iitr368] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND COUNT(DDCTNS:RP.{ForestManInvSeqNum}) > 0
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR368 = tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
                */
                assertion = (report.iitrParentDocument.IITR368 == null && Count(ddctnsChildDocument.RPForestManInvSeqNumContextCollection) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730207",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Forestry managed investment scheme deduction amount must be provided",
                        LongDescription = @"When Forestry managed investment scheme deduction details are present in the attached Deductions schedule, the Forestry managed investment scheme deduction amount must be provided",
                        Location = "/xbrli:xbrl/tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730207" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR368", Value = "null" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730207

                #region VR.ATO.IITR.730208

                /* VR.ATO.IITR.730208
                Low value pool deduction must equal the sum of all Low value pool deduction component amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr145] <> NULL AND ([iitr145] <> RoundDown([ddctns310] + [ddcnts311] + [ddctns312]), 0)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR145 =                        tns:IncomeTax.Deduction.LowValuePool.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS310 = tns:IncomeTax.Deduction.LowValuePoolFinancialInvestments.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS311 = tns:IncomeTax.Deduction.LowValuePoolRentalProperties.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS312 = tns:IncomeTax.Deduction.LowValuePoolOther.Amount
                */
                assertion = (report.iitrParentDocument.IITR145 != null && (Math.Truncate(ddctnsChildDocument.DDCTNS310.GetValueOrDefault() + ddctnsChildDocument.DDCTNS311.GetValueOrDefault() + ddctnsChildDocument.DDCTNS312.GetValueOrDefault())) != report.iitrParentDocument.IITR145.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730208",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Low value pool deduction amount is incorrect",
                        LongDescription = @"The Low value pool deduction amount must equal the sum of all Low value pool deduction component amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.LowValuePool.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730208" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR145", Value = report.iitrParentDocument.IITR145.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS310", Value = ddctnsChildDocument.DDCTNS310.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS311", Value = ddctnsChildDocument.DDCTNS311.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS312", Value = ddctnsChildDocument.DDCTNS312.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730208

                #region VR.ATO.IITR.730209

                /* VR.ATO.IITR.730209
                When Forestry managed investment scheme deduction details are present in the attached Deductions schedule, the Forestry managed investment scheme deduction amount must be provided

                Legacy Rule Format:
                IF [iitr145] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND (([ddcnts310] <> NULL) OR ([ddctns311] <> NULL) OR ([ddctns312] <> NULL))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR502 =                        tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount
                DDCTNS:RP.{MangTaxAffSeqNum}:^DDCTNS304 = tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
                */
                assertion = (report.iitrParentDocument.IITR145 == null && (ddctnsChildDocument.DDCTNS310 != null || ddctnsChildDocument.DDCTNS311 != null || ddctnsChildDocument.DDCTNS312 != null));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730209",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Low value pool deduction must be provided",
                        LongDescription = @"When Low value pool deduction component amounts are present in the attached Deductions Schedule, the Low value pool deduction amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.LowValuePool.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730209" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR145", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS310", Value = (ddctnsChildDocument.DDCTNS310 != null ? ddctnsChildDocument.DDCTNS310.GetValueOrDefault().ToString() : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS311", Value = (ddctnsChildDocument.DDCTNS311 != null ? ddctnsChildDocument.DDCTNS311.GetValueOrDefault().ToString() : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS312", Value = (ddctnsChildDocument.DDCTNS312 != null ? ddctnsChildDocument.DDCTNS312.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730209

                #region VR.ATO.IITR.730210

                /* VR.ATO.IITR.730210
                Other work related expenses amount must equal the sum of all Other work related expenses amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr141] <> NULL AND [iitr141] <> RoundDown(SUM([ddctns315]), 0)
                     RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR141 =                   tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                DDCTNS:RP.{OTHWRSeqNum}:^DDCTNS315 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                */
                assertion = (report.iitrParentDocument.IITR141 != null && (Count(ddctnsChildDocument.RPOTHWRSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPOTHWRSeqNumContextCollection.Sum(context => context.DDCTNS315.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR141.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730210",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other work related expenses amount is incorrect",
                        LongDescription = @"Other work related expenses amount must equal the sum of all Other work related expenses amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.WorkRelatedOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730210" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR141", Value = report.iitrParentDocument.IITR141.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS315", Value = (Count(ddctnsChildDocument.RPForestManInvSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPForestManInvSeqNumContextCollection.Sum(context => context.DDCTNS309.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730210

                #region VR.ATO.IITR.730211

                /* VR.ATO.IITR.730211
                When Forestry managed investment scheme deduction details are present in the attached Deductions schedule, the Forestry managed investment scheme deduction amount must be provided

                Legacy Rule Format:
                IF [iitr141] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND COUNT(DDCTNS:RP.{OTHWRSeqNum}) > 0
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR141 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                */
                assertion = (report.iitrParentDocument.IITR141 == null && Count(ddctnsChildDocument.RPOTHWRSeqNumContextCollection) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730211",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other work related expenses amount must be provided",
                        LongDescription = @"When Other work related expenses details are present in the attached Deductions schedule, the Other work related expenses amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.WorkRelatedOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730211" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR141", Value = "null" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730211

                #region VR.ATO.IITR.730300
                /* VR.ATO.IITR.730300
                The Work related travel expenses amount must equal the sum of all Work Related Travel Amount amounts on the attached Deductions schedule.

                Legacy Rule Format
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND (([iitr136] <> NULL) AND ([iitr136] <> RoundDown(SUM([ddctns410]),0)))
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR136 =                  tns:IncomeTax.Deduction.Travel.Amount
                DDCTNS:RP.{TrvlSeqNum}:^DDCTNS410 = tns:IncomeTax.Deduction.Travel.Amount
                */
                if (Count(ddctnsChildDocument.RPTrvlSeqNumContextCollection) > 0)
                {
                    decimal sumOfTravelExpenses = Math.Truncate(ddctnsChildDocument.RPTrvlSeqNumContextCollection.Sum(context => context.DDCTNS410 ?? 0));

                    assertion = report.iitrParentDocument.IITR136.HasValue && report.iitrParentDocument.IITR136.Value != sumOfTravelExpenses;
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.730300",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Work related travel expenses amount is incorrect",
                            LongDescription = @"The Work related travel expenses amount must be equal to the sum of the Work Related Travel Amount in the attached Deduction schedule",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Travel.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730300" } }
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter()
                        { Name = "IITR136", Value = report.iitrParentDocument.IITR136.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter()
                        { Name = "DDCTNS410", Value = sumOfTravelExpenses.ToString() });

                        response.Add(processMessage);
                    }
                }
                #endregion //VR.ATO.IITR.730300

                #region VR.ATO.IITR.730304
                /* VR.ATO.IITR.730304
                When work related travel expense details are present in the attached Deductions schedule, the Work related travel expenses amount must be provided

                Legacy Rule Format
                IF [iitr136] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND COUNT(DDCTNS:RP.{TrvlseqNum}) > 0
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR136 =      tns:IncomeTax.Deduction.Travel.Amount
                DDCTNS:RP.{TrvlSeqNum}
                */
                assertion = !report.iitrParentDocument.IITR136.HasValue && Count(ddctnsChildDocument.RPTrvlSeqNumContextCollection) > 0;
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730304",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related travel expenses amount must be provided",
                        LongDescription = @"When work related travel expense details are present in the attached Deductions schedule, the Work related travel expenses amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Travel.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730304" } }
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter()
                    { Name = "IITR136", Value = "null" });

                    response.Add(processMessage);
                }
                #endregion //VR.ATO.IITR.730304

                #region VR.ATO.IITR.730212

                /* VR.ATO.IITR.730212
                The Interest deductions amount must equal the sum of all Interest deduction amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "DDCTNS") = 1 AND [iitr146] <> NULL AND [iitr146] <> RoundDown(SUM([ddctns318]), 0)
                     RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR146 = tns:IncomeTax.Deduction.Interest.Amount
                DDCTNS:RP.{IntDedSeqNum}:^DDCTNS318 = tns:IncomeTax.Deduction.Interest.Amount
                */
                assertion = (report.iitrParentDocument.IITR146 != null && (Count(ddctnsChildDocument.RPIntDedSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPIntDedSeqNumContextCollection.Sum(context => context.DDCTNS318.GetValueOrDefault())) : 0) != report.iitrParentDocument.IITR146.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730212",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Interest deductions amount is incorrect",
                        LongDescription = @"The Interest deductions amount must equal the sum of all Interest deduction amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Interest.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730212" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR146", Value = report.iitrParentDocument.IITR146.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS318", Value = (Count(ddctnsChildDocument.RPIntDedSeqNumContextCollection) > 0 ? ddctnsChildDocument.RPIntDedSeqNumContextCollection.Sum(context => context.DDCTNS318.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730212

                #region VR.ATO.IITR.730213

                /* VR.ATO.IITR.730213
                When Interest deduction are present in the attached Deductions schedule, the Interest deductions amount must be provided

                Legacy Rule Format:
                IF [iitr146] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND COUNT(DDCTNS:RP.{IntDedSeqNum}) > 0
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR146 = tns:IncomeTax.Deduction.Interest.Amount
                */
                assertion = (report.iitrParentDocument.IITR146 == null && Count(ddctnsChildDocument.RPIntDedSeqNumContextCollection) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730213",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Interest deductions amount must be provided",
                        LongDescription = @"When Interest deduction details are present in the attached Deductions schedule, the Interest deductions amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Interest.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730213" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR146", Value = "null" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730213

                #region VR.ATO.IITR.730305

                /* VR.ATO.IITR.730305
                The Gifts or donations amount must equal the sum of all Gifts or donations amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) AND (([iitr148] <> NULL) AND ([iitr148] <> RoundDown(SUM([ddctns413]),0)))
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR148 =     tns:IncomeTax.Deduction.GiftDonation.Amount
                DDCTNS:RP.{GiftDonSeqNum}:^DDCTNS413 = IncomeTax.Deduction.GiftDonation.Amount
                */

                // Calculate the sum of the granular Gifts or donations amounts (if a Gifts or donations context exists, add each Occurrence of ddctns413. If not, sum equals 0)
                decimal sumddctns413 = Count(ddctnsChildDocument.RPGiftDonSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPGiftDonSeqNumContextCollection.Sum(context => context.DDCTNS413.GetValueOrDefault())) : 0;

                assertion = (report.iitrParentDocument.IITR148 != null && sumddctns413 != report.iitrParentDocument.IITR148.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730305",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gifts or donations amount is incorrect",
                        LongDescription = @"The Gifts or donations amount must equal the sum of all Gifts or donations amounts in the attached Deductions schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.GiftDonation.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730305" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR148", Value = report.iitrParentDocument.IITR148.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS413", Value = Count(ddctnsChildDocument.RPGiftDonSeqNumContextCollection) > 0 ? Math.Truncate(ddctnsChildDocument.RPGiftDonSeqNumContextCollection.Sum(context => context.DDCTNS413.GetValueOrDefault())).ToString() : "0" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730305

                #region VR.ATO.IITR.730306

                /* VR.ATO.IITR.730306
                When gifts or donations details are present in the attached Deductions schedule, the Gifts or donations amount must be provided

                Legacy Rule Format:
                IF [iitr148] = NULL AND COUNT(SCHEDULE = "DDCTNS") = 1 AND COUNT(DDCTNS:RP.{GiftDonSeqNum}) > 0
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR148 = tns:rvctc2.02.14:IncomeTax.Deduction.GiftDonation.Amount
                */
                assertion = (report.iitrParentDocument.IITR148 == null && Count(ddctnsChildDocument.RPGiftDonSeqNumContextCollection) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730306",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gifts or donations amount must be provided",
                        LongDescription = @"When gifts or donations details are present in the attached Deductions schedule, the Gifts or donations amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.GiftDonation.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730306" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR148", Value = "null" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730306

                #region VR.ATO.IITR.730307

                /* VR.ATO.IITR.730307
                When work related uniform, occupation specific or protective clothing, laundry and dry cleaning expense details are present in the attached
                Deductions schedule, the Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses amount must be provided

                Legacy Rule Format:
                IF ([iitr137] = NULL) AND (COUNT(SCHEDULE = "DDCTNS") = 1) AND COUNT(DDCTNS:RP.{ClthngSeqNum}) > 0
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR137 = tns:IncomeTax.Deduction.Clothing.Amount
                DDCTNS:RP.{ClthngSeqNum}
                */
                assertion = (!report.iitrParentDocument.IITR137.HasValue && Count(ddctnsChildDocument.RPClthngSeqNumContextCollection) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730307",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses must be provided",
                        LongDescription = @"When work related uniform, occupation specific or protective clothing, laundry and dry cleaning expense details are present in the attached Deductions schedule, the Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses amount must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Clothing.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730307" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR137", Value = "null" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730307

                #region VR.ATO.IITR.730308

                /* VR.ATO.IITR.730308
                The Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses amount must equal the sum of all
                Work related clothing expenses amounts in the attached Deductions schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "DDCTNS") = 1) 
                AND (([iitr137] <> NULL) AND ([iitr137] <> RoundDown(SUM([ddctns416]),0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR137 =                      tns:IncomeTax.Deduction.Clothing.Amount
                DDCTNS:RP.{ClthngSeqNum}:^DDCTNS416 =   ddctns:IncomeTax.Deduction.Clothing.Amount
                */
                if (Count(ddctnsChildDocument.RPClthngSeqNumContextCollection) > 0)
                {
                    decimal sumDDCTNS416 = Math.Truncate(ddctnsChildDocument.RPClthngSeqNumContextCollection.Sum(context => context.DDCTNS416 ?? 0));
                    assertion = report.iitrParentDocument.IITR137.HasValue && report.iitrParentDocument.IITR137.Value != sumDDCTNS416;
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.730308",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses amount is incorrect",
                            LongDescription = @"The Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses amount must equal the sum of all Work related clothing expenses amounts in the attached Deductions schedule",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Clothing.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730308" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR137", Value = report.iitrParentDocument.IITR137.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS416", Value = sumDDCTNS416.ToString() });

                        response.Add(processMessage);
                    }
                }

                #endregion // VR.ATO.IITR.730308

                #region Repeating RPSCSequNumContext

                if (Count(ddctnsChildDocument.RPSCSeqNumContextCollection) > 0)
                {
                    DateTime? dateTaxpayerTurns75 = null;
                    bool taxpayerDidNotTurn75ThisPeriod = true;

                    if (report.iitrParentDocument.IITR29.HasValue)
                    {
                        // Calculated for VR.ATO.IITR.730301 and taxpayerDidNotTurn75ThisPeriod
                        dateTaxpayerTurns75 = report.iitrParentDocument.IITR29.Value.AddYears(75);
                        // Calculated for VR.ATO.IITR.730302, but only needs to be calculated once
                        taxpayerDidNotTurn75ThisPeriod = report.iitrParentDocument.RPStartDate.HasValue && report.iitrParentDocument.RPEndDate.HasValue ? dateTaxpayerTurns75.Value < report.iitrParentDocument.RPStartDate.Value || dateTaxpayerTurns75.Value > report.iitrParentDocument.RPEndDate.Value : true;
                    }

                    foreach (DDCTNS2019.RPSCSeqNumContext context in ddctnsChildDocument.RPSCSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.730301
                        /* VR.ATO.IITR.730301
                        The date of last eligible superannuation contribution must not be more than 28 days after the taxpayer's 75th birthday month.

                        Legacy Rule Format
                        IF (COUNT(SCHEDULE="DDCTNS") = 1 AND ([ddctns403] > ConvertToDate(28, MonthAsInt[iitr29] + 1, Year[iitr29] + 75))
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^IITR29 =                   tns:PersonDemographicDetails.Birth.Date
                        DDCTNS:RP.{SCSeqNum}:^DDCTNS403 =   tns:IncomeTax.Deduction.SuperannuationContributionEligible.Date
                        */
                        assertion = (dateTaxpayerTurns75.HasValue && report.iitrParentDocument.IITR29.HasValue && context.DDCTNS403.HasValue && context.DDCTNS403.Value > new DateTime(dateTaxpayerTurns75.Value.Year, dateTaxpayerTurns75.Value.Month, 28).AddMonths(1));
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.730301",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Date of last eligible superannuation contribution on the attached Deductions schedule is incorrect",
                                LongDescription = @"The Date of last eligible superannuation contribution provided on the attached Deductions schedule must not be more than 28 days after the taxpayer's 75th birthday month",
                                Location = "/xbrli:xbrl/ddctns:IncomeTax.Deduction.SuperannuationContributionEligible.Date[@contextRef='" + context.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730301" } }
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter()
                            { Name = "IITR29", Value = GetValueOrEmpty(report.iitrParentDocument.IITR29) });
                            processMessage.Parameters.Add(new ProcessMessageParameter()
                            { Name = "DDCTNS403", Value = GetValueOrEmpty(context.DDCTNS403) });

                            response.Add(processMessage);
                        }
                        #endregion // VR.ATO.IITR.730301

                        #region VR.ATO.IITR.730302
                        /* VR.ATO.IITR.730302
                        Date of last eligible superannuation contribution must not be provided on the attached Deductions schedule unless the taxpayer turned 75 years of age within the financial year.

                        Legacy Rule Format
                        IF COUNT(SCHEDULE="DDCTNS") = 1 AND ANY OCCURRENCE OF([ddctns403]) <> NULL AND ([iitr29] < ConvertToDate(Day(RP:period.startDate), MonthAsInt(RP:period.startDate), Year(RP:period.startDate) - 75) OR [iitr29] > ConvertToDate(Day(RP:period.endDate), MonthAsInt(RP:period.endDate), Year(RP:period.endDate) - 75))
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^IITR29 =                   tns:PersonDemographicDetails.Birth.Date
                        DDCTNS:RP.{SCSeqNum}:^DDCTNS403 =   tns:IncomeTax.Deduction.SuperannuationContributionEligible.Date
                        */
                        assertion = (report.iitrParentDocument.IITR29.HasValue && context.DDCTNS403.HasValue && dateTaxpayerTurns75.HasValue && taxpayerDidNotTurn75ThisPeriod);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.730302",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Date of last eligible superannuation contribution must not be provided",
                                LongDescription = @"Date of last eligible superannuation contribution must not be provided on the attached Deductions schedule unless the taxpayer turned 75 years of age within the financial year",
                                Location = "/xbrli:xbrl/ddctns:IncomeTax.Deduction.SuperannuationContributionEligible.Date[@contextRef='" + context.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730302" } }
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter()
                            { Name = "IITR29", Value = GetValueOrEmpty(report.iitrParentDocument.IITR29) });
                            processMessage.Parameters.Add(new ProcessMessageParameter()
                            { Name = "DDCTNS403", Value = GetValueOrEmpty(context.DDCTNS403) });

                            response.Add(processMessage);
                        }
                        #endregion //VR.ATO.IITR.730302
                    }
                }

                #endregion //Repeating RPSCSequNumContext
            }

            #endregion // DDCTNS Schedule Present

            // Calculated for VR.ATO.IITR.730303 and VR.ATO.IITR.200014
            bool noDDCTNS407Claimed = (ddctnsChildDocument == null || Count(ddctnsChildDocument.RPSCSeqNumContextCollection) == 0) ||
                                       ddctnsChildDocument.RPSCSeqNumContextCollection.All(context => !context.DDCTNS407.HasValue);

            #region VR.ATO.IITR.200014
            /*  VR.ATO.IITR.200014
            Where the 'Seniors and pensioners - Tax offset code' indicates at any time during the year the taxpayer was single or widowed, or separated; and all occurrences of Deduction claimed amounts are null, check deductible personal superannuation contributions.

            Legacy Rule Format:
            IF ([iitr163] = "Single") AND (COUNT(SCHEDULE="DDCTNS") = 0 OR ALL OCCURRENCES OF([ddctns407]) = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            Data Elements:

            RP:^IITR163 =                       tns:TaxOffsetClaim.SeniorAndPensioner.Code
            DDCTNS:RP.{SCSeqNum}:^DDCTNS407 =   tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            */
            assertion = report.iitrParentDocument.IITR163 == @"Single" && (ddctnsScheduleCount == 0 || noDDCTNS407Claimed);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.201014",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check deductible personal superannuation contributions",
                    LongDescription = @"Rebate income is used to determine eligibility for the Seniors and pensioners tax offset. Where applicable, complete Deductible personal superannuation contributions from the Deductions schedule.",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.SeniorAndPensioner.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.200014" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR163", Value = report.iitrParentDocument.IITR163 });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS407", Value = "null" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.200014

            #region VR.ATO.IITR.730303
            /* VR.ATO.IITR.730303
            Where the taxpayer was not covered by private hospital cover for the whole reporting period, the number of days not liable for Medicare surcharge
            was less than the whole reporting period, and no occurrences of Deduction claimed (deductible personal superannuation contributions) amounts are present,
            check Deduction claimed amounts

            Legacy Rule Format
            IF ([iitr176] <> TRUE) AND ([iitr177] < (RP:Period.endDate - RP:Period.startDate + 1)) AND (COUNT(SCHEDULE = "DDCTNS") = 0 OR ALL OCCURRENCES OF([ddctns407]) = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR176 =                  tns:IncomeTax.MedicareLevy.PrivateHospitalCoverWholePeriod.Indicator
            IITR:RP:^IITR177 =                  tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count
            DDCTNS:RP.{SCSeqNum}:^DDCTNS407 =   tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            */
            assertion = report.iitrParentDocument.IITR176.HasValue && !report.iitrParentDocument.IITR176.Value //Indicated that not covered for whole period
                && report.iitrParentDocument.RPEndDate.HasValue && report.iitrParentDocument.RPStartDate.HasValue && report.iitrParentDocument.IITR177.HasValue //Ensure the values to be used have values
                && report.iitrParentDocument.IITR177.Value < (report.iitrParentDocument.RPEndDate.Value - report.iitrParentDocument.RPStartDate.Value).Days + 1 //Days exempt from surcharge less than whole period
                && (ddctnsScheduleCount == 0 || noDDCTNS407Claimed); //there is no DDCTNS schedule or there are no Super Contribution amounts claimed
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.730303",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check deductible personal superannuation contributions",
                    LongDescription = @"Income for surcharge purposes is used to determine if the taxpayer is liable for Medicare Levy Surcharge. Where applicable, complete Deductible personal superannuation contributions from the Deduction schedule.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MedicareLevy.PrivateHospitalCoverWholePeriod.Indicator[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730303" } }
                };
                processMessage.Parameters.Add(new ProcessMessageParameter()
                { Name = "IITR176", Value = report.iitrParentDocument.IITR176.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter()
                { Name = "IITR177", Value = report.iitrParentDocument.IITR177.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter()
                { Name = "DDCTNS407", Value = "null" });

                response.Add(processMessage);
            }
            #endregion //VR.ATO.IITR.730303

            // ####################################################################################################
            //                                         NRFI CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.000639

            /* VR.ATO.IITR.000639
            An amount has been entered at Non-resident foreign income amount and there is no Non-Resident Foreign Income (NRFI) schedule attached. Please complete and attach a NRFI schedule.

            Legacy Rule Format:
            IF [iitr494] <> NULL AND COUNT(SCHEDULE = "NRFI") = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.iitrParentDocument.IITR494 != null && (report.nrfiChildDocuments == null || Count(report.nrfiChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.000639",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-Resident Foreign Income (NRFI) schedule is required for Non-resident foreign income amount",
                    LongDescription = @"An amount has been entered at Non-resident foreign income amount and there is no Non-Resident Foreign Income (NRFI) schedule attached. Please complete and attach a NRFI schedule.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000639" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.000639

            #region VR.ATO.IITR.310093

            /* VR.ATO.IITR.310093
            The number of Non-Resident Foreign Income schedules must not exceed 1

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "NRFI") > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.nrfiChildDocuments != null && Count(report.nrfiChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310093",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Non-Resident Foreign Income schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310093" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310093

            #region NRFI Child Present

            if (report.nrfiChildDocuments != null && Count(report.nrfiChildDocuments) == 1)
            {
                NRFI2018 nrfiChildDocument = report.nrfiChildDocuments[0];

                #region VR.ATO.IITR.000641

                /* VR.ATO.IITR.000641
                When the simple self-assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Gross foreign income amount, less standard occupation deductions must equal the Non-resident foreign income amount on the parent form.

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] <> NULL AND [nrfi3] = "Simple" AND SUBSTRING([nrfi2],1,4) = SET(DOMAIN(OCCUPATION PREFIX CODES)) AND [iitr494] <> ([nrfi4] - RoundDown([nrfi4] * ConditionalLookup(SUBSTRING([nrfi2],1,4), 'Occupation Prefix Codes', NULL, 'Deduction Median Rate', 0),0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 != null && nrfiChildDocument.NRFI3 == "Simple" && DeductionMedianRates.deductionMedianRates.Keys.Contains(nrfiChildDocument.NRFI2.Substring(0, 4)) && report.iitrParentDocument.IITR494.GetValueOrDefault() != (nrfiChildDocument.NRFI4.GetValueOrDefault() - Math.Floor(nrfiChildDocument.NRFI4.GetValueOrDefault() * (Decimal)DeductionMedianRates.deductionMedianRates[nrfiChildDocument.NRFI2.Substring(0, 4)])));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000641",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount using the simple self-assessment method is incorrect",
                        LongDescription = @"When the simple self-assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Gross foreign income amount, less standard occupation deductions must equal the Non-resident foreign income amount on the parent form",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000641" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000641

                #region VR.ATO.IITR.000642

                /* VR.ATO.IITR.000642
                When the overseas assessed method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Foreign tax return taxable income must equal the Non-resident foreign income amount on the parent form, with the exception that when Foreign tax return taxable income on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0).

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] <> NULL AND [nrfi3] = "Overseas" AND (([nrfi10] >= 0 AND [iitr494] <> [nrfi10]) OR ([nrfi10] < 0 AND [iitr494] <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 != null && nrfiChildDocument.NRFI3 == "Overseas" && ((nrfiChildDocument.NRFI10.GetValueOrDefault() >= 0 && report.iitrParentDocument.IITR494.GetValueOrDefault() != nrfiChildDocument.NRFI10.GetValueOrDefault()) || (nrfiChildDocument.NRFI10.GetValueOrDefault() < 0 && report.iitrParentDocument.IITR494.GetValueOrDefault() != 0)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000642",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount using the overseas assessed method is incorrect",
                        LongDescription = @"When the overseas assessed method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Foreign tax return taxable income must equal the Non-resident foreign income amount on the parent form, with the exception that when Foreign tax return taxable income on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0)",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000642" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000642

                #region VR.ATO.IITR.000643

                /* VR.ATO.IITR.000643
                When the comprehensive assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the sum of Total salary or wages amount, Total allowance amount, Total government allowances amount, Total pension amount, Net business income, Total personal services income, Total interest amount, Total dividend amount, Net partnership and trust income amount, Net capital gains amount, Net rent amount and Total other income amount, less the sum of Total employee work related deduction amount, Total depreciation amount, Total interest and dividend amount, Total undeducted purchase price of a foreign pension amount, Total personal super contributions for business, Total other deduction amount and Prior year tax losses amount, must equal the Non-resident foreign income amount on the parent form, with the exception that when the sum of amounts on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0).

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] <> NULL AND [nrfi3] = "Comprehensive" AND ((([nrfi11] + [nrfi12] + [nrfi13] + [nrfi14] + [nrfi15] + [nrfi16] + [nrfi17] + [nrfi18] + [nrfi19] + [nrfi20] + [nrfi21] + [nrfi22]) >= ([nrfi23] + [nrfi24] + [nrfi25] + [nrfi26] + [nrfi27] + [nrfi28] + [nrfi32]) AND [iitr494] <> (([nrfi11] + [nrfi12] + [nrfi13] + [nrfi14] + [nrfi15] + [nrfi16] + [nrfi17] + [nrfi18] + [nrfi19] + [nrfi20] + [nrfi21] + [nrfi22]) - ([nrfi23] + [nrfi24] + [nrfi25] + [nrfi26] + [nrfi27] + [nrfi28] + [nrfi32]))) OR (([nrfi11] + [nrfi12] + [nrfi13] + [nrfi14] + [nrfi15] + [nrfi16] + [nrfi17] + [nrfi18] + [nrfi19] + [nrfi20] + [nrfi21] + [nrfi22]) < ([nrfi23] + [nrfi24] + [nrfi25] + [nrfi26] + [nrfi27] + [nrfi28] + [nrfi32]) AND [iitr494] <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 != null && nrfiChildDocument.NRFI3 == "Comprehensive" && (((nrfiChildDocument.NRFI11.GetValueOrDefault() + nrfiChildDocument.NRFI12.GetValueOrDefault() + nrfiChildDocument.NRFI13.GetValueOrDefault() + nrfiChildDocument.NRFI14.GetValueOrDefault() + nrfiChildDocument.NRFI15.GetValueOrDefault() + nrfiChildDocument.NRFI16.GetValueOrDefault() + nrfiChildDocument.NRFI17.GetValueOrDefault() + nrfiChildDocument.NRFI18.GetValueOrDefault() + nrfiChildDocument.NRFI19.GetValueOrDefault() + nrfiChildDocument.NRFI20.GetValueOrDefault() + nrfiChildDocument.NRFI21.GetValueOrDefault() + nrfiChildDocument.NRFI22.GetValueOrDefault()) >= (nrfiChildDocument.NRFI23.GetValueOrDefault() + nrfiChildDocument.NRFI24.GetValueOrDefault() + nrfiChildDocument.NRFI25.GetValueOrDefault() + nrfiChildDocument.NRFI26.GetValueOrDefault() + nrfiChildDocument.NRFI27.GetValueOrDefault() + nrfiChildDocument.NRFI28.GetValueOrDefault() + nrfiChildDocument.NRFI32.GetValueOrDefault()) && report.iitrParentDocument.IITR494.GetValueOrDefault() != (nrfiChildDocument.NRFI11.GetValueOrDefault() + nrfiChildDocument.NRFI12.GetValueOrDefault() + nrfiChildDocument.NRFI13.GetValueOrDefault() + nrfiChildDocument.NRFI14.GetValueOrDefault() + nrfiChildDocument.NRFI15.GetValueOrDefault() + nrfiChildDocument.NRFI16.GetValueOrDefault() + nrfiChildDocument.NRFI17.GetValueOrDefault() + nrfiChildDocument.NRFI18.GetValueOrDefault() + nrfiChildDocument.NRFI19.GetValueOrDefault() + nrfiChildDocument.NRFI20.GetValueOrDefault() + nrfiChildDocument.NRFI21.GetValueOrDefault() + nrfiChildDocument.NRFI22.GetValueOrDefault()) - (nrfiChildDocument.NRFI23.GetValueOrDefault() + nrfiChildDocument.NRFI24.GetValueOrDefault() + nrfiChildDocument.NRFI25.GetValueOrDefault() + nrfiChildDocument.NRFI26.GetValueOrDefault() + nrfiChildDocument.NRFI27.GetValueOrDefault() + nrfiChildDocument.NRFI28.GetValueOrDefault() + nrfiChildDocument.NRFI32.GetValueOrDefault())) || ((nrfiChildDocument.NRFI11.GetValueOrDefault() + nrfiChildDocument.NRFI12.GetValueOrDefault() + nrfiChildDocument.NRFI13.GetValueOrDefault() + nrfiChildDocument.NRFI14.GetValueOrDefault() + nrfiChildDocument.NRFI15.GetValueOrDefault() + nrfiChildDocument.NRFI16.GetValueOrDefault() + nrfiChildDocument.NRFI17.GetValueOrDefault() + nrfiChildDocument.NRFI18.GetValueOrDefault() + nrfiChildDocument.NRFI19.GetValueOrDefault() + nrfiChildDocument.NRFI20.GetValueOrDefault() + nrfiChildDocument.NRFI21.GetValueOrDefault() + nrfiChildDocument.NRFI22.GetValueOrDefault()) < (nrfiChildDocument.NRFI23.GetValueOrDefault() + nrfiChildDocument.NRFI24.GetValueOrDefault() + nrfiChildDocument.NRFI25.GetValueOrDefault() + nrfiChildDocument.NRFI26.GetValueOrDefault() + nrfiChildDocument.NRFI27.GetValueOrDefault() + nrfiChildDocument.NRFI28.GetValueOrDefault() + nrfiChildDocument.NRFI32.GetValueOrDefault()) && report.iitrParentDocument.IITR494.GetValueOrDefault() != 0)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000643",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount using the comprehensive assessment method is incorrect",
                        LongDescription = @"When the comprehensive assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the sum of Total salary or wages amount, Total allowance amount, Total government allowances amount, Total pension amount, Net business income, Total personal services income, Total interest amount, Total dividend amount, Net partnership and trust income amount, Net capital gains amount, Net rent amount and Total other income amount, less the sum of Total employee work related deduction amount, Total depreciation amount, Total interest and dividend amount, Total undeducted purchase price of a foreign pension amount, Total personal super contributions for business, Total other deduction amount and Prior year tax losses amount, must equal the Non-resident foreign income amount on the parent form, with the exception that when the sum of amounts on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0)",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000643" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000643

                #region VR.ATO.IITR.000644

                /* VR.ATO.IITR.000644
                Foreign tax return financial year provided on Non-Resident Foreign Income (NRFI) schedule must be within one year of Year of return on parent form

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [nrfi7] <> NULL AND [nrfi7] <> [iitr10] +/- 1
                   RETURN VALIDATION RULE
                ENDIF
                */
                assertion = (nrfiChildDocument.NRFI7 != null && OutsideRange((Decimal)nrfiChildDocument.NRFI7, (Decimal)report.iitrParentDocument.IITR10, 1));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000644",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Foreign tax return financial year is invalid",
                        LongDescription = @"Foreign tax return financial year provided on Non-Resident Foreign Income (NRFI) schedule must be within one year of Year of return on parent form",
                        Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000644" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000644

                #region VR.ATO.IITR.000645

                /* VR.ATO.IITR.000645
                Unable to confirm whether the correct Non-resident foreign income amount has been provided for the simple self-assessment method, as the Foreign occupation code provided on the Non-Resident Foreign Income (NRFI) schedule is not recognised. Record the details of the message and contact the agency service desk.

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [nrfi3] = "Simple" AND [nrfi2] <> NULL AND SUBSTRING([nrfi2],1,4) <> SET(DOMAIN(OCCUPATION PREFIX CODES))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.NRFI3 == "Simple" && nrfiChildDocument.NRFI2 != null && !DeductionMedianRates.deductionMedianRates.Keys.Contains(nrfiChildDocument.NRFI2.Substring(0, 4)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000645",
                        Severity = ProcessMessageSeverity.Warning,
                        Description = @"Foreign occupation code is unknown",
                        LongDescription = @"Unable to calculate the correct Non-resident foreign income amount for the simple self-assessment method, as the Foreign occupation code provided on the Non-Resident Foreign Income (NRFI) schedule is not configured. Record the details of the message and contact the agency service desk.",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000645" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000645

                #region VR.ATO.IITR.000651

                /* VR.ATO.IITR.000651
                When a Non-Resident Foreign Income (NRFI) schedule is attached, the Non-resident foreign income amount must be provided

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] = NULL
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000651",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount must be provided",
                        LongDescription = @"When a Non-Resident Foreign Income (NRFI) schedule is attached, the Non-resident foreign income amount must be provided",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000651" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000651

                #region VR.ATO.IITR.310094

                /* VR.ATO.IITR.310094
                Invalid context. Period start date for the Non-Resident Foreign Income Schedule must be the same as the Period start date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "NRFI") = 1) AND (NRFI:RP:period.startDate <> RP:period.startDate)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310094",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period start date for the Non-Resident Foreign Income Schedule is invalid",
                        LongDescription = @"Invalid context. Period start date for the Non-Resident Foreign Income Schedule must be the same as the Period start date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + nrfiChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310094" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310094

                #region VR.ATO.IITR.310095

                /* VR.ATO.IITR.310095
                Invalid context. Period end date for the Non-Resident Foreign Income Schedule must be the same as the Period end date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "NRFI") = 1) AND (NRFI:RP:period.endDate <> RP:period.endDate)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310095",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period end date for the Non-Resident Foreign Income Schedule is invalid",
                        LongDescription = @"Invalid context. Period end date for the Non-Resident Foreign Income Schedule must be the same as the Period end date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + nrfiChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310095" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310095

                #region VR.ATO.IITR.600046

                /* VR.ATO.IITR.600046
                The TFN provided for the Non-Resident Foreign Income schedule must be the same as the TFN of the parent form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "NRFI") = 1) AND (NRFI:RP:entity.identifier.TFN <> RP:entity.identifier.TFN)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600046",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax file number invalid",
                        LongDescription = @"The TFN provided for the Non-Resident Foreign Income schedule must be the same as the TFN of the parent form",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + nrfiChildDocument.RPId + "']/xbrli:entity/xbrli:identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600046" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600046
            }

            #endregion // NRFI Child Present

            // ####################################################################################################
            //                                         PSS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.300041

            /*  VR.ATO.IITR.300041
            When an amount has been provided for 'Net income/loss from business tax withheld voluntary agreement', 'Net income/loss from business tax withheld where Australian business number not quoted', 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)', or 'Net income/loss from business tax withheld - labour hire or other specified payments', a Payment Summary Schedule (PSS) must be attached

            Legacy Rule Format:
            IF (([iitr286] > 0 OR [iitr287] > 0 OR [iitr288] > 0 OR [iitr289] > 0) AND COUNT(SCHEDULE = "PSS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR286 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
            IITR:RP:^IITR287 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
            IITR:RP:^IITR288 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
            IITR:RP:^IITR289 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
            */
            assertion = ((report.iitrParentDocument.IITR286.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR287.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR288.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR289.GetValueOrDefault() > 0) && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.301041",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for Business income tax withheld",
                    LongDescription = @"When an amount has been provided for 'Net income/loss from business tax withheld voluntary agreement', 'Net income/loss from business tax withheld where Australian business number not quoted', 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)', or 'Net income/loss from business tax withheld - labour hire or other specified payments', a Payment Summary Schedule (PSS) must be attached",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300041" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR286", Value = report.iitrParentDocument.IITR286.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR287", Value = report.iitrParentDocument.IITR287.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR288", Value = report.iitrParentDocument.IITR288.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR289", Value = report.iitrParentDocument.IITR289.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300041

            #region VR.ATO.IITR.300046

            /*  VR.ATO.IITR.300046
            An amount has been entered at Attributed personal services income and there is no Payment Summary schedule attached. Please complete and attach a Payment summary schedule for Individuals.

            Legacy Rule Format:
            IF ([iitr112] > 0 AND COUNT(SCHEDULE = "PSS" = 0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR112 = tns:Income.PersonalServicesIncome.Total.Amount
            */
            assertion = (report.iitrParentDocument.IITR112.GetValueOrDefault() > 0 && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300046",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for an Attributed PSI",
                    LongDescription = @"An amount has been entered at Attributed personal services income and there is no Payment Summary schedule attached. Please complete and attach a Payment summary schedule for Individuals.",
                    Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300046" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR112", Value = report.iitrParentDocument.IITR112.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300046

            #region VR.ATO.IITR.300047

            /*  VR.ATO.IITR.300047
            Amounts have been entered at Personal services income for the tax withheld amounts or PSI income and there is no Payment Summary schedule attached. Please complete and attach a Payment Summary schedule for Individuals.

            Legacy Rule Format:
            IF (([iitr276] > 0 OR [iitr277] > 0 OR [iitr278] > 0 OR [iitr411] > 0 OR [iitr412] > 0 OR [iitr413] > 0) AND COUNT(SCHEDULE = "PSS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR276 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount
            IITR:RP:^IITR277 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeABNNotQuoted.Amount
            IITR:RP:^IITR278 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeLabourHireOrOtherPayments.Amount
            IITR:RP:^IITR411 = tns:Income.PersonalServicesIncome.VoluntaryAgreement.Amount
            IITR:RP:^IITR412 = tns:Income.PersonalServicesIncome.ABNNotQuoted.Amount
            IITR:RP:^IITR413 = tns:Income.PersonalServicesIncome.LabourHireOrOtherSpecifiedPayment.Amount
            */
            assertion = ((report.iitrParentDocument.IITR276.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR277.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR278.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR411.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR412.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR413.GetValueOrDefault() > 0) && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300047",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for Personal services income",
                    LongDescription = @"Amounts have been entered at Personal services income for the tax withheld amounts or PSI income and there is no Payment Summary schedule attached. Please complete and attach a Payment Summary schedule for Individuals.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300047" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR276", Value = report.iitrParentDocument.IITR276.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR277", Value = report.iitrParentDocument.IITR277.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR278", Value = report.iitrParentDocument.IITR278.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR411", Value = report.iitrParentDocument.IITR411.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR412", Value = report.iitrParentDocument.IITR412.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR413", Value = report.iitrParentDocument.IITR413.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300047

            #region VR.ATO.IITR.310021

            /*  VR.ATO.IITR.310021
            Amounts have been entered at primary production - Gross payments or non-primary production - Gross payments and there is no Payment Summary schedule attached. A Payment summary schedule for Individuals must be completed

            Legacy Rule Format:
            IF (([iitr436] + [iitr437] + [iitr438] + [iitr439] + [iitr440] + [iitr441] + [iitr442]) > 0 AND COUNT(SCHEDULE = "PSS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR436 = tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount
            IITR:RP:^IITR437 = tns:Income.ABNNotQuotedNonPrimaryProductionPaymentGross.Amount
            IITR:RP:^IITR438 = tns:Income.ForeignResidentWithholdingNonPrimaryProduction.Amount
            IITR:RP:^IITR439 = tns:Income.VoluntaryAgreementPrimaryProductionGross.Amount
            IITR:RP:^IITR440 = tns:Income.VoluntaryAgreementNonPrimaryProductionGross.Amount
            IITR:RP:^IITR441 = tns:Income.LabourHireArrangementPrimaryProductionPaymentGross.Amount
            IITR:RP:^IITR442 = tns:Income.LabourHireArrangementNonPrimaryProductionPaymentGross.Amount
            */
            assertion = ((report.iitrParentDocument.IITR436.GetValueOrDefault() + report.iitrParentDocument.IITR437.GetValueOrDefault() + report.iitrParentDocument.IITR438.GetValueOrDefault() + report.iitrParentDocument.IITR439.GetValueOrDefault() + report.iitrParentDocument.IITR440.GetValueOrDefault() + report.iitrParentDocument.IITR441.GetValueOrDefault() + report.iitrParentDocument.IITR442.GetValueOrDefault()) > 0 && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310021",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for business income",
                    LongDescription = @"Amounts have been entered at primary production - Gross payments or non-primary production - Gross payments and there is no Payment Summary schedule attached. A Payment summary schedule for Individuals must be completed.",
                    Location = "/xbrli:xbrl/tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310021" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR436", Value = report.iitrParentDocument.IITR436.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR437", Value = report.iitrParentDocument.IITR437.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR438", Value = report.iitrParentDocument.IITR438.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR439", Value = report.iitrParentDocument.IITR439.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR440", Value = report.iitrParentDocument.IITR440.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR441", Value = report.iitrParentDocument.IITR441.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR442", Value = report.iitrParentDocument.IITR442.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310021

            #region VR.ATO.IITR.310089

            /*  VR.ATO.IITR.310089
            The number of Payment Summary schedules must not exceed 1.

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "PSS") > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.pssChildDocuments != null && Count(report.pssChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310089",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Payment Summary schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310089" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310089

            #region PSS Child Present

            if (report.pssChildDocuments != null && Count(report.pssChildDocuments) == 1)
            {
                PSS2018 pssChildDocument = report.pssChildDocuments[0];

                #region VR.ATO.IITR.300042

                /*  VR.ATO.IITR.300042
                The sum of Tax withheld where ABN not quoted amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld where Australian business number not quoted on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss23] WHERE [pss15] = "Business")) <> [iitr287]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR287               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS23 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS23.GetValueOrDefault()) != report.iitrParentDocument.IITR287.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300042",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld where ABN not quoted amount is incorrect",
                        LongDescription = @"The sum of Tax withheld where ABN not quoted amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld where Australian business number not quoted on the main return form",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300042" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR287", Value = report.iitrParentDocument.IITR287.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS23", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS23.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300042

                #region VR.ATO.IITR.300043

                /*  VR.ATO.IITR.300043
                The sum of Tax withheld for Voluntary agreement withholding amount in the Payment summary schedule, where the Income Type is set to Business, does not equal Net income/loss from business tax withheld voluntary agreement on the main tax return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss24] WHERE [pss15] = "Business")) <> [iitr286]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR286               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS24 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS24.GetValueOrDefault()) != report.iitrParentDocument.IITR286.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300043",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld - voluntary agreement amount is incorrect",
                        LongDescription = @"The sum of Tax withheld for Voluntary agreement withholding amount in the Payment summary schedule, where the Income Type is set to Business, does not equal Net income/loss from business tax withheld voluntary agreement on the main tax return",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300043" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR286", Value = report.iitrParentDocument.IITR286.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS24", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS24.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300043

                #region VR.ATO.IITR.300044

                /*  VR.ATO.IITR.300044
                The sum of Tax withheld for labour hire or other specified payments withholding amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld - labour hire or other specified payments on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss25] WHERE [pss15] = "Business")) <> [iitr289]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR289               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS25 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS25.GetValueOrDefault()) != report.iitrParentDocument.IITR289.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300044",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld - labour hire or other payments amount is incorrect",
                        LongDescription = @"The sum of Tax withheld for labour hire or other specified payments withholding amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld - labour hire or other specified payments on the main return form",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300044" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR289", Value = report.iitrParentDocument.IITR289.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS25", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS25.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300044

                #region VR.ATO.IITR.300045

                /*  VR.ATO.IITR.300045
                The sum of 'Tax withheld for foreign resident withholding' in the Payment Summary Schedule (PSS), where the associated Income Type is either Business or Personal Services Income, must equal the 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)' amount on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss22] WHERE [pss15] = SET("Business", "Personal Services Income"))) <> [iitr288]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR288               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS22 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").Sum(context => context.PSS22.GetValueOrDefault()) != report.iitrParentDocument.IITR288.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.301045",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains) amount is incorrect",
                        LongDescription = @"The sum of 'Tax withheld for foreign resident withholding' in the Payment Summary Schedule (PSS), where the associated Income Type is either Business or Personal Services Income, must equal the 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)' amount on the main return form",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300045" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR288", Value = report.iitrParentDocument.IITR288.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.Find(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").PSS15 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS22", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS22.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300045

                #region VR.ATO.IITR.300048

                /*  VR.ATO.IITR.300048
                The amount shown at Tax withheld - voluntary agreement is not equal to the sum of the amounts of Tax withheld for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr276] <> NULL AND [iitr276] <> SUM([pss24] WHERE [pss15] = "Personal Services Income")) OR ([iitr276] = NULL AND SUM([pss24] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR276               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS24 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                */
                assertion = ((report.iitrParentDocument.IITR276 != null && report.iitrParentDocument.IITR276 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS24.GetValueOrDefault())) || (report.iitrParentDocument.IITR276 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS24.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300048",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI tax withheld - voluntary agreement is incorrect",
                        LongDescription = @"The amount shown at Tax withheld - voluntary agreement is not equal to the sum of the amounts of Tax withheld for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300048" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR276", Value = report.iitrParentDocument.IITR276.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS24", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS24).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300048

                #region VR.ATO.IITR.300049

                /*  VR.ATO.IITR.300049
                The amount shown at Tax withheld where Australian business number not quoted is not equal to the sum of Tax withheld for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr277] <> NULL AND [iitr277] <> SUM([pss23] WHERE [pss15] = "Personal Services Income")) OR ([iitr277] = NULL AND SUM([pss23] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR277               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeABNNotQuoted.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS23 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                */
                assertion = ((report.iitrParentDocument.IITR277 != null && report.iitrParentDocument.IITR277 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS23.GetValueOrDefault())) || (report.iitrParentDocument.IITR277 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS23.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300049",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI tax withheld where ABN not quoted is incorrect",
                        LongDescription = @"The amount shown at Tax withheld where Australian business number not quoted is not equal to the sum of Tax withheld for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeABNNotQuoted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300049" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR277", Value = report.iitrParentDocument.IITR277.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS23", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS23).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300049

                #region VR.ATO.IITR.300050

                /*  VR.ATO.IITR.300050
                The amount shown at Personal Services Income tax withheld - labour hire and other specified payments is not equal to the sum of Tax withheld for labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr278] <> NULL AND [iitr278] <> SUM([pss25] WHERE [pss15] = "Personal Services Income")) OR ([iitr278] = NULL AND SUM([pss25] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR278               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeLabourHireOrOtherPayments.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS25 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                */
                assertion = ((report.iitrParentDocument.IITR278 != null && report.iitrParentDocument.IITR278 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS25.GetValueOrDefault())) || (report.iitrParentDocument.IITR278 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS25.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300050",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI tax withheld - labour hire and other specified payments is incorrect",
                        LongDescription = @"The amount shown at Personal Services Income tax withheld - labour hire and other specified payments is not equal to the sum of Tax withheld for labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeLabourHireOrOtherPayments.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300050" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR278", Value = report.iitrParentDocument.IITR278.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS25", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS25.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300050

                #region VR.ATO.IITR.300051

                /*  VR.ATO.IITR.300051
                The amount shown at Tax withheld from attributed personal services income is not equal to the sum of Tax withheld for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr111] <> NULL AND [iitr111] <> (SUM([pss32] WHERE [pss15] = "Personal Services Income")) +/- 1) OR ([iitr111] = NULL AND (SUM([pss32] WHERE [pss15] = "Personal Services Income")) +/- 1 <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR111               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS32 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount
                */
                assertion = ((report.iitrParentDocument.IITR111 != null && OutsideRange(report.iitrParentDocument.IITR111.GetValueOrDefault(), pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS32.GetValueOrDefault()), 1)) || (report.iitrParentDocument.IITR111 == null && OutsideRange(0, pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS32.GetValueOrDefault()), 1)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300051",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld from attributed personal services income is incorrect",
                        LongDescription = @"The amount shown at Tax withheld from attributed personal services income is not equal to the sum of Tax withheld for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300051" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR111", Value = report.iitrParentDocument.IITR111.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS32", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS32.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300051

                #region VR.ATO.IITR.300052

                /*  VR.ATO.IITR.300052
                The amount shown at Attributed personal services income is not equal to the sum of Gross payment for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Gross payment for Attributed Personal Services Income

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "PSS") = 1 AND (([iitr112] <> NULL AND [iitr112] <> SUM([pss31] WHERE [pss15] = "Personal Services Income")) OR ([iitr112] = NULL AND SUM([pss31] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR112               = tns:Income.PersonalServicesIncome.Total.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS31 = tns:Remuneration.PersonalServicesIncome.AttributedGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR112 != null && report.iitrParentDocument.IITR112.GetValueOrDefault() != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS31.GetValueOrDefault())) || (report.iitrParentDocument.IITR112 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS31.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300052",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Attributed personal services income is incorrect",
                        LongDescription = @"The amount shown at Attributed personal services income is not equal to the sum of Gross payment for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Gross payment for Attributed Personal Services Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300052" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR112", Value = report.iitrParentDocument.IITR112.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS32", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS31.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300052

                #region VR.ATO.IITR.310035

                /*  VR.ATO.IITR.310035
                The sum of Gross payment for withholding where ABN not quoted in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments where Australian Business Number not quoted and Non-PP - Gross payments where Australian Business Number not quoted on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss19] WHERE [pss15] = "Business")) <> ([iitr436] + [iitr437])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR436               = tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount
                IITR:RP:^IITR437               = tns:Income.ABNNotQuotedNonPrimaryProductionPaymentGross.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS19 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS19.GetValueOrDefault()) != (report.iitrParentDocument.IITR436.GetValueOrDefault() + report.iitrParentDocument.IITR437.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310035",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payment where ABN not quoted is incorrect",
                        LongDescription = @"The sum of Gross payment for withholding where ABN not quoted in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments where Australian Business Number not quoted and Non-PP - Gross payments where Australian Business Number not quoted on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310035" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR436", Value = report.iitrParentDocument.IITR436.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR437", Value = report.iitrParentDocument.IITR437.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS19", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS19.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310035

                #region VR.ATO.IITR.310036

                /*  VR.ATO.IITR.310036
                The sum of Gross payment for Voluntary agreement withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP -  Gross payments voluntary agreement and Non-PP - Gross payments voluntary agreement on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss20] WHERE [pss15] = "Business")) <> ([iitr439] + [iitr440])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR439               = tns:Income.VoluntaryAgreementPrimaryProductionGross.Amount
                IITR:RP:^IITR440               = tns:Income.VoluntaryAgreementNonPrimaryProductionGross.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS20 = tns:Remuneration.VoluntaryAgreementGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS20.GetValueOrDefault()) != (report.iitrParentDocument.IITR439.GetValueOrDefault() + report.iitrParentDocument.IITR440.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310036",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payments - voluntary agreement is incorrect",
                        LongDescription = @"The sum of Gross payment for Voluntary agreement withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP -  Gross payments voluntary agreement and Non-PP - Gross payments voluntary agreement on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.VoluntaryAgreementPrimaryProductionGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310036" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR439", Value = report.iitrParentDocument.IITR439.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR440", Value = report.iitrParentDocument.IITR440.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS20", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS20.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310036

                #region VR.ATO.IITR.310037

                /*  VR.ATO.IITR.310037
                The sum of Gross payment for Labour hire or other specified payments withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments labour hire or other specified payments and Non-PP - Gross payments labour hire or other specified payments on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss21] WHERE [pss15] = "Business")) <> ([iitr441] + [iitr442])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR441               = tns:Income.LabourHireArrangementPrimaryProductionPaymentGross.Amount
                IITR:RP:^IITR442               = tns:Income.LabourHireArrangementNonPrimaryProductionPaymentGross.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS21 = tns:Remuneration.LabourHireArrangementPaymentGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS21.GetValueOrDefault()) != (report.iitrParentDocument.IITR441.GetValueOrDefault() + report.iitrParentDocument.IITR442.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310037",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payments - labour hire or other payments is incorrect",
                        LongDescription = @"The sum of Gross payment for Labour hire or other specified payments withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments labour hire or other specified payments and Non-PP - Gross payments labour hire or other specified payments on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.LabourHireArrangementPrimaryProductionPaymentGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310037" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR441", Value = report.iitrParentDocument.IITR441.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR442", Value = report.iitrParentDocument.IITR442.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS21", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS21.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310037

                #region VR.ATO.IITR.310038

                /*  VR.ATO.IITR.310038
                The sum of Gross payment for foreign resident withholding in the Payment Summary schedule where Income Type is set to Business or Personal Services Income, should be equal to the Non-PP - Gross payments subject to foreign resident withholding (excluding capital gains) on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss18] WHERE [pss15] = SET("Business", "Personal Services Income")) <> [iitr438])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR438               = tns:Income.ForeignResidentWithholdingNonPrimaryProduction.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS18 = tns:Remuneration.PaymentToForeignResidentGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").Sum(context => context.PSS18.GetValueOrDefault()) != report.iitrParentDocument.IITR438.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.311038",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payment - Foreign Resident Withholding is incorrect",
                        LongDescription = @"The sum of Gross payment for foreign resident withholding in the Payment Summary schedule where Income Type is set to Business or Personal Services Income, should be equal to the Non-PP - Gross payments subject to foreign resident withholding (excluding capital gains) on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.ForeignResidentWithholdingNonPrimaryProduction.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310038" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR438", Value = report.iitrParentDocument.IITR438.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS18", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").Sum(context => context.PSS18.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310038

                #region VR.ATO.IITR.310039

                /*  VR.ATO.IITR.310039
                The amount shown at PSI - voluntary agreement is not equal to the sum of Gross payment for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND ((([iitr411] <> NULL) AND [iitr411] <> (SUM([pss20] WHERE [pss15] = "Personal Services Income"))) OR (([iitr411] = NULL) AND (SUM([pss20] WHERE [pss15] = "Personal Services Income") <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR411               = tns:Income.PersonalServicesIncome.VoluntaryAgreement.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS20 = tns:Remuneration.VoluntaryAgreementGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR411 != null && report.iitrParentDocument.IITR411 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS20.GetValueOrDefault())) || (report.iitrParentDocument.IITR411 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS20.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310039",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI - voluntary agreement is incorrect",
                        LongDescription = @"The amount shown at PSI - voluntary agreement is not equal to the sum of Gross payment for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.VoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310039" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR411", Value = report.iitrParentDocument.IITR411.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS20", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS20.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310039

                #region VR.ATO.IITR.310040

                /*  VR.ATO.IITR.310040
                The amount shown at PSI where Australian business number not quoted is not equal to the sum of Gross payment for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND ((([iitr412] <> NULL) AND [iitr412] <> (SUM([pss19] WHERE [pss15] = "Personal Services Income"))) OR (([iitr412] = NULL) AND (SUM([pss19] WHERE [pss15] = "Personal Services Income") <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR412               = tns:Income.PersonalServicesIncome.ABNNotQuoted.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS19 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR412 != null && report.iitrParentDocument.IITR412 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS19.GetValueOrDefault())) || (report.iitrParentDocument.IITR412 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS19.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310040",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI where ABN not quoted is incorrect",
                        LongDescription = @"The amount shown at PSI where Australian business number not quoted is not equal to the sum of Gross payment for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.ABNNotQuoted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310040" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR412", Value = report.iitrParentDocument.IITR412.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS19", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS19.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310040

                #region VR.ATO.IITR.310041

                /*  VR.ATO.IITR.310041
                The amount shown at PSI - labour hire or other specified payments is not equal to the sum of Gross payment for Labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND ((([iitr413] <> NULL) AND [iitr413] <> (SUM([pss21] WHERE [pss15] = "Personal Services Income"))) OR (([iitr413] = NULL) AND (SUM([pss21] WHERE [pss15] = "Personal Services Income") <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR413               = tns:Income.PersonalServicesIncome.LabourHireOrOtherSpecifiedPayment.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS21 = tns:Remuneration.LabourHireArrangementPaymentGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR413 != null && report.iitrParentDocument.IITR413 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS21.GetValueOrDefault())) || (report.iitrParentDocument.IITR413 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS21.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310041",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI - labour hire and other specified payments is incorrect",
                        LongDescription = @"The amount shown at PSI - labour hire or other specified payments is not equal to the sum of Gross payment for Labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.LabourHireOrOtherSpecifiedPayment.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310041" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR413", Value = report.iitrParentDocument.IITR413.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS21", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS21.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310041

                #region Repeating RPPayerPSSeqNumContext

                if (pssChildDocument.RPPayerPSSeqNumContextCollection != null)
                {
                    foreach (PSS2018.RPPayerPSSeqNumContext pssPayerContext in pssChildDocument.RPPayerPSSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.310088

                        /*  VR.ATO.IITR.310088
                        When a Payment summary schedule for individuals is attached, the 'Income Type: Business or Personal Services Business' codes on the schedule must be provided.

                        Legacy Rule Format:
                        IF COUNT(SCHEDULE = "PSS") = 1 AND (ANY OCCURRENCE OF [pss15] = NULL)
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                        */
                        assertion = (pssPayerContext.PSS15 == null);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.310088",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Income Type: Business or Personal Services is required",
                                LongDescription = @"When a Payment summary schedule for individuals is attached, the 'Income Type: Business or Personal Services Business' codes on the schedule must be provided",
                                Location = "/xbrli:xbrl/tns:IncomeTax.IncomeType.Code[@contextRef='" + pssPayerContext.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310088" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "PSS15", Value = "null" });

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.310088

                        #region VR.ATO.IITR.400059

                        /*  VR.ATO.IITR.400059
                        Invalid context. Reporting Party Payer period start date must be the same as the Reporting Party Period start date.

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "PSS") = 1) AND (RP:period.startDate <> PSS:RP.Payer.{PSSeqNum}:period.StartDate)
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^StartDate                 = RP
                        PSS:RP.Payer.{PSSeqNum}:^StartDate = RP
                        */
                        assertion = (pssPayerContext.StartDate != report.iitrParentDocument.RPStartDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.400059",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Reporting Party Payer Period start date is invalid",
                                LongDescription = @"Invalid context. Reporting Party Payer period start date must be the same as the Reporting Party Period start date.",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + pssPayerContext.Id + "']/xbrli:period/xbrli:startDate",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.400059" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.400059

                        #region VR.ATO.IITR.400060

                        /*  VR.ATO.IITR.400060
                        Invalid context. Reporting Party Payer period end date must be the same as the Reporting Party Period end date.

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "PSS") = 1) AND (RP:period.startDate <> PSS:RP.Payer.{PSSeqNum}:period.StartDate)
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^EndDate                 = RP
                        PSS:RP.Payer.{PSSeqNum}:^EndDate = RP
                        */
                        assertion = (pssPayerContext.EndDate != report.iitrParentDocument.RPEndDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.400060",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Reporting Party Payer Period end date is invalid",
                                LongDescription = @"Invalid context. Reporting Party Payer period end date must be the same as the Reporting Party Period end date.",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + pssPayerContext.Id + "']/xbrli:period/xbrli:endDate",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.400060" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.400060

                        #region VR.ATO.PSS.000036

                        /*  VR.ATO.PSS.000036
                        Your supplied TFN does not match the TFN supplied on the form it was submitted with

                        Legacy Rule Format:
                        WHERE IN CONTEXT (RP.Payer.{PSSeqNum})
                        IF (RP.Payer.{PSSeqNum}.entity.identifier.TFN <> PARENT RETURN:RP.entity.identifier.TFN)
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^EntityTFN                 = RP
                        PSS:RP.Payer.{PSSeqNum}:^EntityTFN = RP
                        */
                        assertion = (pssPayerContext.IdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.PSS.000036",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + pssPayerContext.Id + "']/xbrli:entity/xbrli:identifier",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000036" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.PSS.000036
                    }
                }

                #endregion // Repeating RPPayerPSSeqNumContext
            }

            #endregion // PSS Child Present

            // ####################################################################################################
            //                                       RNTLPRPTY CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.730313

            /*  VR.ATO.IITR.730313
            The number of Multi-Property Rental schedules must not exceed 1

            Legacy Rule Format:
            IF COUNT(SCHEDULE="RNTLPRPTY") > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            */
            assertion = Count(report.rntlprptyChildDocuments) > 1;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.730313",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Multi-Property Rental schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730313" } }
                };

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.730313

            #region RNTLPRPTY Child Present

            if (report.rntlprptyChildDocuments != null && Count(report.rntlprptyChildDocuments) == 1)
            {
                RNTLPRPTY2019 rntlprptyChildDocument = report.rntlprptyChildDocuments[0];

                #region Helper Variables

                // String array variables to contain lists of contexts with 100% share and contexts with less than 100% share
                string[] baseSequencesFullShare = report.rntlprptyChildDocuments[0].RPPrptySeqNumContextCollection.FindAll(context => context.RNTLPRPTY104 == 100).Select(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue).ToArray();
                string[] baseSequencesPartShare = report.rntlprptyChildDocuments[0].RPPrptySeqNumContextCollection.FindAll(context => context.RNTLPRPTY104 < 100).Select(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue).ToArray();

                #endregion

                #region VR.ATO.IITR.730214

                /* VR.ATO.IITR.730214
                When any Rental income, Other rental related income, Your share of rental income, or Your share of other rental related income amount is present on the attached Multi-Property Rental schedule, then Gross rent must be provided

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr334] = NULL AND (ANY OCCURRENCE OF([rntlprpty114]) <> NULL OR ANY OCCURRENCE OF([rntlprpty115]) <> NULL OR ANY OCCURRENCE OF([rntlprpty136]) <> NULL OR ANY OCCURRENCE OF([rntlprpty137]) <> NULL)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR334                                 = tns:Income.RealEstateProperty.Rental.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY114   = tns:Income.RealEstateProperty.Rental.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY115   = tns:Income.RealEstateProperty.RentalRelatedOther.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY136 = tns:Income.RealEstateProperty.Rental.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY137 = tns:Income.RealEstateProperty.RentalRelatedOther.Amount
                */
                assertion = report.iitrParentDocument.IITR334 == null && ((report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Any(context => context.RNTLPRPTY114 != null || context.RNTLPRPTY115 != null) : false) || (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Any(context => context.RNTLPRPTY136 != null || context.RNTLPRPTY137 != null) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730214",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross rent must be provided",
                        LongDescription = @"When any Rental income, Other rental related income, Your share of rental income, or Your share of other rental related income amount is present on the attached Multi-Property Rental schedule, then Gross rent must be provided",
                        Location = "/xbrli:xbrl/tns:Income.RealEstateProperty.Rental.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730214" } },
                    };

                    RNTLPRPTY2019.RPPrptySeqNumTotalContext totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RNTLPRPTY114 != null || context.RNTLPRPTY115 != null) : null);
                    RNTLPRPTY2019.RPPrptySeqNumPortionContext portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RNTLPRPTY136 != null || context.RNTLPRPTY137 != null) : null);

                    if (totalContext != null && portionContext != null)
                    {
                        if (Convert.ToInt16(totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) < Convert.ToInt16(portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue))
                            portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                        else
                            totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                    }

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR334", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY114", Value = (totalContext != null && totalContext.RNTLPRPTY114.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY114) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY115", Value = (totalContext != null && totalContext.RNTLPRPTY115.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY115) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY136", Value = (portionContext != null && portionContext.RNTLPRPTY136.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY136) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY137", Value = (portionContext != null && portionContext.RNTLPRPTY137.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY137) : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730214

                #region VR.ATO.IITR.730215

                /* VR.ATO.IITR.730215
                Gross rent must equal the sum of all Rental income and Other rental income, where the Ownership percentage is 100%, plus the sum of all Your share of rental income and Your share of other rental related income, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr334] <> NULL AND [iitr334] <> RoundDown(SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty114], [rntlprpty136])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty115], [rntlprpty137])))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR334                                 = tns:Income.RealEstateProperty.Rental.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY114   = tns:Income.RealEstateProperty.Rental.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY115   = tns:Income.RealEstateProperty.RentalRelatedOther.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY136 = tns:Income.RealEstateProperty.Rental.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY137 = tns:Income.RealEstateProperty.RentalRelatedOther.Amount
                */
                decimal rntlprpty114Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.FindAll(context => baseSequencesFullShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY114.GetValueOrDefault()) : 0);
                decimal rntlprpty115Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.FindAll(context => baseSequencesFullShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY115.GetValueOrDefault()) : 0);
                decimal rntlprpty136Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.FindAll(context => baseSequencesPartShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY136.GetValueOrDefault()) : 0);
                decimal rntlprpty137Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.FindAll(context => baseSequencesPartShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY137.GetValueOrDefault()) : 0);
                decimal vr730215Total = Math.Truncate(rntlprpty114Total + rntlprpty115Total + rntlprpty136Total + rntlprpty137Total);

                assertion = report.iitrParentDocument.IITR334 != null && report.iitrParentDocument.IITR334.GetValueOrDefault() != vr730215Total;
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730215",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross rent amount is incorrect",
                        LongDescription = @"Gross rent must equal the sum of all Rental income and Other rental income, where the Ownership percentage is 100%, plus the sum of all Your share of rental income and Your share of other rental related income, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule",
                        Location = "/xbrli:xbrl/tns:Income.RealEstateProperty.Rental.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730215" } },
                    };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY114) WHERE RNTLPRPTY104 = 100", Value = rntlprpty114Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY115) WHERE RNTLPRPTY104 = 100", Value = rntlprpty115Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY136) WHERE RNTLPRPTY104 < 100", Value = rntlprpty136Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY137) WHERE RNTLPRPTY104 < 100", Value = rntlprpty137Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Gross Rent Total", Value = vr730215Total.ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730215

                #region VR.ATO.IITR.730216

                /* VR.ATO.IITR.730216
                When any Interest on loans, or Your share of interest on loans amount is present on the attached Multi-Property Rental schedule, then Interest deductions must be provided

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr335] = NULL AND (ANY OCCURRENCE OF([rntlprpty124]) <> NULL OR ANY OCCURRENCE OF([rntlprpty146]) <> NULL)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR335                                 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY124   = tns:Expense.RealEstateProperty.LoanInterest.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY146 = tns:Expense.RealEstateProperty.LoanInterest.Amount
                */
                assertion = report.iitrParentDocument.IITR335 == null && ((report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Any(context => context.RNTLPRPTY124 != null) : false) || (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Any(context => context.RNTLPRPTY146 != null) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730216",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Interest deductions must be provided",
                        LongDescription = @"When any Interest on loans, or Your share of interest on loans amount is present on the attached Multi-Property Rental schedule, then Interest deductions must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730216" } },
                    };

                    RNTLPRPTY2019.RPPrptySeqNumTotalContext totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RNTLPRPTY124 != null) : null);
                    RNTLPRPTY2019.RPPrptySeqNumPortionContext portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RNTLPRPTY146 != null) : null);

                    if (totalContext != null && portionContext != null)
                    {
                        if (Convert.ToInt16(totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) < Convert.ToInt16(portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue))
                            portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                        else
                            totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                    }

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR335", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY124", Value = (totalContext != null && totalContext.RNTLPRPTY124.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY124) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY146", Value = (portionContext != null && portionContext.RNTLPRPTY146.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY146) : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730216

                #region VR.ATO.IITR.730217

                /* VR.ATO.IITR.730217
                Interest deductions must equal the sum of all Interest on loans, where the Ownership percentage is 100%, plus the sum of all Your share of interest on loans, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr335] <> NULL AND [iitr335] <> RoundDown(SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty124], [rntlprpty146])))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR335                                 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY124   = tns:Expense.RealEstateProperty.LoanInterest.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY146 = tns:Expense.RealEstateProperty.LoanInterest.Amount
                */
                decimal rntlprpty124Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.FindAll(context => baseSequencesFullShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY124.GetValueOrDefault()) : 0);
                decimal rntlprpty146Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.FindAll(context => baseSequencesPartShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY146.GetValueOrDefault()) : 0);
                decimal vr730217Total = Math.Truncate(rntlprpty124Total + rntlprpty146Total);

                assertion = report.iitrParentDocument.IITR335 != null && report.iitrParentDocument.IITR335.GetValueOrDefault() != vr730217Total;
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730217",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Interest deductions amount is incorrect",
                        LongDescription = @"Interest deductions must equal the sum of all Interest on loans, where the Ownership percentage is 100%, plus the sum of all Your share of interest on loans, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730217" } },
                    };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY124) WHERE RNTLPRPTY104 = 100", Value = rntlprpty124Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY146) WHERE RNTLPRPTY104 < 100", Value = rntlprpty146Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Interest on Loans Total", Value = vr730217Total.ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730217

                #region VR.ATO.IITR.730218

                /* VR.ATO.IITR.730218
                When any Capital works deductions, or Your share of capital works deductions amount is present on the attached Multi-Property Rental schedule, then Capital works deductions must be provided

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr336] = NULL AND (ANY OCCURRENCE OF([rntlprpty130]) <> NULL OR ANY OCCURRENCE OF([rntlprpty152]) <> NULL)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR336                                 = tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY130   = tns:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY152 = tns:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                */
                assertion = report.iitrParentDocument.IITR336 == null && ((report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Any(context => context.RNTLPRPTY130 != null) : false) || (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Any(context => context.RNTLPRPTY152 != null) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730218",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Capital works deductions must be provided",
                        LongDescription = @"When any Capital works deductions, or Your share of capital works deductions amount is present on the attached Multi-Property Rental schedule, then Capital works deductions must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730218" } },
                    };

                    RNTLPRPTY2019.RPPrptySeqNumTotalContext totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RNTLPRPTY130 != null) : null);
                    RNTLPRPTY2019.RPPrptySeqNumPortionContext portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RNTLPRPTY152 != null) : null);

                    if (totalContext != null && portionContext != null)
                    {
                        if (Convert.ToInt16(totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) < Convert.ToInt16(portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue))
                            portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                        else
                            totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                    }

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR336", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY130", Value = (totalContext != null && totalContext.RNTLPRPTY130.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY130) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY152", Value = (portionContext != null && portionContext.RNTLPRPTY152.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY152) : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730218

                #region VR.ATO.IITR.730219

                /* VR.ATO.IITR.730219
                Capital works deductions must equal the sum of all Capital works deductions, where the Ownership percentage is 100%, plus the sum of all Your share capital works deductions, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr336] <> NULL AND [iitr336] <> RoundDown(SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty130], [rntlprpty152])))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR336                                 = tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY130   = tns:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY152 = tns:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                */
                decimal rntlprpty130Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.FindAll(context => baseSequencesFullShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY130.GetValueOrDefault()) : 0);
                decimal rntlprpty152Total = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.FindAll(context => baseSequencesPartShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)).Sum(context => context.RNTLPRPTY152.GetValueOrDefault()) : 0);
                decimal vr730219Total = Math.Truncate(rntlprpty130Total + rntlprpty152Total);

                assertion = report.iitrParentDocument.IITR336 != null && report.iitrParentDocument.IITR336.GetValueOrDefault() != vr730219Total;
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730219",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Capital works deductions amount is incorrect",
                        LongDescription = @"Capital works deductions must equal the sum of all Capital works deductions, where the Ownership percentage is 100%, plus the sum of all Your share capital works deductions, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730219" } },
                    };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR336", Value = report.iitrParentDocument.IITR336.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY130) WHERE RNTLPRPTY104 = 100", Value = rntlprpty130Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY152) WHERE RNTLPRPTY104 < 100", Value = rntlprpty152Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Capital Works Deductions Total", Value = vr730219Total.ToString() });
                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730219

                #region VR.ATO.IITR.730220

                /* VR.ATO.IITR.730220
                When any other rental deductions, or your share of other rental deductions amount is present on the attached Multi-Property Rental schedule, then Other rental deductions must be provided

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr337] = NULL AND (ANY OCCURRENCE OF([rntlprpty116]) <> NULL OR ANY OCCURRENCE OF([rntlprpty117]) <> NULL OR ANY OCCURRENCE OF([rntlprpty118]) <> NULL OR ANY OCCURRENCE OF([rntlprpty119]) <> NULL OR ANY OCCURRENCE OF([rntlprpty120]) <> NULL OR ANY OCCURRENCE OF([rntlprpty121]) <> NULL OR ANY OCCURRENCE OF([rntlprpty122]) <> NULL OR ANY OCCURRENCE OF([rntlprpty123]) <> NULL OR ANY OCCURRENCE OF([rntlprpty125]) <> NULL OR ANY OCCURRENCE OF([rntlprpty126]) <> NULL OR ANY OCCURRENCE OF([rntlprpty127]) <> NULL OR ANY OCCURRENCE OF([rntlprpty128]) <> NULL OR ANY OCCURRENCE OF([rntlprpty129]) <> NULL OR ANY OCCURRENCE OF([rntlprpty131]) <> NULL OR ANY OCCURRENCE OF([rntlprpty132]) <> NULL OR ANY OCCURRENCE OF([rntlprpty133]) <> NULL OR ANY OCCURRENCE OF([rntlprpty134]) <> NULL OR ANY OCCURRENCE OF([rntlprpty138]) <> NULL OR ANY OCCURRENCE OF([rntlprpty139]) <> NULL OR ANY OCCURRENCE OF([rntlprpty140]) <> NULL OR ANY OCCURRENCE OF([rntlprpty141]) <> NULL OR ANY OCCURRENCE OF([rntlprpty142]) <> NULL OR ANY OCCURRENCE OF([rntlprpty143]) <> NULL OR ANY OCCURRENCE OF([rntlprpty144]) <> NULL OR ANY OCCURRENCE OF([rntlprpty145]) <> NULL OR ANY OCCURRENCE OF([rntlprpty147]) <> NULL OR ANY OCCURRENCE OF([rntlprpty148]) <> NULL OR ANY OCCURRENCE OF([rntlprpty149]) <> NULL OR ANY OCCURRENCE OF([rntlprpty150]) <> NULL OR ANY OCCURRENCE OF([rntlprpty151]) <> NULL OR ANY OCCURRENCE OF([rntlprpty153]) <> NULL OR ANY OCCURRENCE OF([rntlprpty154]) <> NULL OR ANY OCCURRENCE OF([rntlprpty155]) <> NULL OR ANY OCCURRENCE OF([rntlprpty156]) <> NULL)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR337                                 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY116   = tns:Expense.RealEstateProperty.Marketing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY117   = tns:Expense.RealEstateProperty.BodyCorporate.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY118   = tns:Expense.RealEstateProperty.Borrowing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY119   = tns:Expense.RealEstateProperty.Cleaning.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY120   = tns:Expense.RealEstateProperty.CouncilRates.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY121   = tns:Expense.DepreciationandAmortisation.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY122   = tns:Expense.RealEstateProperty.Gardening.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY123   = tns:Expense.RealEstateProperty.Insurance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY125   = tns:Expense.LandTax.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY126   = tns:Expense.RealEstateProperty.LegalFees.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY127   = tns:Expense.RealEstateProperty.PestControl.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY128   = tns:Expense.RealEstateProperty.AgentFeesCommission.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY129   = tns:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY131   = tns:Expense.RealEstateProperty.OfficeSupplies.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY132   = tns:Expense.Travel.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY133   = tns:Expense.RealEstateProperty.Water.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY134   = tns:Expense.RealEstateProperty.Sundry.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY138   = tns:Expense.RealEstateProperty.Marketing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY139   = tns:Expense.RealEstateProperty.BodyCorporate.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY140   = tns:Expense.RealEstateProperty.Borrowing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY141   = tns:Expense.RealEstateProperty.Cleaning.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY142   = tns:Expense.RealEstateProperty.CouncilRates.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY143   = tns:Expense.DepreciationandAmortisation.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY144   = tns:Expense.RealEstateProperty.Gardening.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY145   = tns:Expense.RealEstateProperty.Insurance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY147   = tns:Expense.LandTax.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY148   = tns:Expense.RealEstateProperty.LegalFees.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY149   = tns:Expense.RealEstateProperty.PestControl.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY150   = tns:Expense.RealEstateProperty.AgentFeesCommission.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY151   = tns:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY153   = tns:Expense.RealEstateProperty.OfficeSupplies.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY154   = tns:Expense.Travel.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY155   = tns:Expense.RealEstateProperty.Water.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY156   = tns:Expense.RealEstateProperty.Sundry.Amount
                */
                assertion = report.iitrParentDocument.IITR337 == null && ((report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Any(context => context.RNTLPRPTY116 != null || context.RNTLPRPTY117 != null || context.RNTLPRPTY118 != null || context.RNTLPRPTY119 != null || context.RNTLPRPTY120 != null || context.RNTLPRPTY121 != null || context.RNTLPRPTY122 != null || context.RNTLPRPTY123 != null || context.RNTLPRPTY125 != null || context.RNTLPRPTY126 != null || context.RNTLPRPTY127 != null || context.RNTLPRPTY128 != null || context.RNTLPRPTY129 != null || context.RNTLPRPTY131 != null || context.RNTLPRPTY132 != null || context.RNTLPRPTY133 != null || context.RNTLPRPTY134 != null) : false) || (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Any(context => context.RNTLPRPTY138 != null || context.RNTLPRPTY139 != null || context.RNTLPRPTY140 != null || context.RNTLPRPTY141 != null || context.RNTLPRPTY142 != null || context.RNTLPRPTY143 != null || context.RNTLPRPTY144 != null || context.RNTLPRPTY145 != null || context.RNTLPRPTY147 != null || context.RNTLPRPTY148 != null || context.RNTLPRPTY149 != null || context.RNTLPRPTY150 != null || context.RNTLPRPTY151 != null || context.RNTLPRPTY153 != null || context.RNTLPRPTY154 != null || context.RNTLPRPTY155 != null || context.RNTLPRPTY156 != null) : false));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730220",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other rental deductions must be provided",
                        LongDescription = @"When any other rental deductions, or your share of other rental deductions amount is present on the attached Multi-Property Rental schedule, then Other rental deductions must be provided",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730220" } },
                    };

                    #region ProcessMessage Parameters

                    // Get the first RP.{PrptySeqNum}.Total and RP.{PrptySeqNum}.Portion context that contains an Other rental deductions field that is not null
                    RNTLPRPTY2019.RPPrptySeqNumTotalContext totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RNTLPRPTY116 != null || context.RNTLPRPTY117 != null || context.RNTLPRPTY118 != null || context.RNTLPRPTY119 != null || context.RNTLPRPTY120 != null || context.RNTLPRPTY121 != null || context.RNTLPRPTY122 != null || context.RNTLPRPTY123 != null || context.RNTLPRPTY125 != null || context.RNTLPRPTY126 != null || context.RNTLPRPTY127 != null || context.RNTLPRPTY128 != null || context.RNTLPRPTY129 != null || context.RNTLPRPTY131 != null || context.RNTLPRPTY132 != null || context.RNTLPRPTY133 != null || context.RNTLPRPTY134 != null) : null);
                    RNTLPRPTY2019.RPPrptySeqNumPortionContext portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RNTLPRPTY138 != null || context.RNTLPRPTY139 != null || context.RNTLPRPTY140 != null || context.RNTLPRPTY141 != null || context.RNTLPRPTY142 != null || context.RNTLPRPTY143 != null || context.RNTLPRPTY144 != null || context.RNTLPRPTY145 != null || context.RNTLPRPTY147 != null || context.RNTLPRPTY148 != null || context.RNTLPRPTY149 != null || context.RNTLPRPTY150 != null || context.RNTLPRPTY151 != null || context.RNTLPRPTY153 != null || context.RNTLPRPTY154 != null || context.RNTLPRPTY155 != null || context.RNTLPRPTY156 != null) : null);

                    // Check which of the above contexts has the lowest sequence number and retrieve the other context that has a matching sequence number (if applicable in the case of a RP.{PrptySeqNum}.Portion context)
                    if (totalContext != null && portionContext != null)
                    {
                        if (Convert.ToInt16(totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) < Convert.ToInt16(portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue))
                            portionContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == totalContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                        else
                            totalContext = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.Find(context => context.RentalPropertyFinancialDetailsSequenceDimensionValue == portionContext.RentalPropertyFinancialDetailsSequenceDimensionValue) : null);
                    }
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR337", Value = "null" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY116", Value = (totalContext != null && totalContext.RNTLPRPTY116.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY116) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY117", Value = (totalContext != null && totalContext.RNTLPRPTY117.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY117) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY118", Value = (totalContext != null && totalContext.RNTLPRPTY118.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY118) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY119", Value = (totalContext != null && totalContext.RNTLPRPTY119.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY119) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY120", Value = (totalContext != null && totalContext.RNTLPRPTY120.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY120) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY121", Value = (totalContext != null && totalContext.RNTLPRPTY121.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY121) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY122", Value = (totalContext != null && totalContext.RNTLPRPTY122.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY122) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY123", Value = (totalContext != null && totalContext.RNTLPRPTY123.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY123) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY125", Value = (totalContext != null && totalContext.RNTLPRPTY125.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY125) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY126", Value = (totalContext != null && totalContext.RNTLPRPTY126.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY126) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY127", Value = (totalContext != null && totalContext.RNTLPRPTY127.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY127) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY128", Value = (totalContext != null && totalContext.RNTLPRPTY128.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY128) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY129", Value = (totalContext != null && totalContext.RNTLPRPTY129.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY129) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY131", Value = (totalContext != null && totalContext.RNTLPRPTY131.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY131) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY132", Value = (totalContext != null && totalContext.RNTLPRPTY132.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY132) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY133", Value = (totalContext != null && totalContext.RNTLPRPTY133.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY133) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY134", Value = (totalContext != null && totalContext.RNTLPRPTY134.HasValue ? GetValueOrEmpty(totalContext.RNTLPRPTY134) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY138", Value = (portionContext != null && portionContext.RNTLPRPTY138.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY138) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY139", Value = (portionContext != null && portionContext.RNTLPRPTY139.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY139) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY140", Value = (portionContext != null && portionContext.RNTLPRPTY140.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY140) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY141", Value = (portionContext != null && portionContext.RNTLPRPTY141.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY141) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY142", Value = (portionContext != null && portionContext.RNTLPRPTY142.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY142) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY143", Value = (portionContext != null && portionContext.RNTLPRPTY143.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY143) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY144", Value = (portionContext != null && portionContext.RNTLPRPTY144.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY144) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY145", Value = (portionContext != null && portionContext.RNTLPRPTY145.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY145) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY147", Value = (portionContext != null && portionContext.RNTLPRPTY147.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY147) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY148", Value = (portionContext != null && portionContext.RNTLPRPTY148.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY148) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY149", Value = (portionContext != null && portionContext.RNTLPRPTY149.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY149) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY150", Value = (portionContext != null && portionContext.RNTLPRPTY150.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY150) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY151", Value = (portionContext != null && portionContext.RNTLPRPTY151.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY151) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY153", Value = (portionContext != null && portionContext.RNTLPRPTY153.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY153) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY154", Value = (portionContext != null && portionContext.RNTLPRPTY154.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY154) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY155", Value = (portionContext != null && portionContext.RNTLPRPTY155.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY155) : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RNTLPRPTY156", Value = (portionContext != null && portionContext.RNTLPRPTY156.HasValue ? GetValueOrEmpty(portionContext.RNTLPRPTY156) : "null") });

                    #endregion

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730220

                #region VR.ATO.IITR.730221

                /* VR.ATO.IITR.730221
                Other rental deductions must equal the sum of all other rental deductions, where the Ownership percentage is 100%, plus the sum of all your share of other rental deductions, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND [iitr337] <> NULL AND [iitr337] <> RoundDown(SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty116], [rntlprpty138])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty117], [rntlprpty139])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty118], [rntlprpty140])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty119], [rntlprpty141])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty120], [rntlprpty142])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty121], [rntlprpty143])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty122], [rntlprpty144])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty123], [rntlprpty145])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty125], [rntlprpty147])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty126], [rntlprpty148])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty127], [rntlprpty149])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty128], [rntlprpty150])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty129], [rntlprpty151])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty131], [rntlprpty153])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty132], [rntlprpty154])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty133], [rntlprpty155])) + SUM(ConditionalValue([rntlprpty104] = 100, [rntlprpty134], [rntlprpty156])))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR337                                 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY116   = tns:Expense.RealEstateProperty.Marketing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY117   = tns:Expense.RealEstateProperty.BodyCorporate.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY118   = tns:Expense.RealEstateProperty.Borrowing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY119   = tns:Expense.RealEstateProperty.Cleaning.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY120   = tns:Expense.RealEstateProperty.CouncilRates.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY121   = tns:Expense.DepreciationandAmortisation.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY122   = tns:Expense.RealEstateProperty.Gardening.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY123   = tns:Expense.RealEstateProperty.Insurance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY125   = tns:Expense.LandTax.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY126   = tns:Expense.RealEstateProperty.LegalFees.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY127   = tns:Expense.RealEstateProperty.PestControl.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY128   = tns:Expense.RealEstateProperty.AgentFeesCommission.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY129   = tns:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY131   = tns:Expense.RealEstateProperty.OfficeSupplies.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY132   = tns:Expense.Travel.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY133   = tns:Expense.RealEstateProperty.Water.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Total:^RNTLPRPTY134   = tns:Expense.RealEstateProperty.Sundry.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY138   = tns:Expense.RealEstateProperty.Marketing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY139   = tns:Expense.RealEstateProperty.BodyCorporate.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY140   = tns:Expense.RealEstateProperty.Borrowing.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY141   = tns:Expense.RealEstateProperty.Cleaning.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY142   = tns:Expense.RealEstateProperty.CouncilRates.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY143   = tns:Expense.DepreciationandAmortisation.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY144   = tns:Expense.RealEstateProperty.Gardening.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY145   = tns:Expense.RealEstateProperty.Insurance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY147   = tns:Expense.LandTax.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY148   = tns:Expense.RealEstateProperty.LegalFees.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY149   = tns:Expense.RealEstateProperty.PestControl.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY150   = tns:Expense.RealEstateProperty.AgentFeesCommission.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY151   = tns:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY153   = tns:Expense.RealEstateProperty.OfficeSupplies.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY154   = tns:Expense.Travel.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY155   = tns:Expense.RealEstateProperty.Water.Amount
                RNTLPRPTY:RP.{PrptySeqNum}.Portion:^RNTLPRPTY156   = tns:Expense.RealEstateProperty.Sundry.Amount
                */
                // Get every RP.{PrptySeqNum}.Total context where the Ownership percentage in the base context = 100%
                List<RNTLPRPTY2019.RPPrptySeqNumTotalContext> totalContexts = (report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumTotalContextCollection.FindAll(context => baseSequencesFullShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)) : null);
                // Get every RP.{PrptySeqNum}.Portion context where the Ownership percentage in the base context < 100%
                List<RNTLPRPTY2019.RPPrptySeqNumPortionContext> portionContexts = (report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection != null ? report.rntlprptyChildDocuments[0].RPPrptySeqNumPortionContextCollection.FindAll(context => baseSequencesPartShare.Contains(context.RentalPropertyFinancialDetailsSequenceDimensionValue)) : null);

                #region Sum Totals

                decimal rntlprpty116Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY116.GetValueOrDefault()) : 0);
                decimal rntlprpty117Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY117.GetValueOrDefault()) : 0);
                decimal rntlprpty118Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY118.GetValueOrDefault()) : 0);
                decimal rntlprpty119Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY119.GetValueOrDefault()) : 0);
                decimal rntlprpty120Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY120.GetValueOrDefault()) : 0);
                decimal rntlprpty121Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY121.GetValueOrDefault()) : 0);
                decimal rntlprpty122Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY122.GetValueOrDefault()) : 0);
                decimal rntlprpty123Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY123.GetValueOrDefault()) : 0);
                decimal rntlprpty125Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY125.GetValueOrDefault()) : 0);
                decimal rntlprpty126Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY126.GetValueOrDefault()) : 0);
                decimal rntlprpty127Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY127.GetValueOrDefault()) : 0);
                decimal rntlprpty128Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY128.GetValueOrDefault()) : 0);
                decimal rntlprpty129Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY129.GetValueOrDefault()) : 0);
                decimal rntlprpty131Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY131.GetValueOrDefault()) : 0);
                decimal rntlprpty132Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY132.GetValueOrDefault()) : 0);
                decimal rntlprpty133Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY133.GetValueOrDefault()) : 0);
                decimal rntlprpty134Total = (totalContexts != null ? totalContexts.Sum(context => context.RNTLPRPTY134.GetValueOrDefault()) : 0);
                decimal rntlprpty138Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY138.GetValueOrDefault()) : 0);
                decimal rntlprpty139Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY139.GetValueOrDefault()) : 0);
                decimal rntlprpty140Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY140.GetValueOrDefault()) : 0);
                decimal rntlprpty141Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY141.GetValueOrDefault()) : 0);
                decimal rntlprpty142Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY142.GetValueOrDefault()) : 0);
                decimal rntlprpty143Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY143.GetValueOrDefault()) : 0);
                decimal rntlprpty144Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY144.GetValueOrDefault()) : 0);
                decimal rntlprpty145Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY145.GetValueOrDefault()) : 0);
                decimal rntlprpty147Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY147.GetValueOrDefault()) : 0);
                decimal rntlprpty148Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY148.GetValueOrDefault()) : 0);
                decimal rntlprpty149Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY149.GetValueOrDefault()) : 0);
                decimal rntlprpty150Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY150.GetValueOrDefault()) : 0);
                decimal rntlprpty151Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY151.GetValueOrDefault()) : 0);
                decimal rntlprpty153Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY153.GetValueOrDefault()) : 0);
                decimal rntlprpty154Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY154.GetValueOrDefault()) : 0);
                decimal rntlprpty155Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY155.GetValueOrDefault()) : 0);
                decimal rntlprpty156Total = (portionContexts != null ? portionContexts.Sum(context => context.RNTLPRPTY156.GetValueOrDefault()) : 0);
                decimal vr730221Total = Math.Truncate(rntlprpty116Total + rntlprpty117Total + rntlprpty118Total + rntlprpty119Total + rntlprpty120Total + rntlprpty121Total + rntlprpty122Total + rntlprpty123Total + rntlprpty125Total + rntlprpty126Total + rntlprpty127Total + rntlprpty128Total + rntlprpty129Total + rntlprpty131Total + rntlprpty132Total + rntlprpty133Total + rntlprpty134Total + rntlprpty138Total + rntlprpty139Total + rntlprpty140Total + rntlprpty141Total + rntlprpty142Total + rntlprpty143Total + rntlprpty144Total + rntlprpty145Total + rntlprpty147Total + rntlprpty148Total + rntlprpty149Total + rntlprpty150Total + rntlprpty151Total + rntlprpty153Total + rntlprpty154Total + rntlprpty155Total + rntlprpty156Total);

                #endregion

                assertion = report.iitrParentDocument.IITR337 != null && report.iitrParentDocument.IITR337.GetValueOrDefault() != vr730221Total;
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.730221",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other rental deductions amount is incorrect",
                        LongDescription = @"Other rental deductions must equal the sum of all other rental deductions, where the Ownership percentage is 100%, plus the sum of all your share of other rental deductions, where the Ownership percentage is less than 100%, in the attached Multi-Property Rental schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730221" } },
                    };

                    #region ProcessMessage Parameters

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY116) WHERE RNTLPRPTY104 = 100", Value = rntlprpty116Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY117) WHERE RNTLPRPTY104 = 100", Value = rntlprpty117Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY118) WHERE RNTLPRPTY104 = 100", Value = rntlprpty118Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY119) WHERE RNTLPRPTY104 = 100", Value = rntlprpty119Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY120) WHERE RNTLPRPTY104 = 100", Value = rntlprpty120Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY121) WHERE RNTLPRPTY104 = 100", Value = rntlprpty121Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY122) WHERE RNTLPRPTY104 = 100", Value = rntlprpty122Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY123) WHERE RNTLPRPTY104 = 100", Value = rntlprpty123Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY125) WHERE RNTLPRPTY104 = 100", Value = rntlprpty125Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY126) WHERE RNTLPRPTY104 = 100", Value = rntlprpty126Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY127) WHERE RNTLPRPTY104 = 100", Value = rntlprpty127Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY128) WHERE RNTLPRPTY104 = 100", Value = rntlprpty128Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY129) WHERE RNTLPRPTY104 = 100", Value = rntlprpty129Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY131) WHERE RNTLPRPTY104 = 100", Value = rntlprpty131Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY132) WHERE RNTLPRPTY104 = 100", Value = rntlprpty132Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY133) WHERE RNTLPRPTY104 = 100", Value = rntlprpty133Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY134) WHERE RNTLPRPTY104 = 100", Value = rntlprpty134Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY138) WHERE RNTLPRPTY104 < 100", Value = rntlprpty138Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY139) WHERE RNTLPRPTY104 < 100", Value = rntlprpty139Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY140) WHERE RNTLPRPTY104 < 100", Value = rntlprpty140Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY141) WHERE RNTLPRPTY104 < 100", Value = rntlprpty141Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY142) WHERE RNTLPRPTY104 < 100", Value = rntlprpty142Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY143) WHERE RNTLPRPTY104 < 100", Value = rntlprpty143Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY144) WHERE RNTLPRPTY104 < 100", Value = rntlprpty144Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY145) WHERE RNTLPRPTY104 < 100", Value = rntlprpty145Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY147) WHERE RNTLPRPTY104 < 100", Value = rntlprpty147Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY148) WHERE RNTLPRPTY104 < 100", Value = rntlprpty148Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY149) WHERE RNTLPRPTY104 < 100", Value = rntlprpty149Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY150) WHERE RNTLPRPTY104 < 100", Value = rntlprpty150Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY151) WHERE RNTLPRPTY104 < 100", Value = rntlprpty151Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY153) WHERE RNTLPRPTY104 < 100", Value = rntlprpty153Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY154) WHERE RNTLPRPTY104 < 100", Value = rntlprpty154Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY155) WHERE RNTLPRPTY104 < 100", Value = rntlprpty155Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "SUM(RNTLPRPTY156) WHERE RNTLPRPTY104 < 100", Value = rntlprpty156Total.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "Calculated Other Rental Deductions Total", Value = vr730221Total.ToString() });

                    #endregion

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.730221

                #region Repeating RPPrptySeqNumContext
                if (rntlprptyChildDocument.RPPrptySeqNumContextCollection != null)
                {
                    foreach (RNTLPRPTY2019.RPPrptySeqNumContext rntlprptyContext in rntlprptyChildDocument.RPPrptySeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.730310

                        /* VR.ATO.IITR.730310
                        The TFN provided for the Multi-Property Rental schedule must be the same as the TFN of the parent form

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND (ANY OCCURRENCE OF(RNTLPRPTY:RP.{PrptySeqNum}:entity.identifier.TFN) <> RP:entity.identifier.TFN)
                           RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (rntlprptyContext.IdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.730310",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Tax file number invalid",
                                LongDescription = @"The TFN provided for the Multi-Property Rental schedule must be the same as the TFN of the parent form",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + rntlprptyContext.Id + "']/xbrli:entity/xbrli:identifier",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730310" } }
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.730310

                        #region VR.ATO.IITR.730311

                        /* VR.ATO.IITR.730311
                        Invalid context. Period start date for the Multi-Property Rental schedule must be the same as the Period start date of the parent form.

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND (ANY OCCURRENCE OF(RNTLPRPTY:RP.{PrptySeqNum}:period.startDate) <> RP:period.startDate)
                           RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (rntlprptyContext.StartDate != report.iitrParentDocument.RPStartDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.730311",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Period start date for the Multi-Property Rental schedule is invalid",
                                LongDescription = @"Invalid context. Period start date for the Multi-Property Rental schedule must be the same as the Period start date of the parent form.",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + rntlprptyContext.Id + "']/xbrli:period/xbrli:startDate",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730311" } }
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.730311

                        #region VR.ATO.IITR.730312

                        /* VR.ATO.IITR.730312
                        Invalid context. Period end date for the Multi-Property Rental schedule must be the same as the Period end date of the parent form.

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND (ANY OCCURRENCE OF(RNTLPRPTY:RP.{PrptySeqNum}:period.endDate) <> RP:period.endDate)
                           RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (rntlprptyContext.EndDate != report.iitrParentDocument.RPEndDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.730312",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Period end date for the Multi-Property Rental schedule is invalid",
                                LongDescription = @"Invalid context. Period end date for the Multi-Property Rental schedule must be the same as the Period end date of the parent form.",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + rntlprptyContext.Id + "']/xbrli:period/xbrli:endDate",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730312" } }
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.730312

                    }
                }
                #endregion // Repeating RPPrptySeqNumContext
            }

            #endregion // RNTLPRPTY Child Present

            // ####################################################################################################
            //                                          RS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.210049

            /*  VR.ATO.IITR.210049
            For each Rental Schedule submitted with this parent return, the only valid Entity Type is 117

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "RS") > 0 AND ANY OCCURRENCE OF [RS12] <> "117"
                RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            RS:RP:^RS12 = pyde.02.21:Party.Type.Code
            */
            assertion = (report.rsChildDocuments != null && report.rsChildDocuments.Count > 0 && report.rsChildDocuments.Any(doc => doc.RS12 != "117"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.210049",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For each Rental Schedule submitted with this parent return, the only valid Entity Type is 117",
                    Location = "/xbrli:xbrl/tns:Party.Type.Code[@contextRef='" + report.rsChildDocuments.Find(doc => doc.RS12 != "117").RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.210049" } },
                };

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.210049

            #region VR.ATO.IITR.310083

            /*  VR.ATO.IITR.310083
            If there is a positive amount at Gross rent or at any rental deductions item, then a Rental property schedule for each rental property or a Multi-Property Rental schedule must be provided

            Legacy Rule Format:
            IF ([iitr334] > 0 OR [iitr335] > 0 OR [iitr336] > 0 OR [iitr337] > 0) AND (COUNT(SCHEDULE = "RS") = 0 AND COUNT(SCHEDULE = "RNTLPRPTY") = 0)
                RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR334 = tns:Income.RealEstateProperty.Rental.Amount
            IITR:RP:^IITR335 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
            IITR:RP:^IITR336 = tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount
            IITR:RP:^IITR337 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = ((report.iitrParentDocument.IITR334 > 0 || report.iitrParentDocument.IITR335 > 0 || report.iitrParentDocument.IITR336 > 0 || report.iitrParentDocument.IITR337 > 0) && Count(report.rsChildDocuments) == 0 && Count(report.rntlprptyChildDocuments) == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310083",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"A rental schedule must be provided",
                    LongDescription = @"If there is a positive amount at Gross rent or at any rental deductions item, then a Rental property schedule for each rental property or a Multi-Property Rental schedule must be provided",
                    Location = "/xbrli:xbrl/tns:Income.RealEstateProperty.Rental.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310083" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR336", Value = report.iitrParentDocument.IITR336.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.310083

            #region VR.ATO.IITR.310084

            /*  VR.ATO.IITR.310084
            A rental schedule is attached AND rental income AND deduction amounts are not present in the return form. 

            Legacy Rule Format:
            IF (COUNT(SCHEDULE = "RS") > 0 OR COUNT(SCHEDULE = "RNTLPRPTY") = 1) AND ([iitr334] = NULL AND ([iitr335] = NULL OR [iitr335] = 0) AND ([iitr336] = NULL OR [iitr336] = 0) AND ([iitr337] = NULL OR [iitr337] = 0)) 
                RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR334 = tns:Income.RealEstateProperty.Rental.Amount
            IITR:RP:^IITR335 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
            IITR:RP:^IITR336 = tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount
            IITR:RP:^IITR337 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = ((Count(report.rsChildDocuments) > 0 || Count(report.rntlprptyChildDocuments) == 1) && (report.iitrParentDocument.IITR334 == null && (report.iitrParentDocument.IITR335 == null || report.iitrParentDocument.IITR335 == 0) && (report.iitrParentDocument.IITR336 == null || report.iitrParentDocument.IITR336 == 0) && (report.iitrParentDocument.IITR337 == null || report.iitrParentDocument.IITR337 == 0)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310084",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and rental deduction amounts must be present",
                    LongDescription = @"If a Rental property schedule or Multi-Property Rental schedule is present, Gross rent and rental deduction amounts must be present",
                    Location = "/xbrli:xbrl/tns:Income.RealEstateProperty.Rental.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310084" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR336", Value = report.iitrParentDocument.IITR336.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.310084

            #region RS Child Present

            if (report.rsChildDocuments != null && Count(report.rsChildDocuments) > 0)
            {
                foreach (RS2018 rsChildDocument in report.rsChildDocuments)
                {
                    #region VR.ATO.GEN.402009

                    /*  VR.ATO.GEN.402009
                    The Entity Identifier on the Schedule must equal the Entity Identifier on the Parent Return

                    Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    IITR:RP:^EntityTFN = RP
                    RS:RP:^EntityTFN   = RP
                    */
                    assertion = (rsChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.402009",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                            LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + rsChildDocument.RPId + "']/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } },
                        };

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.GEN.402009

                    #region VR.ATO.IITR.300153

                    /*  VR.ATO.IITR.300153
                    Invalid context. Period end date for the Rental schedule must be the same as the Period end date of the parent form.

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (ANY OCCURRENCE OF (RS:RP:period.endDate) <> RP:period.endDate)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    IITR:RP:^EndDate = RP
                    RS:RP:^EndDate   = RP
                    */
                    assertion = (rsChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.300153",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Period end date for the Rental schedule is invalid",
                            LongDescription = @"Invalid context. Period end date for the Rental schedule must be the same as the Period end date of the parent form.",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + rsChildDocument.RPId + "']/xbrli:period/xbrli:endDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300153" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.300153

                    #region VR.ATO.IITR.300155

                    /*  VR.ATO.IITR.300153
                    Invalid context. Period start date for the Rental schedule must be the same as the Period start date of the parent form.

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (ANY OCCURRENCE OF (RS:RP:period.startDate) <> RP:period.startDate)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    IITR:RP:^StartDate = RP
                    RS:RP:^StartDate   = RP
                    */
                    assertion = (rsChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.300155",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Period start date for the Rental schedule is invalid",
                            LongDescription = @"Invalid context. Period start date for the Rental schedule must be the same as the Period start date of the parent form.",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + rsChildDocument.RPId + "']/xbrli:period/xbrli:startDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300155" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.300155

                    #region VR.ATO.IITR.730324

                    /*  VR.ATO.IITR.730324
                    Total borrowing expenses claimed is very high. Borrowing expenses must be claimed over five years from when the loan was established and must not include the loan balance, interest or stamp duty on the transfer of the property.

                    Legacy Rule Format:
                    IF COUNT(SCHEDULE = "RS") > 0 AND ANY OCCURRENCE OF([RS42]) > 10000
                         RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    CountDocument('RS') > 0 AND AnyOccurrence(^RS42, ^RS42 > 10000)

                    Data Elements:

                    ^RS42 = bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
                    */
                    assertion = rsChildDocument.RS42.GetValueOrDefault() > 10000;
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.730324",
                            Severity = ProcessMessageSeverity.Warning,
                            Description = @"Check (total) Borrowing expenses",
                            LongDescription = @"Total borrowing expenses claimed is very high. Borrowing expenses must be claimed over five years from when the loan was established and must not include the loan balance, interest or stamp duty on the transfer of the property.",
                            Location = "/xbrli:xbrl/rs:Expense.RealEstateProperty.Borrowing.Amount[@contextRef='" + rsChildDocument.RPId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730324" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "RS42", Value = rsChildDocument.RS42.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.730324

                    #region VR.ATO.IITR.730325

                    /*  VR.ATO.IITR.730325
                    Only deductions for repairs and maintenance may be claimed that relate directly to wear and tear or other damage to rental properties and must not include capital works or depreciation

                    Legacy Rule Format:
                    IF COUNT(SCHEDULE = "RS") > 0 AND ANY OCCURRENCE OF([RS53]) > 3500
                         RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    CountDocument('RS') > 0 AND AnyOccurrence(^RS53, ^RS53 > 3500)

                    Data Elements:

                    ^RS53 = bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                    */
                    assertion = rsChildDocument.RS53.GetValueOrDefault() > 3500;
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.730325",
                            Severity = ProcessMessageSeverity.Warning,
                            Description = @"Check (total) Repairs and maintenance expenses",
                            LongDescription = @"Only deductions for repairs and maintenance may be claimed that relate directly to wear and tear or other damage to rental properties and must not include capital works or depreciation",
                            Location = "/xbrli:xbrl/rs:Expense.RealEstateProperty.RepairsAndMaintenance.Amount[@contextRef='" + rsChildDocument.RPId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.730325" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "RS53", Value = rsChildDocument.RS53.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.730325
                }
            }

            #endregion // Repeating RS Child Documents 

            // ####################################################################################################
            //                                         WRE CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.310086

            /* VR.ATO.IITR.310086
            The number of Work Related Expense schedules must not exceed 1.

            Legacy Rule Format:
            IF COUNT(SCHEDULE="WRE") > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            */
            assertion = (report.wreChildDocuments != null && Count(report.wreChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Work Related Expense schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[1].RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310086" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310086

            #region WRE Child Present

            if (report.wreChildDocuments != null && Count(report.wreChildDocuments) == 1)
            {
                WRE2017 wreChildDocument = report.wreChildDocuments[0];

                #region VR.ATO.IITR.300059

                /* VR.ATO.IITR.300059
                A schedule of work related expenses is attached and other travel expenses type is 'Award' and allowances, earnings, tips etc is missing or equal to zero in the individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (ANY OCCURRENCE OF([wre25]) = "Award") AND (([iitr65] = NULL) OR ([iitr65] = 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR65            = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount
                WRE:RP.{TrvlSeqNum}:^WRE25 = tns:IncomeTax.Deduction.TravelCategory.Code
                */
                assertion = ((Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 ? wreChildDocument.RPTrvlSeqNumContextCollection.Any(child => child.WRE25 == "Award") : false) && (report.iitrParentDocument.IITR65 == null || report.iitrParentDocument.IITR65.GetValueOrDefault() == 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300059",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Amount of allowances, earnings, tips, director's fees, etc is required",
                        LongDescription = @"When ""Award"" is selected for 'Other travel expenses - Category of expense' on the attached Work related expenses schedule, an amount above zero must be provided for 'Amount of allowances, earnings, tips, director's fees, etc'",
                        Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300059" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE25", Value = "Award" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300059

                #region VR.ATO.IITR.300079

                /* VR.ATO.IITR.300079
                A schedule of work related expenses is attached and other work expenses type is 'Overtime meal allowance' and allowances, earnings, tips, etc is missing or equal to zero in the individual return form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (ANY OCCURRENCE OF([wre50]) = "Overtime meal allowance") AND (([iitr65] = NULL) OR ([iitr65] = 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR65             = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount
                WRE:RP.{OTHWRSeqNum}:^WRE50 = tns:IncomeTax.Deduction.WorkRelatedCategoryOther.Code
                */
                assertion = ((Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 ? wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => context.WRE50 == "Overtime meal allowance") : false) && (report.iitrParentDocument.IITR65 == null || report.iitrParentDocument.IITR65.GetValueOrDefault() == 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300079",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Amount of allowances, earnings, tips, director's fees, etc is required",
                        LongDescription = @"When ""Overtime meal allowance"" is selected for 'Other work expenses - Category of expense' on the attached Work related expenses schedule, an amount above zero must be provided for 'Amount of allowances, earnings, tips, director's fees, etc'",
                        Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300079" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE50", Value = "Overtime meal allowance" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300079

                #region VR.ATO.IITR.310102

                /* VR.ATO.IITR.310102
                Work related car expenses amount is present and a Schedule of Work Related Expenses is attached, however, car expenses comments, or car expenses description, or car expenses amount claimed, or car expenses method used code is missing from the schedule.

                Legacy Rule Format:
                IF ([iitr134] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre12] = NULLORBLANK) OR (COUNT(WRE:RP.{CarSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{CarSeqNum}) = (WRE:RP.{CarSeqNum} WHERE [wre13] = NULLORBLANK OR [wre22] = NULL OR [wre15] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR134          = tns:IncomeTax.Deduction.Car.Amount
                WRE:RP:^WRE12             = tns:IncomeTax.Deduction.CarReason.Description
                WRE:RP.{CarSeqNum}:^WRE13 = tns:Assets.CarMakeModel.Description
                WRE:RP.{CarSeqNum}:^WRE15 = tns:IncomeTax.Deduction.CarCalculationMethod.Code
                WRE:RP.{CarSeqNum}:^WRE22 = tns:IncomeTax.Deduction.Car.Amount
                */
                assertion = (report.iitrParentDocument.IITR134.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE12) || Count(wreChildDocument.RPCarSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310102",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related car expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related car expenses' is provided, the 'Car expenses - Explanation and justification for work purposes', 'Car expenses - Description of car (make, model)', 'Car expenses - Amount' and 'Car expenses - Claim code' work related car expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Car.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310102" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR134", Value = report.iitrParentDocument.IITR134.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE12", Value = wreChildDocument.WRE12 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE13", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)) ? wreChildDocument.RPCarSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)).WRE13 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE15", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)) ? wreChildDocument.RPCarSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)).WRE15 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE22", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)) ? wreChildDocument.RPCarSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)).WRE22.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310102

                #region VR.ATO.IITR.310103

                /* VR.ATO.IITR.310103
                Other work related travel expenses is present and a Schedule of Work Related Expenses is attached, however, other travel expenses comments, or other travel expenses type, or other travel expenses amount claimed, or other travel expenses additional information code is missing from the schedule.

                Legacy Rule Format:
                IF ([iitr136] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre24] = NULLORBLANK) OR (COUNT(WRE:RP.{TrvlSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{TrvlSeqNum}) = (WRE:RP.{TrvlSeqNum} WHERE [wre25] = NULLORBLANK OR [wre29] = NULL OR [wre27] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR136           = tns:IncomeTax.Deduction.Travel.Amount
                WRE:RP:^WRE24              = tns:IncomeTax.Deduction.TravelReason.Description
                WRE:RP.{TrvlSeqNum}:^WRE25 = tns:IncomeTax.Deduction.TravelCategory.Code
                WRE:RP.{TrvlSeqNum}:^WRE27 = tns:IncomeTax.Deduction.TravelStandardEvidenceCalculation.Code
                WRE:RP.{TrvlSeqNum}:^WRE29 = tns:IncomeTax.Deduction.Travel.Amount
                */
                assertion = (report.iitrParentDocument.IITR136.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE24) || Count(wreChildDocument.RPTrvlSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310103",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related other travel expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related travel expenses' is provided, the 'Other travel expenses - Explain how these expenses relate to your work', 'Other travel expenses - Category of expense ', 'Other travel expenses - Amount' and 'Other travel expenses - Standard evidence/calculation code' work related other travel expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Travel.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310103" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR136", Value = report.iitrParentDocument.IITR136.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE24", Value = report.wreChildDocuments[0].WRE24 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE25", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)) ? wreChildDocument.RPTrvlSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)).WRE25 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE27", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)) ? wreChildDocument.RPTrvlSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)).WRE27 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE29", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)) ? wreChildDocument.RPTrvlSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)).WRE29.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310103

                #region VR.ATO.IITR.310104

                /* VR.ATO.IITR.310104
                Work related uniforms/protective clothing/laundry expenses is present and a Schedule of Work Related Expenses is attached, however, uniform/protective clothing/laundry expenses comments, or clothing expenses type, or clothing expenses amount claimed, or clothing expenses additional information code is missing from the schedule.

                Legacy Rule Format:
                IF ([iitr137] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre31] = NULLORBLANK) OR (COUNT(WRE:RP.{ClthngSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{ClthngSeqNum}) = (WRE:RP.{ClthngSeqNum} WHERE [wre32] = NULLORBLANK OR [wre36] = NULL OR [wre34] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR137             = tns:IncomeTax.Deduction.Clothing.Amount
                WRE:RP:^WRE31                = tns:IncomeTax.Deduction.ClothingReason.Description
                WRE:RP.{ClthngSeqNum}:^WRE32 = tns:IncomeTax.Deduction.ClothingCategory.Code
                WRE:RP.{ClthngSeqNum}:^WRE34 = tns:IncomeTax.Deduction.ClothingStandardEvidenceCalculation.Code
                WRE:RP.{ClthngSeqNum}:^WRE36 = tns:IncomeTax.Deduction.Clothing.Amount
                */
                assertion = (report.iitrParentDocument.IITR137.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE31) || Count(wreChildDocument.RPClthngSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310104",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related uniform expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses' is provided, the 'Uniform, protective or occupation specific clothing and laundry expenses - Description of clothing and justification for work purposes', 'Clothing expenses - Category', 'Clothing expenses - Amount' and 'Clothing expenses - Standard evidence/calculation information code' work related self-education expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Clothing.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310104" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR137", Value = report.iitrParentDocument.IITR137.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE31", Value = report.wreChildDocuments[0].WRE31 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE32", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)) ? wreChildDocument.RPClthngSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)).WRE32 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE34", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)) ? wreChildDocument.RPClthngSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)).WRE34 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE36", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)) ? wreChildDocument.RPClthngSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)).WRE36.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310104

                #region VR.ATO.IITR.310105

                /* VR.ATO.IITR.310105
                Work related self-education expenses is present and a Schedule of Work Related Expenses is attached, however, self-education expenses comments, or self-education expenses - name of course, or self-education expenses - institution, or self-education expenses type, or self-education expenses gross amount claimed, or self-education expenses net of expenses amount, or self-education expenses additional information code is missing.

                Legacy Rule Format:
                IF ([iitr139] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre38] = NULLORBLANK) OR ([wre47] = NULL) OR (COUNT(WRE:RP.{EduSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{EduSeqNum}) = (WRE:RP.{EduSeqNum} WHERE ([wre39] = NULLORBLANK OR [wre40] = NULLORBLANK OR [wre41] = NULLORBLANK OR [wre45] = NULL OR [wre43] = NULLORBLANK)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR139          = tns:IncomeTax.Deduction.EducationNet.Amount
                WRE:RP:^WRE38             = tns:IncomeTax.Deduction.EducationReason.Description
                WRE:RP:^WRE47             = tns:IncomeTax.Deduction.EducationNet.Amount
                WRE:RP.{EduSeqNum}:^WRE40 = tns:IncomeTax.Identifiers.InstitutionName.Text
                WRE:RP.{EduSeqNum}:^WRE41 = tns:IncomeTax.Deduction.EducationCategory.Code
                WRE:RP.{EduSeqNum}:^WRE43 = tns:IncomeTax.Deduction.EducationStandardEvidenceCalculation.Code
                WRE:RP.{EduSeqNum}:^WRE45 = tns:IncomeTax.Deduction.Education.Amount
                */
                assertion = (report.iitrParentDocument.IITR139.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE38) || wreChildDocument.WRE47 == null || Count(wreChildDocument.RPEduSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310105",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related self-education expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related self-education expenses' is provided, the 'Self education expenses - Explain how expenses relate to current work activities', 'Self education expenses - Name of course', 'Self education expenses - Institution', 'Self education expenses - Category of expense', 'Self education expenses - Gross amount', 'Self education expenses - Net deduction' and 'Self education expenses - Explanation of other evidence/calculation code used' work related self-education expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationNet.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310105" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE38", Value = report.wreChildDocuments[0].WRE38 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE40", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE40 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE41", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE41 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE43", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE43 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE45", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE45.GetValueOrDefault().ToString() : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE47", Value = report.wreChildDocuments[0].WRE47.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310105

                #region VR.ATO.IITR.600043

                /* VR.ATO.IITR.600043
                Invalid context. Period start date for the Work Related Expenses schedule must be the same as the Period start date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (WRE:RP:period.startDate <> RP:period.startDate)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^StartDate = RP
                WRE:RP:^StartDate  = RP
                */
                assertion = (wreChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600043",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period start date for the Work Related Expenses schedule is invalid",
                        LongDescription = @"Invalid context. Period start date for the Work Related Expenses schedule must be the same as the Period start date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[0].RPId + "']/xbrli:period/xbrli:startDate",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600043" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600043

                #region VR.ATO.IITR.600044

                /* VR.ATO.IITR.600044
                Invalid context. Period end date for the Work Related Expenses schedule must be the same as the Period end date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (WRE:RP:period.endDate <> RP:period.endDate)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^EndDate = RP
                WRE:RP:^EndDate  = RP
                */
                assertion = (wreChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600044",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period end date for the Work Related Expenses schedule is invalid",
                        LongDescription = @"Invalid context. Period end date for the Work Related Expenses schedule must be the same as the Period end date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[0].RPId + "']/xbrli:period/xbrli:endDate",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600044" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600044

                #region VR.ATO.IITR.600045

                /* VR.ATO.IITR.600045
                The TFN provided for the Work Related Expenses schedule should be the same as the TFN of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (WRE:RP:entity.identifier.TFN <> RP:entity.identifier.TFN)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^EntityTFN = RP
                WRE:RP:^EntityTFN  = RP
                */
                assertion = (wreChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600045",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax file number invalid",
                        LongDescription = @"The TFN provided for the Work Related Expenses schedule must be the same as the TFN of the parent form",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[0].RPId + "']/xbrli:entity/xbrli:identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600045" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600045

                #region Repeating RPOTHWRSeqNumContext

                if (wreChildDocument.RPOTHWRSeqNumContextCollection != null)
                {
                    foreach (WRE2017.RPOTHWRSeqNumContext wreOTHWRContext in wreChildDocument.RPOTHWRSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.310106

                        /* VR.ATO.IITR.310106
                        Other work related travel expenses is present and a Schedule of Work Related Expenses is attached and other work expenses type is indicated, however, other work expenses comments, or other work expenses information code is missing from the schedule.

                        Legacy Rule Format:
                        IF ([iitr141] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (COUNT(WRE:RP.{OTHWRSeqNum}) > 0) AND (ANY OCCURRENCE OF([wre50]) <> NULLORBLANK) AND (([wre49] = NULLORBLANK) OR (ANY OCCURRENCE OF(WRE:RP.{OTHWRSeqNum}) = (WRE:RP.{OTHWRSeqNum} WHERE (([wre50] <> NULLORBLANK) AND ([wre55] = NULL OR [wre53] = NULLORBLANK)))))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:
                        IITR:RP:^IITR141            = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                        WRE:RP:^WRE49               = tns:IncomeTax.Deduction.WorkRelatedReasonOther.Description
                        WRE:RP.{OTHWRSeqNum}:^WRE50 = tns:IncomeTax.Deduction.WorkRelatedCategoryOther.Code
                        WRE:RP.{OTHWRSeqNum}:^WRE53 = tns:IncomeTax.Deduction.WorkRelatedStandardEvidenceCalculationOther.Code
                        WRE:RP.{OTHWRSeqNum}:^WRE55 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                        */
                        assertion = (report.iitrParentDocument.IITR141.GetValueOrDefault() > 0 && Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 && !string.IsNullOrEmpty(wreOTHWRContext.WRE50) && (string.IsNullOrEmpty(wreChildDocument.WRE49) || (!string.IsNullOrEmpty(wreOTHWRContext.WRE50) && (wreOTHWRContext.WRE55 == null || string.IsNullOrEmpty(wreOTHWRContext.WRE53)))));
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.310106",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Other work related expense item details on attached Work Related Expenses schedule required",
                                LongDescription = @"When an amount above zero for 'Other work related expenses' is provided, as well as 'Other work expenses - Category of expense' on the attached Work Related Expenses schedule, the other work related expense item details, 'Other work expenses - Explanation and justification for work purposes of each expense', 'Other work expenses - Amount' and 'Other work expenses - Standard evidence/calculation information code', must be provided on the attached Work Related Expenses schedule",
                                Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.WorkRelatedOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310106" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITR141", Value = report.iitrParentDocument.IITR141.GetValueOrDefault().ToString() });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE49", Value = report.wreChildDocuments[0].WRE49 });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE50", Value = (wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))) ? wreChildDocument.RPOTHWRSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))).WRE50 : "null") });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE53", Value = (wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))) ? wreChildDocument.RPOTHWRSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))).WRE53 : "null") });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE55", Value = (wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))) ? wreChildDocument.RPOTHWRSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))).WRE55.GetValueOrDefault().ToString() : "null") });

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.310106
                    }
                }

                #endregion // Repeating RPOTHWRSeqNumContext

                #region Repeating RPTrvlSeqNumContext

                if (wreChildDocument.RPTrvlSeqNumContextCollection != null)
                {
                    foreach (WRE2017.RPTrvlSeqNumContext wreTravelContext in wreChildDocument.RPTrvlSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.300064

                        /* VR.ATO.IITR.300064
                        A schedule of work related expenses is attached and other travel expenses type is 'Domestic', 'Overseas' and other expenses additional information code is 'Allowance' and allowance, earnings, tips is missing or equal to zero in the Individual return.

                        Legacy Rule Format:
                        WHERE IN (WRE:RP.{TrvlSeqNum})
                        IF (COUNT(SCHEDULE = "WRE") = 1) AND ([wre25] = SET("Domestic","Overseas")) AND ([wre27] = "Allowance") AND (([iitr65] = NULL) OR ([iitr65] = 0))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:
                        IITR:RP:^IITR65            = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount
                        WRE:RP.{TrvlSeqNum}:^WRE25 = tns:IncomeTax.Deduction.TravelCategory.Code
                        WRE:RP.{TrvlSeqNum}:^WRE27 = tns:IncomeTax.Deduction.TravelStandardEvidenceCalculation.Code
                        */
                        assertion = (IsMatch(wreTravelContext.WRE25, "^(Domestic|Overseas)$") && wreTravelContext.WRE27 == "Allowance" && (report.iitrParentDocument.IITR65 == null || report.iitrParentDocument.IITR65.GetValueOrDefault() == 0));
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.300064",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Amount of allowances, earnings, tips, director's fees, etc is required",
                                LongDescription = @"When ""Domestic"" or ""Overseas"" is selected for 'Other travel expenses - Category of expense', and ""Allowance"" is selected for 'Other travel expenses - Standard evidence/calculation code', both on the attached Work related expenses schedule, an amount above zero must be provided for 'Amount of allowances, earnings, tips, director's fees, etc'",
                                Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300064" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE25", Value = wreTravelContext.WRE25 });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE27", Value = wreTravelContext.WRE27 });

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.300064
                    }
                }

                #endregion // Repeating RPTrvlSeqNumContext
            }

            #endregion // WRE Child Present

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (!string.IsNullOrEmpty(currentParameter.Value) && currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }
    }
}